// Copyright (C) 2005 CEREA
//     Authors: Lin Wu, Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_OBSERVATION_BASEOBSERVATIONMANAGER_CXX


#include "BaseObservationManager.hxx"


namespace Polyphemus
{


  //! Main constructor.
  /*!
    \param config_file configuration file.
   */
  template <class T>
  BaseObservationManager<T>::BaseObservationManager(string config_file):
    config(config_file), availability(false)
  {
  }


  //! Destructor.
  template <class T>
  BaseObservationManager<T>::~BaseObservationManager()
  {
  }


  ///////////////////
  // CONFIGURATION //
  ///////////////////


  //! Reads the configuration.
  /*! It reads observed species list and observation error parameters.
   */
  template <class T>
  void BaseObservationManager<T>::ReadConfiguration()
  {
    string obs_manager_file;

    this->config.SetSection("[observation_management]");

    this->config.PeekValue("Configuration_file", obs_manager_file);

    /*** Observation configuration ***/

    ConfigStream config_obs(obs_manager_file);

    config_obs.SetSection("[general]");

    config_obs.PeekValue("Error_variance", error_variance);
    config_obs.PeekValue("With_perturbation", with_perturbation);
    if (with_perturbation)
      config_obs.PeekValue("Perturbation_scale", perturbation_scale);

    config_obs.Find("Species");
    obs_species_list = split(config_obs.GetLine());
    Ns_obs = int(obs_species_list.size());
  }


  ////////////////////
  // INITIALIZATION //
  ////////////////////


  //! Initialization for observation manager.
  /*!
    \param Model model with the following interface:
    <ul>
    <li> GetNx()
    <li> GetNy()
    <li> GetX_min()
    <li> GetY_min()
    <li> GetDelta_x()
    <li> GetDelta_y()
    <li> GetStateNz()
    <li> GetStateLevels()
    <li> GetStateNs()
    <li> GetStateSpeciesList()
    <li> GetNstate()
    </ul>
   */
  template <class T>
  template <class ClassModel>
  void BaseObservationManager<T>::Init(ClassModel& Model)
  {
    this->ReadConfiguration();

    model_x_min = Model.GetX_min();
    model_y_min = Model.GetY_min();
    model_Delta_x = Model.GetDelta_x();
    model_Delta_y = Model.GetDelta_y();
    model_Nx = Model.GetNx();
    model_Ny = Model.GetNy();

    Nz_state = Model.GetStateNz();
    state_levels.resize(Nz_state);
    state_levels = Model.GetStateLevels();

    Ns_state = Model.GetStateNs();
    state_species_list = Model.GetStateSpeciesList();

    Nstate = Model.GetNstate();
    Nobs = 0;
  }


  //! Returns the observation number.
  /*!
    \return Observation dimension.
   */
  template <class T>
  int BaseObservationManager<T>::GetNobs() const
  {
    return Nobs;
  }


  //! Checks whether observations are available.
  /*!
    \return True if observations are available, false otherwise.
   */
  template <class T>
  bool BaseObservationManager<T>::IsAvailable() const
  {
    return availability;
  }


  //! Returns the values of observations.
  /*!
    \return The values of observations.
   */
  template <class T>
  const Array<T, 1>& BaseObservationManager<T>::GetObservation() const
  {
    return Observation;
  }


  //! Returns the values of observations.
  /*!
    \return The values of observations.
   */
  template <class T>
  Array<T, 1>& BaseObservationManager<T>::GetObservation()
  {
    return Observation;
  }


  //! Returns the values of observation error covariance.
  /*!
    \return Observation error covariance.
   */
  template <class T>
  const Array<T, 2>& BaseObservationManager<T>::GetCovariance() const
  {
    return error_covariance;
  }


  //! Returns the values of observation error covariance.
  /*!
    \return Observation error covariance.
   */
  template <class T>
  Array<T, 2>& BaseObservationManager<T>::GetCovariance()
  {
    return error_covariance;
  }


  /*! \brief Returns indices (along x) of the closest model-grid-points for
    observations (at current date).
   */
  /*!
    \return The indices (along x) of the closest model-grid-points for
    observations (at current date).
   */
  template <class T>
  Array<int, 1>& BaseObservationManager<T>::GetObservationIndex_x()
  {
    return ObservationIndex_x;
  }


  /*! \brief Returns indices (along y) of the closest model-grid-points for
    observations (at current date).
   */
  /*!
    \return The indices (along y) of the closest model-grid-points for
    observations (at current date).
   */
  template <class T>
  Array<int, 1>& BaseObservationManager<T>::GetObservationIndex_y()
  {
    return ObservationIndex_y;
  }


  /*! \brief Returns indices of species in state array for all observations
    (at current date).
   */
  /*!
    \return The indices of species in state array for all observations
    (at current date)
   */
  template <class T>
  Array<int, 1>& BaseObservationManager<T>::GetObservationStateIndex_s()
  {
    return ObservationStateIndex_s;
  }


  /*! \brief Returns the array of level storage sequence indices in state
    array for all observations (at current date).
   */
  /*!
    \return The array of level storage sequence indices in state array for
    all observations (at current date).
   */
  template <class T>
  Array<int, 1>&
  BaseObservationManager<T>::GetObservationStateSequenceIndex_z()
  {
    return ObservationStateSequenceIndex_z;
  }


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_OBSERVATION_BASEOBSERVATIONMANAGER_CXX
#endif
