// Copyright (C) 2005 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_OUTPUT_SAVER_SAVERUNITDOMAIN_CXX


#include "SaverUnitDomain.hxx"


namespace Polyphemus
{


  /////////////////////
  // SAVERUNITDOMAIN //
  /////////////////////


  //! Main constructor.
  template<class T, class ClassModel>
  SaverUnitDomain<T, ClassModel>
  ::SaverUnitDomain(): BaseSaverUnit<T, ClassModel>()
  {
  }


  //! Destructor.
  template<class T, class ClassModel>
  SaverUnitDomain<T, ClassModel>::~SaverUnitDomain()
  {
  }


  //! Type of saver.
  /*!
    \return The string "domain".
   */
  template<class T, class ClassModel>
  string SaverUnitDomain<T, ClassModel>::GetType()  const
  {
    return "domain";
  }


  //! First initialization.
  /*! Reads the configuration.
    \param config_stream configuration stream.
    \param Model model with the following interface:
    <ul>
    <li> GetSpeciesIndex(string)
    <li> GetConcentration()
    </ul>
   */
  template<class T, class ClassModel>
  void SaverUnitDomain<T, ClassModel>::Init(ConfigStream& config_stream,
					    ClassModel& Model)
  {
    BaseSaverUnit<T, ClassModel>::Init(config_stream, Model);

    // Vertical levels to be saved.
    config_stream.Find("Levels");
    split(config_stream.GetLine(), levels);
    Nlevels = int(levels.size());
    
    // Species to be saved.
    this->Ns = int(this->species_list.size());
    for (int s = 0; s < this->Ns; s++)
      this->species_index.push_back(Model.
				    GetSpeciesIndex(this->species_list[s]));

    // Output filename.
    string filename = config_stream.GetValue("Output_file");
    // Output filenames for all species.
    output_file.resize(this->Ns);
    for (unsigned int i = 0; i < this->species_list.size(); i++)
      output_file[i] = find_replace(filename, "&f", this->species_list[i]);

    // Empties output files.
    for (unsigned int s = 0; s < this->species_list.size(); s++)
      ofstream tmp_stream(output_file[s].c_str());

    if (this->averaged)
      {
	int s, k, j, i;
	Concentration_.Resize(this->Ns, Nlevels,
			      this->base_Ny, this->base_Nx);
	for (s = 0; s < this->Ns; s++)
	  for (k = 0; k < Nlevels; k++)
	    for (j = 0; j < this->base_Ny; j++)
	      for (i = 0; i < this->base_Nx; i++)
		Concentration_(s, k, j, i) = 0.5
		  * Model.GetConcentration()(this->species_index[s],
					     levels[k], j, i);
      }

    if (this->initial_concentration && !this->averaged)
      this->Save(Model);
  }


  //! Initializes the saver at the beginning of each step.
  /*!
    \param Model model (dummy argument).
   */
  template<class T, class ClassModel>
  void SaverUnitDomain<T, ClassModel>::InitStep(ClassModel& Model)
  {
    BaseSaverUnit<T, ClassModel>::InitStep(Model);
  }


  //! Saves concentrations if needed.
  /*!
    \param Model model with the following interface:
    <ul>
    <li> GetConcentration()
    <li> GetCurrentDate()
    </ul>
   */
  template<class T, class ClassModel>
  void SaverUnitDomain<T, ClassModel>::Save(ClassModel& Model)
  {
    if (this->averaged)
      if (this->counter % this->interval_length == 0)
	{
	  int s, k, j, i;
	  for (s = 0; s < this->Ns; s++)
	    for (k = 0; k < Nlevels; k++)
	      for (j = 0; j < this->base_Ny; j++)
		for (i = 0; i < this->base_Nx; i++)
		  Concentration_(s, k, j, i) += 0.5
		    * Model.GetConcentration()(this->species_index[s],
					       levels[k], j, i);

	  Concentration_.GetArray() /= T(this->interval_length);

	  if (Model.GetCurrentDate() >= this->date_beg
	      && Model.GetCurrentDate() <= this->date_end)
	    for (s = 0; s < this->Ns; s++)
	      {
		Data<T, 3>
		  Concentration_tmp(&Concentration_(s, 0, 0, 0),
				    shape(Nlevels, this->base_Ny,
					  this->base_Nx));
		FormatBinary<float>().Append(Concentration_tmp,
					     output_file[s]);
	      }

	  for (s = 0; s < this->Ns; s++)
	    for (k = 0; k < Nlevels; k++)
	      for (j = 0; j < this->base_Ny; j++)
		for (i = 0; i < this->base_Nx; i++)
		  Concentration_(s, k, j, i) = 0.5
		    * Model.GetConcentration()(this->species_index[s],
					       levels[k], j, i);

	  this->counter = 0;
	}
      else
	{
	  int s, k, j, i;
	  for (s = 0; s < this->Ns; s++)
	    for (k = 0; k < Nlevels; k++)
	      for (j = 0; j < this->base_Ny; j++)
		for (i = 0; i < this->base_Nx; i++)
		  Concentration_(s, k, j, i) +=
		    Model.GetConcentration()(this->species_index[s],
					     levels[k], j, i);
	}
    else if (this->counter % this->interval_length == 0
	     &&	Model.GetCurrentDate() >= this->date_beg
	     && Model.GetCurrentDate() <= this->date_end)
      // Instantaneous concentrations.
      for (int s = 0; s < this->Ns; s++)
	for (int k = 0; k < Nlevels; k++)
	  {
	    Data<T, 2>
	      Concentration_tmp(&Model.GetConcentration()
				(this->species_index[s], levels[k], 0, 0),
				shape(this->base_Ny, this->base_Nx));
	    FormatBinary<float>().Append(Concentration_tmp, output_file[s]);
	  }
  }


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_OUTPUT_SAVER_SAVERUNITDOMAIN_CXX
#endif
