// Copyright (C) 2005 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODELS_BASEMODEL_HXX


#include "InputFiles.cxx"
#include <vector>
#include <map>
#include <cmath>
#include "AtmoData.hxx"


namespace Polyphemus
{


  using namespace std;
  using namespace AtmoData;


  ///////////////
  // BASEMODEL //
  ///////////////


  /*! \brief 'BaseModel' collects the attributes and the methods that are
    shared by most models. */
  /*! 'BaseModels' mainly collects the grids, several option keys, and methods
    to read data on file and to interpolate them in time.
   */
  template<class T>
  class BaseModel
  {

  public:

    /*** Type declarations ***/

    typedef typename map<string, InputFiles<T> >
    ::iterator input_files_iterator;

  protected:

    /*** Configuration ***/

    //! Name of the main configuration file.
    string file_config;
    //! Configuration stream for the main configuration file.
    ConfigStream config;

    //! Options to enable or disable physical processes.
    map<string, bool> option_process;
    /*! \brief Variables that are managed (that is, read or computed) by the
      model or not. */
    map<string, bool> option_manage;

    //! List of input-files groups.
    map<string, InputFiles<T> > input_files;

    /*** Domain ***/

    //! Simulation starting date.
    Date Date_min;
    //! Time step in seconds.
    T Delta_t;
    //! Number of time steps.
    int Nt;

    //! File storing species list.
    string file_species;
    //! Species list.
    vector<string> species_list;
    //! Number of species.
    int Ns;
    //! List of aerosol species.
    vector<string> species_list_aer;
    //! Number of aerosol species.
    int Ns_aer;
    //! Number of bins (for aerosols).
    int Nbin_aer;

    //! File storing vertical-levels coordinates.
    string file_vertical_levels;
    //! Number of vertical layers.
    int Nz;
    //! Vertical levels.
    Array<T, 1> LayerInterface;
    //! Domain origin along y.
    T y_min;
    //! Space step along y.
    T Delta_y;
    //! Number of points along y.
    int Ny;
    //! Domain origin along x.
    T x_min;
    //! Space step along x.
    T Delta_x;
    //! Number of points along x.
    int Nx;

    //! 3D grid for species.
    RegularGrid<T> GridS3D;
    //! 4D grid for species.
    RegularGrid<T> GridS4D;

    //! 3D grid along z.
    RegularGrid<T> GridZ3D;
    //! 3D grid for interfaces along z.
    RegularGrid<T> GridZ3D_interf;
    //! 4D grid along z.
    RegularGrid<T> GridZ4D;
    //! 4D grid for interfaces along z.
    RegularGrid<T> GridZ4D_interf;

    //! 2D grid along y.
    RegularGrid<T> GridY2D;
    //! 3D grid along y.
    RegularGrid<T> GridY3D;
    //! 3D grid for interfaces along y.
    RegularGrid<T> GridY3D_interf;
    //! 4D grid along y.
    RegularGrid<T> GridY4D;
    //! 4D grid for interfaces along y.
    RegularGrid<T> GridY4D_interf;

    //! 2D grid along x.
    RegularGrid<T> GridX2D;
    //! 3D grid along x.
    RegularGrid<T> GridX3D;
    //! 3D grid for interfaces along x.
    RegularGrid<T> GridX3D_interf;
    //! 4D grid along x.
    RegularGrid<T> GridX4D;
    //! 4D grid for interfaces along x.
    RegularGrid<T> GridX4D_interf;

    /*** Integration ***/

    //! Date of the previous time-step.
    Date previous_date;
    //! Date of the current time-step.
    Date current_date;
    //! Date of the next time-step.
    Date next_date;
    //! Number of seconds between the simulation beginning and the current
    //! date.
    T current_time;
    //! Number of steps achieved so far.
    int step;

    /*** Fields ***/

    //! Map from 2D field names to their arrays.
    map<string, Array<T, 2>* > A2_map;
    //! Map from 2D field names to their data.
    map<string, Data<T, 2>* > D2_map;
    //! Map from 3D field names to their arrays.
    map<string, Array<T, 3>* > A3_map;
    //! Map from 3D field names to their data.
    map<string, Data<T, 3>* > D3_map;
    //! Map from 4D field names to their arrays.
    map<string, Array<T, 4>* > A4_map;
    //! Map from 4D field names to their data.
    map<string, Data<T, 4>* > D4_map;
    //! Map from 5D field names to their arrays.
    map<string, Array<T, 5>* > A5_map;
    //! Map from 5D field names to their data.
    map<string, Data<T, 5>* > D5_map;

    //! Map from field names to species lists.
    map<string, vector<string>* > field_species;

    /*** State ***/

    //! Concentrations.
    Data<T, 4> Concentration;
    //! Aerosols concentrations.
    Data<T, 5> Concentration_aer;

  public:

    /*** Constructor and destructor ***/

    BaseModel(string config_file);
    virtual ~BaseModel();

    /*** Configuration ***/

    virtual void ReadConfiguration();
    virtual void CheckConfiguration();

    /*** Initializations ***/

    virtual void Allocate();
    virtual void Init();
    virtual void InitStep();

    virtual void SetDate(Date date);
    virtual void StepBack();
    virtual void StepBack(const Array<T, 4>& concentration);

    /*** Integration ***/

    virtual void Forward();

    /*** Access methods ***/

    string GetConfigurationFile() const;

    T GetDelta_t() const;
    int GetNt() const;

    string GetSpeciesFile() const;
    int GetNs() const;
    int GetNumSpecies() const;
    int GetNs_aer() const;
    int GetNumSpecies_aer() const;
    int GetNbin_aer() const;

    int GetNz() const;
    T GetY_min() const;
    T GetDelta_y() const;
    int GetNy() const;
    T GetX_min() const;
    T GetDelta_x() const;
    int GetNx() const;

    vector<string> GetSpeciesList() const;
    string GetSpeciesName(int i) const;
    bool IsSpecies(string name) const;
    int GetSpeciesIndex(string species) const;
    int GetSpeciesIndex(string species,
			const vector<string>& ref_species_list) const;
    int GetSpeciesIndex(string field, string species);

    vector<string> GetSpeciesList_aer() const;
    string GetSpeciesName_aer(int i) const;
    bool IsSpecies_aer(string name) const;
    int GetSpeciesIndex_aer(string species) const;

    const vector<string>& GetSpeciesList(string field);

    Array<T, 1>& GetGridZArray1D();
    Array<T, 1>& GetGridYArray1D();
    Array<T, 1>& GetGridXArray1D();
    Array<T, 1>& GetLayerInterface();

    Date GetPreviousDate() const;
    Date GetCurrentDate() const;
    T GetCurrentTime() const;
    Date GetNextDate() const;

    bool HasField(string field);
    string FindField(string field);
    int GetFieldDimension(string field);
    Array<T, 2>& A2(string field);
    Data<T, 2>& D2(string field);
    Array<T, 3>& A3(string field);
    Data<T, 3>& D3(string field);
    Array<T, 4>& A4(string field);
    Data<T, 4>& D4(string field);
    Array<T, 5>& A5(string field);
    Data<T, 5>& D5(string field);

    const Data<T, 4>& GetConcentration() const;
    Data<T, 4>& GetConcentration();
    const Data<T, 5>& GetConcentration_aer() const;
    Data<T, 5>& GetConcentration_aer();

    bool IsManaged(string field) const;
    void Manage(string field, bool status = true);

  protected:

    void AddTime(T time);
    void SubtractTime(T time);

    template<int N>
    void InitData(string section, string field,
		  Data<T, N>& FileData_i, Data<T, N>& FileData_f,
		  Date date, Data<T, N>& CurrentData,
		  bool interpolated = true);
    template<int N>
    void InitData(string section, string field,
		  Data<T, N>& FileData_i, Data<T, N>& FileData_f,
		  Date date, int index, Data<T, N>& CurrentData,
		  bool interpolated = true);
    template<int N>
    void InitData(string section, string field,
		  Data<T, N>& FileData_i, Data<T, N>& FileData_f,
		  Date date, int first_index, int second_index,
		  Data<T, N>& CurrentData);
    template<int N>
    void InitData(string input_file, Date date_min_file, T Delta_t_file,
		  Data<T, N>& FileData_i, Data<T, N>& FileData_f,
		  Date date, Data<T, N>& CurrentData,
		  bool interpolated = true);
    template<int N>
    void InitData(string input_file, Date date_min_file, T Delta_t_file,
		  Data<T, N>& FileData_i, Data<T, N>& FileData_f,
		  Date date, int index, Data<T, N>& CurrentData,
		  bool interpolated = true);
    template<int N>
    void InitData(string input_file, Date date_min_file, T Delta_t_file,
		  Data<T, N>& FileData_i, Data<T, N>& FileData_f,
		  Date date, int first_index, int second_index,
		  Data<T, N>& CurrentData);

    template<int N>
    void UpdateData(string section, string field, Data<T, N>& FileData_i,
		    Data<T, N>& FileData_f, Data<T, N>& CurrentData,
		    bool interpolated = true);
    template<int N>
    void UpdateData(string section, string field, Data<T, N>& FileData_i,
		    Data<T, N>& FileData_f, int index,
		    Data<T, N>& CurrentData, bool interpolated = true);
    template<int N>
    void UpdateData(string section, string field, Data<T, N>& FileData_i,
		    Data<T, N>& FileData_f, int first_index, int second_index,
		    Data<T, N>& CurrentData);
    template<int N>
    void UpdateData(string input_file, Date date_min_file,
		    T Delta_t_file, Data<T, N>& FileData_i,
		    Data<T, N>& FileData_f, Data<T, N>& CurrentData,
		    bool interpolated = true);
    template<int N>
    void UpdateData(string input_file, Date date_min_file,
		    T Delta_t_file, Data<T, N>& FileData_i,
		    Data<T, N>& FileData_f, int index,
		    Data<T, N>& CurrentData, bool interpolated = true);
    template<int N>
    void UpdateData(string input_file, Date date_min_file,
		    T Delta_t_file, Data<T, N>& FileData_i,
		    Data<T, N>& FileData_f, int first_index, int second_index,
		    Data<T, N>& CurrentData);

    template<int N>
    void UpdateData(string section, string field, Data<T, N>& FileData_i,
		    Data<T, N>& FileData_f, Data<T, N>& CurrentData_i,
		    Data<T, N>& CurrentData_f, bool interpolated = true);
    template<int N>
    void UpdateData(string section, string field, Data<T, N>& FileData_i,
		    Data<T, N>& FileData_f, int index,
		    Data<T, N>& CurrentData_i, Data<T, N>& CurrentData_f,
		    bool interpolated = true);
    template<int N>
    void UpdateData(string section, string field, Data<T, N>& FileData_i,
		    Data<T, N>& FileData_f, int first_index, int second_index,
		    Data<T, N>& CurrentData_i, Data<T, N>& CurrentData_f);
    template<int N>
    void UpdateData(string input_file, Date date_min_file,
		    T Delta_t_file, Data<T, N>& FileData_i,
		    Data<T, N>& FileData_f, Data<T, N>& CurrentData_i,
		    Data<T, N>& CurrentData_f, bool interpolated = true);
    template<int N>
    void UpdateData(string input_file, Date date_min_file,
		    T Delta_t_file, Data<T, N>& FileData_i,
		    Data<T, N>& FileData_f, int index,
		    Data<T, N>& CurrentData_i, Data<T, N>& CurrentData_f,
		    bool interpolated = true);
    template<int N>
    void UpdateData(string input_file, Date date_min_file,
		    T Delta_t_file, Data<T, N>& FileData_i,
		    Data<T, N>& FileData_f, int first_index, int second_index,
		    Data<T, N>& CurrentData_i, Data<T, N>& CurrentData_f);

    void Interpolate(Date date_min_file, T Delta_t_file, int record,
		     const Data<T, 3>& FileData_i,
		     const Data<T, 3>& FileData_f,
		     Date date, Data<T, 3>& InterpolatedData);
    void Interpolate(Date date_min_file, T Delta_t_file, int record,
		     const Data<T, 2>& FileData_i,
		     const Data<T, 2>& FileData_f,
		     Date date, Data<T, 2>& InterpolatedData);

  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_BASEMODEL_HXX
#endif
