// Copyright (C) 2006 CEREA
//     Authors: Hadjira Foudhil, Irne Korsakissok
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the modeling system Polyphemus.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODELS_BRIGGSFORMULA_HXX


namespace Polyphemus
{


  //! Estimates an effective source height of a hot source.
  /*! It uses the Briggs' plume rise formulation.
    \param temperature temperature of ambient air (K).
    \param wind wind velocity (m/s).
    \param inversion_height inversion height (m).
    \param stability stability in [0, 5].
    \param source_velocity efflux speed of gases (m/s).
    \param source_temperature temperature of emitted gases (K).
    \param diameter source diameter (m).
    \param source_height source height (m).
    \return The effective source height (m).
  */
  template<class T>
  T ComputeEffectiveSourceHeight(T temperature, T wind, T inversion_height,
				 int stability, T source_velocity,
				 T source_temperature, T diameter,
				 T source_height)
  {
    const T g = 9.81;

    T dTdz;

    // Vertical gradient of the potential temperature.
    if (stability == 0)
      dTdz = -0.015;
    else if (stability == 1)
      dTdz = -0.008;
    else if (stability == 2)
      dTdz = -0.006;
    else if (stability == 3)
      dTdz = 0.;
    else if (stability == 4)
      dTdz = 0.02;
    else if (stability == 5)
      dTdz = 0.035;
    else
      throw string("Stability index should be in [0, 5], but ")
	+ to_str(stability) + " was provided.";

    T sp_briggs;
    T Fm;
    T effective_height = 0.;
    T Dhz = 0.;
  

    // Briggs static stability parameter.
    sp_briggs = (g * dTdz)/temperature ;
  
    // Initial momentum flux parameter.
    Fm = g * source_velocity * 0.25 * diameter * diameter
      * abs(source_temperature - temperature) / source_temperature;
  
    if (dTdz > 0.)
      // Stable.
      Dhz = 2.6 * pow(Fm / (wind * sp_briggs), 1./3.);
    else
      // Unstable or neutral.
      Dhz = 38.71 / wind * pow(Fm, 3./5.);
  
    // Effective source height.
    effective_height = Dhz + source_height;

    if (inversion_height <= 0.)
      return effective_height;
    else
      return min(effective_height, inversion_height);
  }


  //! Computes horizontal diffusion parameter over rural area.
  /*! It uses Briggs' dispersion parameterizations for open country with
    Pasquill stability classes.
    \param distance distance downwind of the source (m).
    \param stability Pasquill stability classes in [0, 5].
    \return The horizontal plume-dispersion parameter (m).
  */
  template<class T>
  T ComputeRuralPlumeHorizontalSigma(T distance, int stability)
  {
    if (stability == 0)
      return 0.22 * distance / sqrt(1. + 1.e-4 * distance);
    else if (stability == 1)
      return 0.16 * distance / sqrt(1. + 1.e-4 * distance);
    else if (stability == 2)
      return 0.11 * distance / sqrt(1. + 1.e-4 * distance);
    else if (stability == 3)
      return 0.08 * distance / sqrt(1. + 1.e-4 * distance);
    else if (stability == 4)
      return 0.06 * distance / sqrt(1. + 1.e-4 * distance);
    else if (stability == 5)
      return 0.04 * distance / sqrt(1. + 1.e-4 * distance);
    else
      throw string("Stability index should be in [0, 5], but ")
	+ to_str(stability) + " was provided.";
  }


  //! Computes vertical diffusion parameter over rural area.
  /*! It uses Briggs' dispersion parameterizations for open country with
    Pasquill stability classes.
    \param distance distance downwind of the source (m).
    \param stability Pasquill stability classes in [0, 5].
    \return The vertical plume-dispersion parameter (m).
  */
  template<class T>
  T ComputeRuralPlumeVerticalSigma(T distance, int stability)
  {
    if (stability == 0)
      return 0.2 * distance;
    else if (stability == 1)
      return 0.12 * distance;
    else if (stability == 2)
      return 0.08 * distance / sqrt(1. + 2.e-4 * distance);
    else if (stability == 3)
      return 0.06 * distance / sqrt(1. + 1.5e-3 * distance);
    else if (stability == 4)
      return 0.03 * distance / (1. + 3.e-4 * distance);
    else if (stability == 5)
      return 0.016 * distance / (1. + 3.e-4 * distance);
    else
      throw string("Stability index should be in [0, 5], but ")
	+ to_str(stability) + " was provided.";
  }


  //! Computes derivative of vertical diffusion parameter over rural area.
  /*! It uses Briggs' dispersion parameterizations for open country with
    Pasquill stability classes.
    \param distance distance downwind of the source (m).
    \param stability Pasquill stability classes in [0, 5].
    \return The derivative of vertical plume-dispersion parameter
    with respect to the distance downwind of the source.
  */
  template<class T>
  T DifferentiateRuralPlumeVerticalSigma(T distance, int stability)
  {
    if (stability == 0)
      return 0.2;
    else if (stability == 1)
      return 0.12;
    else if (stability == 2)
      {
	T diff_factor = sqrt(1. + 2.e-4 * distance) * (1. + 2.e-4 * distance);
	return 0.08 * (1. + 1.e-4 * distance) / diff_factor;
      }
    else if (stability == 3)
      {
	T diff_factor = sqrt(1. + 1.5e-3 * distance) *
	  (1. + 1.5e-3 * distance);
	return 0.03 * (2. + 1.5e-3 * distance) / diff_factor;
      }
    else if (stability == 4)
      {
	T diff_factor = (1. + 3.e-4 * distance) * (1. + 3.e-4 * distance);
	return 0.03 * (1. + 6.e-3 *  distance) / diff_factor;
      }
    else if (stability == 5)
      {
	T diff_factor = (1. + 3.e-4 * distance) * (1. + 3.e-4 * distance);
	return 0.016 * (1. + 6.e-3 *  distance) / diff_factor;
      }
    else
      throw string("Stability index should be in [0, 5], but ")
	+ to_str(stability) + " was provided.";
  }


  //! Computes horizontal diffusion parameter over urban area.
  /*! It uses Briggs' dispersion parameterizations for open country with
    Pasquill stability classes.
    \param distance distance downwind of the source (m).
    \param stability Pasquill stability classes in [0, 5].
    \return The horizontal plume-dispersion parameter (m).
  */
  template<class T>
  T ComputeUrbanPlumeHorizontalSigma(T distance, int stability)
  {
    if (stability == 0)
      return 0.32 * distance / sqrt(1. + 4.e-4 * distance);
    else if (stability == 1)
      return 0.32 * distance / sqrt(1. + 4.e-4 * distance);
    else if (stability == 2)
      return 0.22 * distance / sqrt(1. + 4.e-4 * distance);
    else if (stability == 3)
      return 0.16 * distance / sqrt(1. + 4.e-4 * distance);
    else if (stability == 4)
      return 0.11 * distance / sqrt(1. + 4.e-4 * distance);
    else if (stability == 5)
      return 0.11 * distance / sqrt(1. + 4.e-4 * distance);
    else
      throw string("Stability index should be in [0, 5], but ")
	+ to_str(stability) + " was provided.";
  }


  //! Computes vertical diffusion parameter over urban area.
  /*! It uses Briggs' dispersion parameterizations for open country with
    Pasquill stability classes.
    \param distance distance downwind of the source (m).
    \param stability Pasquill stability classes in [0, 5].
    \return The vertical plume-dispersion parameter (m).
  */
  template<class T>
  T ComputeUrbanPlumeVerticalSigma(T distance, int stability)
  {
    if (stability == 0)
      return 0.24 * distance * sqrt(1. + 1.e-3 * distance);
    else if (stability == 1)
      return 0.24 * distance * sqrt(1. + 1.e-3 * distance);
    else if (stability == 2)
      return 0.2 * distance;
    else if (stability == 3)
      return 0.14 * distance / sqrt(1. + 3.e-4 * distance);
    else if (stability == 4)
      return 0.08 * distance /sqrt(1. + 1.5e-3 * distance);
    else if (stability == 5)
      return 0.08 * distance /sqrt(1. + 1.5e-3 * distance);
    else
      throw string("Stability index should be in [0, 5], but ")
	+ to_str(stability) + " was provided.";
  }


  //! Computes derivative of vertical diffusion parameter over urban area.
  /*! It uses Briggs' dispersion parameterizations for open country with
    Pasquill stability classes.
    \param distance distance downwind of the source (m).
    \param stability Pasquill stability classes in [0, 5].
    \return The derivative of vertical plume-dispersion parameter
    with respect to the distance downwind of the source.
  */
  template<class T>
  T DifferentiateUrbanPlumeVerticalSigma(T distance, int stability)
  {
    if (stability == 0 || stability == 1)
      return 0.12 * (2. + 3.e-3 * distance) / sqrt(1. + 1.e-3 * distance);
    else if (stability == 2)
      return 0.20;
    else if (stability == 3)
      {
	T diff_factor = sqrt(1. + 3e-4 * distance) * (1. + 3.e-4 * distance);
	return 0.07 * (2. + 3.e-4 * distance) / diff_factor;
      }
    else if (stability == 4 || stability == 5)
      {
	T diff_factor = sqrt(1. + 1.5e-3 * distance) *
	  (1. + 1.5e-3 * distance);
	return 0.04 * (2. + 1.5e-3 *  distance) / diff_factor;
      }
    else
      throw string("Stability index should be in [0, 5], but ")
	+ to_str(stability) + " was provided.";
  }


  //! Computes a concentration at a given point.
  /*! Gaussian plume model for a continuous point source in uniform flow with
    homogeneous turbulence.
    \param wind mean transport velocity across the plume (m/s).
    \param inversion_height inversion height (m).
    \param effective_height effective source height (m).
    \param rate source strength or emission rate (mass unit/s).
    \param y ordinate (crosswind) (m).
    \param z height (m).
    \param sigma_y horizontal diffusion parameter (m).
    \param sigma_z vertical diffusion parameter (m).
    \param alpha coefficient for Overcamp model (in [-1, 1])..
    \param minimum_volume (optional) minimum volume in which the tracer is
    mixed within one second (m^3/s). It is used to set an upper bound on the
    concentration. Put zero to avoid this threshold. Default: 1.e-6.
    \return The concentration at (\a y, \a z).
    \warning The wind should not be zero.
  */
  template<class T>
  T ComputePlumeConcentration(T wind, T inversion_height, T effective_height,
			      T rate, T y, T z, T sigma_y, T sigma_z, T alpha,
			      T minimum_volume = 1.e-6)
  {
    const T pi = 3.14159265358979323846264;

    // Crosswind contribution.
    T Fy = 0.;
    Fy = exp(-y*y  / (2. * sigma_y*sigma_y));
  
    // Vertical contribution (real and image sources respectively).
    T Fz = 0.;
    T dzr = z - effective_height;
    T dzi = z + effective_height;

    T sigma_z_2 = sigma_z*sigma_z;
  
    Fz = exp(-dzr*dzr / (2. * sigma_z_2))
      +  alpha * exp(-dzi*dzi / (2. * sigma_z_2));

    // Reflexion if inversion (real and image sources respectively).
    T FzI = 0.;

    if (inversion_height > 0.)
      for (int N = 1; N < 6; N++)
	{
	  T dzmi = z - effective_height + 2. * N * inversion_height;
	  T dzpi = z + effective_height - 2. * N * inversion_height;
	  
	  FzI += exp(-dzmi*dzmi / (2. * sigma_z_2))
	    + alpha * exp(-dzpi*dzpi / (2. * sigma_z_2));
	}

    T concentration
      = rate * Fy * (Fz + FzI) / (2. * pi * wind * sigma_y * sigma_z);

    return minimum_volume != 0. ? min(concentration, rate / minimum_volume)
      : concentration;
  }


}


#define POLYPHEMUS_FILE_MODELS_BRIGGSFORMULA_HXX
#endif
