// Copyright (C) 2006 CEREA
//     Authors: Hadjira Foudhil, Vivien Mallet, Irne Korsakissok
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the modeling system Polyphemus.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODELS_GAUSSIANPLUME_CXX


#include "GaussianPlume.hxx"
#include "BriggsFormula.hxx"


namespace Polyphemus
{


  //! Main constructor.
  /*!
    \param config_file configuration file.
  */
  template<class T>
  GaussianPlume<T>::GaussianPlume(string config_file):
    BaseModel<T>(config_file), Nsource(0)
  {
  }


  //! Sources initialization.
  /*! It sets all sources from a text file. Each source is described in a
    dedicated section "[source]" in which one finds the following entries:
    <ul>
    <li> Rate: the rate (mass unit per second),
    <li> Velocity: the efflux speed (m/s),
    <li> Temperature: the temperature of emissions (Celsius degrees),
    <li> Section: source cross-sectional area (m^2),
    <li> Abscissa: abscissa (m),
    <li> Ordinate: ordinate (m),
    <li> Altitude: height (m),
    <li> Species_name: species name.
    </ul>
    \param source_file file that describes the sources.
    \note The cross-sectional surface of the source is assumed to be a disc.
  */
  template<class T>
  void GaussianPlume<T>::InitSource(string source_file)
  {
    T rate, velocity, temperature, section, diameter, x, y, z;
    string line, species_name;

    const T pi = 3.14159265358979323846264;

    ConfigStream source_stream(source_file);

    SourceList.clear();
    Nsource = 0;
    while (!source_stream.IsEmpty())
      {
	line = source_stream.GetLine();
	if (split(line)[0] == "[source]")
	  {
	    source_stream.PeekValue("Abscissa", x);
	    source_stream.PeekValue("Ordinate", y);
	    source_stream.PeekValue("Altitude", z);
	    source_stream.PeekValue("Rate", rate);
	    source_stream.PeekValue("Velocity", velocity);
	    source_stream.PeekValue("Temperature", temperature);
	    source_stream.PeekValue("Section", section);
	    source_stream.PeekValue("Species_name", species_name);
	    temperature += 273.15;
	    diameter = sqrt(4. * section / pi);

	    SourceList.push_back
	      (PlumeSource<T>(rate, velocity, temperature, diameter, x,
			      y, z, this->GetSpeciesIndex(species_name)));
	    
	    Nsource++;

	  }
      }
  }


  //! Model initialization.
  /*! It reads the configuration.
   */
  template<class T>
  void GaussianPlume<T>::Init()
  {

    /*** Configuration ***/

    BaseModel<T>::Init();

    this->config.SetSection("[domain]");

    // Land category ("rural" or "urban").
    this->config.PeekValue("Land_category", land_category);
    if (land_category != "rural" && land_category != "urban")
      throw string("Land category should be \"rural\" or \"urban\", but \"")
	+ land_category + "\" was provided in section \"[domain]\".";

    // Time ("night" or "day").
    this->config.PeekValue("Time", day_night);
    if (day_night != "day" && day_night != "night")
      throw string("Time should be \"day\" or \"night\", but \"")
	+ day_night + "\" was provided in section \"[domain]\".";

    // Gaussian options.
    this->config.SetSection("[gaussian]");
    this->config.PeekValue("With_plume_rise",
			   this->option_process["with_plume_rise"]);
    this->config.PeekValue("With_radioactive_decay",
			   this->option_process["with_radioactive_decay"]);
    this->config.PeekValue("With_biological_decay",
			   this->option_process["with_biological_decay"]);
    this->config.PeekValue("With_scavenging",
			   this->option_process["with_scavenging"]);
    this->config.PeekValue("With_dry_deposition",
			   this->option_process["with_deposition"]);
    this->config.PeekValue("File_source", file_source);

    ConfigStream species_stream(this->file_species);

    // Reads the species half-life in the species file.
    if (this->option_process["with_radioactive_decay"])
      {
	half_life_time.resize(this->Ns);
	species_stream.SetSection("[radioactive_decay]");
	for (int i = 0; i < this->Ns; i++)
	  species_stream.PeekValue(this->species_list[i], half_life_time(i));

	// Conversion from days to seconds.
	T factor = 24. * 3600.;
	for (int i = 0; i < this->Ns; i++)
	  half_life_time(i) *= factor;
      }

    // Reads the species biological half-life in the species file.
    if (this->option_process["with_biological_decay"])
      {
	T day_value, night_value;
	biological_half_life_time.resize(this->Ns);
	for (int i = 0; i < this->Ns; i++)
	  {
	    species_stream.SetSection("[biological_decay]");
	    species_stream.Find(this->species_list[i]);
	    species_stream.GetNumber(day_value);
	    species_stream.GetNumber(night_value);
	    if (day_night == "day")
	      biological_half_life_time(i) = day_value;
	    else
	      biological_half_life_time(i) = night_value;
	  }
      }

    // Gets the deposition model.
    if (this->option_process["with_deposition"])
      {
	this->config.SetSection("[deposition]");
	this->config.PeekValue("Deposition_model", deposition_model);

	if (deposition_model == "Chamberlain")
	  this->config.PeekValue("Nchamberlain", Nchamberlain);
	else if (deposition_model != "Overcamp")
	  throw string("Deposition model should be \"Chamberlain\" or")
	    + " \"Overcamp\" but \"" + deposition_model + "\" was provided "
	    + "in section \"[deposition]\".";
      }
    
    /*** Initializes the sources ***/

    InitSource(file_source);
  }


  //! Initializes meteorological conditions.
  /*! It sets the meteorological data from a configuration file.  The
    situation is described in a dedicated section "[situation]" in which one
    finds the following entries:
    <ul>
    <li> Temperature: the temperature of ambient air (Celsius degrees).
    <li> Wind_angle: the angle between positive x-axis and wind,
    counterclockwise (degrees).
    <li> Wind: the wind velocity (m/s).
    <li> Inversion_height: the inversion height (m).
    <li> Stability: stability class in [A, F].
    </ul>
    \param meteo ConfigStream instance through which all entries (Temperature,
    Wind_angle, ...) may be read to set the meteorological situation.
    \param show_meteo indicates whether the meteorological data is to be
    displayed on screen.
  */
  template<class T>
  void GaussianPlume<T>::InitMeteo(ConfigStream& meteo, bool show_meteo)
  {
    meteo.PeekValue("Temperature", temperature_);
    meteo.PeekValue("Wind_angle", wind_angle_);
    meteo.PeekValue("Wind", wind_);
    meteo.PeekValue("Inversion_height", inversion_height_);
    meteo.PeekValue("Stability", stability_class_);

    if (show_meteo)
      cout << "\t" << temperature_
	   << "\t\t" << wind_angle_
	   << "\t\t" << wind_
	   << "\t\t" << stability_class_ << endl;

    const T pi = 3.14159265358979323846264;

    temperature_ = temperature_ + 273.15;
    T wind_angle_rad = wind_angle_ / 180. * pi;
    cos_angle_ = cos(wind_angle_rad);
    sin_angle_ = sin(wind_angle_rad);

    stability_class_ = upper_case(stability_class_);
    if (stability_class_ == "A")
      stability_ = 0;
    else if (stability_class_ == "B")
      stability_ = 1;
    else if (stability_class_ == "C")
      stability_ = 2;
    else if (stability_class_ == "D")
      stability_ = 3;
    else if (stability_class_ == "E")
      stability_ = 4;
    else if (stability_class_ == "F")
      stability_ = 5;
    else
      throw string("Stability class should be in [A, F], but \"")
	+ stability_class_ + "\" was provided.";

    // Gets the scavenging coefficients for all species.
    if (this->option_process["with_scavenging"])
      {
	scavenging_coefficient.resize(this->Ns);

	meteo.Find("Scavenging_coefficient");
	vector<string> scav_coefficient = split(meteo.GetLine());
	vector<string>::iterator iter;
	for (int i = 0; i < this->Ns; i++)
	  {
	    iter = find(scav_coefficient.begin(), scav_coefficient.end(),
			this->species_list[i]);
	    if (iter++ == scav_coefficient.end() ||
		iter == scav_coefficient.end())
	      throw string("Unable to find scavenging coefficient for")
		+ string(" species \"") + this->species_list[i] + "\".";
	    scavenging_coefficient(i) = to_num<T>(*iter);
	  }
      }

    // Gets the deposition velocities of the species.
    if (this->option_process["with_deposition"])
      {
	deposition_velocity.resize(this->Ns);

	meteo.Find("Deposition_velocity");
	vector<string> dep_velocity = split(meteo.GetLine());
	vector<string>::iterator iter;
	for (int  i = 0; i < this->Ns; i++)
	  {
	    iter = find(dep_velocity.begin(), dep_velocity.end(),
			this->species_list[i]);
	    if (iter++ == dep_velocity.end() ||
		iter == dep_velocity.end())
	      throw string("Unable to find deposition velocity for")
		+ string(" species \"") + this->species_list[i] + "\".";
	    deposition_velocity(i) = to_num<T>(*iter);
	  }
      }
  }


  //! Computes concentration of a given species at a given point.
  /*!
    \param species species index.
    \param x abscissa (m).
    \param y ordinate (m).
    \param z height (m).
    \return The value of concentration at the given point (mass/m^3).
  */
  template<class T>
  T GaussianPlume<T>::GetConcentration(int species, T x, T y, T z)
  {
    const T pi = 3.14159265358979323846264;

    // Distances downwind and crosswind from the source (m).
    T distance_x, distance_y;
    // Horizontal and vertical standard deviations (m).
    T sigma_y, sigma_z;
    // Effective height of release (m).
    T effective_height;
    
    bool rural = land_category == "rural";

    // Loss factors.
    T radioactive_factor = 1.;
    T biological_factor = 1.;
    T scavenging_factor = 1.;
    T chamberlain_factor = 1.;
    T overcamp_factor = 1.;

    // Output concentration.
    T concentration = 0.;

    // Loop over all sources.
    for (typename list<PlumeSource<T> >::const_iterator iter
	   = SourceList.begin();
	 iter != SourceList.end(); iter++)
      if (iter->GetSpeciesIndex() == species)
	{
	  // Downwind distance from source.
	  distance_x = (x - iter->GetX()) * cos_angle_
	    + (y - iter->GetY()) * sin_angle_;

	  // Distance from downwind axis.
	  distance_y = (iter->GetX() - x) * sin_angle_
	    + (y - iter->GetY()) * cos_angle_;
	  distance_y = abs(distance_y);

	  // If there is anything to compute.
	  if (distance_x > 0)
	    {
	      if (this->option_process["with_plume_rise"])
		effective_height
		  = ComputeEffectiveSourceHeight(temperature_, wind_,
						 inversion_height_,
						 stability_,
						 iter->GetVelocity(),
						 iter->GetTemperature(),
						 iter->GetDiameter(),
						 iter->GetZ());
	      else
		effective_height = iter->GetZ();

	      // Horizontal diffusion parameter.
	      if (rural)
		sigma_y =
		  ComputeRuralPlumeHorizontalSigma(distance_x, stability_);
	      else
		sigma_y =
		  ComputeUrbanPlumeHorizontalSigma(distance_x, stability_);

	      // Vertical diffusion parameter.
	      if (rural)
		sigma_z =
		  ComputeRuralPlumeVerticalSigma(distance_x, stability_);
	      else
		sigma_z =
		  ComputeUrbanPlumeVerticalSigma(distance_x, stability_);

	      // Radioactive decay.
	      if (this->option_process["with_radioactive_decay"]
		  && half_life_time(species) != 0)
		radioactive_factor = exp(-log(2.) * distance_x
					 / (half_life_time(species) * wind_));

	      // Biological decay.
	      if (this->option_process["with_biological_decay"]
		  && biological_half_life_time(species) != 0)
		biological_factor
		  = exp(-log(2.) * distance_x
			/ (biological_half_life_time(species) * wind_));

	      // Scavenging.
	      if (this->option_process["with_scavenging"])
		scavenging_factor = exp(-scavenging_coefficient(species)
					* distance_x / wind_);

	      // Dry deposition.
	      if (this->option_process["with_deposition"])
		if (deposition_model == "Chamberlain")
		  {
		    T dx = distance_x / T(Nchamberlain);
		    T x, dsigma_z;
		    T square_height = 0.5 * effective_height
		      * effective_height;
		    T exp_factor = -deposition_velocity(species)
		      / wind_ * sqrt(2. / pi);

		    T fdry = 0.;

		    for (int i = 1; i < Nchamberlain + 1; i++)
		      {
			x = T(i) * dx;
			if (rural)
			  dsigma_z =
			    ComputeRuralPlumeVerticalSigma(x, stability_);
			else
			  dsigma_z =
			    ComputeUrbanPlumeVerticalSigma(x, stability_);

			T df = exp(-square_height / (dsigma_z * dsigma_z))
			  / dsigma_z;
			fdry = fdry + df;
		      }
		    chamberlain_factor = exp(exp_factor * fdry * dx);
		  }
		else
		  {
		    T vd = deposition_velocity(species);
		    T dsigma_z;
		    if (rural)
		      dsigma_z =
			DifferentiateRuralPlumeVerticalSigma(distance_x,
							     stability_);
		    else
		      dsigma_z =
			DifferentiateUrbanPlumeVerticalSigma(distance_x,
							     stability_);
		    overcamp_factor = 1. - 2. * vd
		      / (vd + wind_ * effective_height * dsigma_z / sigma_z);
		  }
	      
	      // Minimum volume.
	      T minimum_volume = pi * iter->GetDiameter()
		* iter->GetDiameter() * iter->GetVelocity();

	      concentration += radioactive_factor * biological_factor
		* scavenging_factor * chamberlain_factor
		* ComputePlumeConcentration(wind_, inversion_height_,
					    effective_height, iter->GetRate(),
					    distance_y, z, sigma_y, sigma_z,
					    overcamp_factor, minimum_volume);
	    }
	}
    return concentration;
  }
  

  //! Computes concentrations in the whole domain.
  template<class T>
  void GaussianPlume<T>::Compute()
  {
    int i, j, k, species;

    for (species = 0; species < this->Ns; species++)
      for (k = 0; k < this->Nz; k++)
	for (j = 0; j < this->Ny; j++)
	  for (i = 0; i < this->Nx; i++)
	    this->Concentration(species, k, j, i)
	      = GetConcentration(species, this->GridX4D(i),
				 this->GridY4D(j), this->GridZ4D(k));
  }


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_GAUSSIANPLUME_CXX
#endif
