// Copyright (C) 2006 CEREA
//     Author: Irne Korsakissok
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the modeling system Polyphemus.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODELS_GAUSSIANPLUME_AER_CXX


#include "GaussianPlume_aer.hxx"


namespace Polyphemus
{


  //! Main constructor.
  /*!
    \param config_file configuration file.
  */
  template<class T>
  GaussianPlume_aer<T>::GaussianPlume_aer(string config_file):
    GaussianPlume<T>(config_file)
  {
  }


  //! Allocates memory.
  /*! Allocates the grids and the concentration Data for aerosol species.
   */
  template<class T>
  void GaussianPlume_aer<T>::Allocate()
  {
    // Grids.
    GridS5D_aer = RegularGrid<T>(Ns_aer);
    GridB5D_aer = RegularGrid<T>(Ndiam);
    GridX5D = RegularGrid<T>(this->x_min, this->Delta_x, this->Nx);
    GridY5D = RegularGrid<T>(this->y_min, this->Delta_y, this->Ny);
    GridZ5D = RegularGrid<T>(this->Nz);

    // Aerosol concentrations.
    Concentration_aer.Resize(GridS5D_aer,GridB5D_aer, GridZ5D,
			     GridY5D, GridX5D);
  }


  //! Model initialization.
  /*! It reads the configuration.
   */
  template<class T>
  void GaussianPlume_aer<T>::Init()
  {

    /*** Model initialization for gaseous species ***/

    GaussianPlume<T>::Init();

    /*** Configuration ***/

    // File containing the aerosol diameters.
    this->config.SetSection("[domain]");
    this->config.PeekValue("File_diameter", file_diameter);

    // File containing the source data for aerosol species.
    this->config.SetSection("[gaussian]");
    this->config.PeekValue("File_source_aer", file_source_aer);

    // Gets aerosol species names.
    ConfigStream species_stream(this->file_species);
    species_stream.SetSection("[aerosol_species]");
    while (!species_stream.IsEmpty())
      species_list_aer.push_back(species_stream.GetElement());
    Ns_aer = int(species_list_aer.size());

    // Gets the diameters of aerosol species.
    ConfigStream diameter_stream(file_diameter);
    diameter_stream.SetSection("[diameter]");
    while (!diameter_stream.IsEmpty())
      diameter_list.push_back(to_num<T>(diameter_stream.GetElement()));
    Ndiam = int(diameter_list.size());

    /*** Allocations ***/

    Allocate();

    // Vertical layers heights.
    for (int k = 0; k < this->Nz; k++)
      GridZ5D(k) = this->GridZ4D(k);

    /*** Aerosol species data ***/

    // Reads the aerosol species half-lives in the species file.
    if (this->option_process["with_radioactive_decay"])
      {
	half_life_time_aer.resize(Ns_aer);
	species_stream.SetSection("[radioactive_decay]");
	for (int i = 0; i < Ns_aer; i++)
	  species_stream.PeekValue(species_list_aer[i],
				   half_life_time_aer(i));
	
	// Conversion from days to seconds.
	T factor = 24. * 3600.;
	for (int i = 0; i < this->Ns; i++)
	  half_life_time_aer(i) *= factor;
        }

    // Reads the biological half-lives of aerosol species in the species file.
    if (this->option_process["with_biological_decay"])
      {
	T day_value, night_value;
	biological_half_life_time_aer.resize(Ns_aer);
	for (int i = 0; i < this->Ns; i++)
	  {
	    species_stream.SetSection("[biological_decay]");
	    species_stream.Find(this->species_list[i]);
	    species_stream.GetNumber(day_value);
	    species_stream.GetNumber(night_value);
	    if (this->day_night == "day")
	      biological_half_life_time_aer(i) = day_value;
	    else
	      biological_half_life_time_aer(i) = night_value;
	  }
      }

    /*** Sources initialization for aerosol species ***/
    
    InitSource_aer(file_source_aer);
  }


  //! Sources initialization.
  /*! It sets all aerosol sources from a text file. Each source is described
    in a dedicated section "[source]" in which one finds the following
    entries:
    <ul>
    <li> Rate: the rate (mass unit per second),
    <li> Velocity: the efflux speed (m/s),
    <li> Temperature: the temperature of emissions (Celsius degrees),
    <li> Section: source cross-sectional area (m^2),
    <li> Abscissa: abscissa (m),
    <li> Ordinate: ordinate (m),
    <li> Altitude: height (m),
    <li> Species_name: name of aerosol species.
    </ul>
    \param source_file_aer file that describes the sources for aerosol
    species.
    \note The cross-sectional surface of the source is assumed to be a disc.
  */
  template<class T>
  void GaussianPlume_aer<T>::InitSource_aer(string source_file_aer)
  {
    T rate, velocity, temperature, section, diameter, x, y, z;
    string line, species_name;

    const T pi = 3.14159265358979323846264;

    ConfigStream source_stream(source_file_aer);

    SourceList_aer.clear();
    Nsource_aer = 0;
    while (!source_stream.IsEmpty())
      {
	line = source_stream.GetLine();
	if (split(line)[0] == "[aerosol_source]")
	  {
	    source_stream.PeekValue("Abscissa", x);
	    source_stream.PeekValue("Ordinate", y);
	    source_stream.PeekValue("Altitude", z);
	    source_stream.PeekValue("Rate", rate);
	    source_stream.PeekValue("Velocity", velocity);
	    source_stream.PeekValue("Temperature", temperature);
	    source_stream.PeekValue("Section", section);
	    source_stream.PeekValue("Species_name", species_name);
	    temperature += 273.15;
	    diameter = sqrt(4. * section / pi);

	    SourceList_aer.push_back
	      (PlumeSource<T>(rate, velocity, temperature, diameter, x, y,
			      z, GetSpeciesIndex_aer(species_name)));
	    Nsource_aer++;

	  }
      }
  }


  //! Initializes meteorological conditions.
  /*! It sets the meteorological data from a configuration file.  The
    situation is described in a dedicated section "[situation]" in which one
    finds the following entries:
    <ul>
    <li> Temperature: the temperature of ambient air (Celsius degrees).
    <li> Wind_angle: the angle between positive x-axis and wind,
    counterclockwise (degrees).
    <li> Wind: the wind velocity (m/s).
    <li> Inversion_height: the inversion height (m).
    <li> Stability: stability class in [A, F].
    </ul>
    \param meteo ConfigStream instance through which all entries (Temperature,
    Wind_angle, ...) may be read to set the meteorological situation.
    \param show_meteo indicates whether the meteorological data is to be
    displayed on screen.
  */
  template<class T>
  void GaussianPlume_aer<T>::InitMeteo(ConfigStream& meteo, bool show_meteo)
  {
    /*** Main meteorological variables and gaseous species data ***/

    GaussianPlume<T>::InitMeteo(meteo, show_meteo);

    /*** Scavenging coefficients for aerosol species ***/

    if (this->option_process["with_scavenging"])
      {
	scavenging_coefficient_aer.resize(Ns_aer, Ndiam);
	meteo.Find("Scavenging_coefficient_aer");
	vector<string> scav_coefficient = split(meteo.GetLine());
	int Nscav = int(scav_coefficient.size());

	for (int  i = 0; i < Ns_aer; i++)
	  for (int k = 0; k < Ndiam; k++)
	    {
	      T lambda = 0;
	      for (int j = 0; j < Nscav-1; j++)
		if (scav_coefficient[j] == species_list_aer[i]
		    + string("_") + to_str(k))
		  lambda = to_num<T>(scav_coefficient[j+1]);
	      scavenging_coefficient_aer(i, k) = lambda;
	    }
      }

    /*** Deposition velocities for aerosol species ***/

    if (this->option_process["with_deposition"])
      {
	deposition_velocity_aer.resize(Ns_aer, Ndiam);
	meteo.Find("Deposition_velocity_aer");
	vector<string> dep_velocity = split(meteo.GetLine());
	int Ndep = int(dep_velocity.size());

	for (int  i = 0; i < Ns_aer; i++)
	  for (int k = 0; k < Ndiam; k++)
	    {
	      T vd = 0;
	      for (int j = 0; j < Ndep-1; j++)
		if (dep_velocity[j] == species_list_aer[i]
		    + string("_") + to_str(k))
		  vd = to_num<T>(dep_velocity[j+1]);
	      deposition_velocity_aer(i, k) = vd;
	    }
      }
  }


  //! Computes concentrations in the whole domain.
  template<class T>
  void GaussianPlume_aer<T>::Compute()
  {
    GaussianPlume<T>::Compute();

    int i, j, k, species_aer;
    for (species_aer = 0; species_aer < Ns_aer; species_aer++)
      for (int diam = 0; diam < Ndiam; diam++)
     	for (k = 0; k < this->Nz; k++)
     	  for (j = 0; j < this->Ny; j++)
     	    for (i = 0; i < this->Nx; i++)
     	      Concentration_aer(species_aer, diam, k, j, i)
    		= GetConcentration_aer(species_aer, diam, GridX5D(i),
				       GridY5D(j), GridZ5D(k));
  }


  /*! \brief Computes concentration of a given aerosol species and a given
    aerosol diameter at a given point.
  */
  /*!
    \param species index of aerosol species.
    \param diam index of aerosol diameter.
    \param x abscissa (m).
    \param y ordinate (m).
    \param z height (m).
    \return The value of concentration at the given point (mass/m^3).
  */
  template<class T>
  T GaussianPlume_aer<T>::GetConcentration_aer(int species, int diam,
					       T x, T y, T z)
  {
    const T pi = 3.14159265358979323846264;

    // Distances downwind and crosswind from the source (m).
    T distance_x, distance_y;
    // Horizontal and vertical standard deviations (m).
    T sigma_y, sigma_z;
    // Effective height of release (m).
    T effective_height;

    bool rural = this->land_category == "rural";

    // Loss factors.
    T radioactive_factor = 1.;
    T biological_factor = 1.;
    T scavenging_factor = 1.;
    T chamberlain_factor = 1.;
    T overcamp_factor = 1.;

    // Output concentration.
    T concentration = 0.;

    // Loop over all sources for aerosol species.
    for (typename list<PlumeSource<T> >::const_iterator iter
	   = this->SourceList_aer.begin();
	 iter != this->SourceList_aer.end(); iter++)
      if (iter->GetSpeciesIndex() == species)
	{
	  // Downwind distance from source.
	  distance_x = (x - iter->GetX()) * this->cos_angle_
	    + (y - iter->GetY()) * this->sin_angle_;

	  // Distance from downwind axis.
	  distance_y = (iter->GetX() - x) * this->sin_angle_
	    + (y - iter->GetY()) * this->cos_angle_;
	  distance_y = abs(distance_y);

	  // If there is anything to compute.
	  if (distance_x > 0)
	    {
	      if (this->option_process["with_plume_rise"])
		effective_height
		  = ComputeEffectiveSourceHeight(this->temperature_,
						 this->wind_,
						 this->inversion_height_,
						 this->stability_,
						 iter->GetVelocity(),
						 iter->GetTemperature(),
						 iter->GetDiameter(),
						 iter->GetZ());
	      else
		effective_height = iter->GetZ();

	      // Horizontal diffusion parameter.
	      if (rural)
		sigma_y = ComputeRuralPlumeHorizontalSigma(distance_x,
							   this->stability_);
	      else
		sigma_y = ComputeUrbanPlumeHorizontalSigma(distance_x,
							   this->stability_);

	      // Vertical diffusion parameter.
	      if (rural)
		sigma_z = ComputeRuralPlumeVerticalSigma(distance_x,
							 this->stability_);
	      else
		sigma_z = ComputeUrbanPlumeVerticalSigma(distance_x,
							 this->stability_);
	      
	      // Radioactive decay.
	      if (this->option_process["with_radioactive_decay"]
		  && half_life_time_aer(species) != 0)
		radioactive_factor = exp(-log(2.) * distance_x
					 / (half_life_time_aer(species)
					    * this->wind_));

	      // Biological decay.
	      if (this->option_process["with_biological_decay"]
		  && biological_half_life_time_aer(species) != 0)
		biological_factor
		  = exp(-log(2.) * distance_x
			/ (biological_half_life_time_aer(species)
			   * this->wind_));

	      // Scavenging.
	      if (this->option_process["with_scavenging"])
		scavenging_factor
		  = exp(-scavenging_coefficient_aer(species, diam)
			* distance_x / this->wind_);

	      // Dry deposition.
	      if (this->option_process["with_deposition"])
		if (this->deposition_model == "Chamberlain")
		  {
		    T dx = distance_x / T(this->Nchamberlain);
		    T x, dsigma_z;
		    T square_height = 0.5 * effective_height
		      * effective_height;
		    T exp_factor = -deposition_velocity_aer(species, diam)
		      / this->wind_ * sqrt(2. / pi);
		  
		  T fdry = 0;
		  for (int i = 1; i < this->Nchamberlain + 1; i++)
		    {
		      x = T(i) * dx;
		      if (rural)
			dsigma_z =
			  ComputeRuralPlumeVerticalSigma(x, this->stability_);
		      else
		        dsigma_z =
			  ComputeUrbanPlumeVerticalSigma(x, this->stability_);

			T df = exp(-square_height / (dsigma_z * dsigma_z))
			  / dsigma_z;
			fdry = fdry + df;
		      }
		    chamberlain_factor = exp(exp_factor * fdry * dx);
		  }
		else
		  {
		    T vd = deposition_velocity_aer(species, diam);
		    T dsigma_z;
		    int stability = this->stability_;

		    if (rural)
		      dsigma_z =
			DifferentiateRuralPlumeVerticalSigma(distance_x,
							     stability);
		    else
		      dsigma_z =
			DifferentiateUrbanPlumeVerticalSigma(distance_x,
							     stability);
		    overcamp_factor = 1. - 2. * vd
		      / (vd + this->wind_ *
			 effective_height * dsigma_z / sigma_z) ;
		  }

	      // Minimum volume.
	      T minimum_volume = pi * iter->GetDiameter()
		* iter->GetDiameter() * iter->GetVelocity();

	      concentration += radioactive_factor * biological_factor
		* scavenging_factor *  chamberlain_factor
		* ComputePlumeConcentration(this->wind_,
					    this->inversion_height_,
					    effective_height, iter->GetRate(),
					    distance_y, z, sigma_y, sigma_z,
					    overcamp_factor, minimum_volume);
	    }
	}

   return concentration;
  }

  
  //! Returns the index of a given aerosol species.
  /*! If the species is not found, an exception is thrown.
    \param species the aerosol species name.
    \return The index (in the list of aerosol species) of the
    species named \a species.
  */
  template<class T>
  int GaussianPlume_aer<T>::GetSpeciesIndex_aer(string species) const
  {
    int index = 0;
    int length = species_list_aer.size();
    while (index < length && species_list_aer[index] != species)
      index++;
    if (index == length)
      throw string("Species \"") + species + "\" unknown.";
    return index;
  }


  //! Returns the concentrations Data for aerosols.
  /*!
    \return The concentrations Data for aerosols.
  */
  template<class T>
  Data<T, 5>& GaussianPlume_aer<T>::GetConcentration_aer()
  {
    return Concentration_aer;
  }


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_GAUSSIANPLUME_AER_CXX
#endif
