// Copyright (C) 2006 CEREA
//     Author: Irne Korsakissok
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODELS_GAUSSIANPUFF_AER_CXX


#include "GaussianPuff_aer.hxx"


namespace Polyphemus
{


  //! Main constructor.
  /*!
    \param config_file configuration file.
  */
  template<class T>
  GaussianPuff_aer<T>::GaussianPuff_aer(string config_file):
    GaussianPuff<T>(config_file), Npuff_aer(0)
  {
  }


  //! Allocates memory.
  /*! Allocates the grids and the concentration Data for aerosol species.
   */
  template<class T>
  void GaussianPuff_aer<T>::Allocate()
  {
    // Grids.
    GridS5D_aer = RegularGrid<T>(Ns_aer);
    GridB5D_aer = RegularGrid<T>(Ndiam);
    GridX5D = RegularGrid<T>(this->x_min, this->Delta_x, this->Nx);
    GridY5D = RegularGrid<T>(this->y_min, this->Delta_y, this->Ny);
    GridZ5D = RegularGrid<T>(this->Nz);

    // Concentrations of aerosol species.
    Concentration_aer.Resize(GridS5D_aer,GridB5D_aer, GridZ5D,
			     GridY5D, GridX5D);
  }


  //! Model initialization.
  /*! It reads the configuration.
   */
  template<class T>
  void GaussianPuff_aer<T>::Init()
  {

    /*** Model initialization for gaseous species ***/

    GaussianPuff<T>::Init();

    // File containing the aerosol diameters.
    this->config.SetSection("[domain]");
    this->config.PeekValue("File_diameter", file_diameter);

    // File containing the source data for aerosol species.
    this->config.SetSection("[gaussian]");
    this->config.PeekValue("File_puff_aer", file_puff_aer);

    // Gets aerosol species names.
    ConfigStream species_stream(this->file_species);
    species_stream.SetSection("[aerosol_species]");
    while (!species_stream.IsEmpty())
      species_list_aer.push_back(species_stream.GetElement());
    Ns_aer = int(species_list_aer.size());

    // Gets the aerosol diameters.
    ConfigStream diameter_stream(file_diameter);
    diameter_stream.SetSection("[diameter]");
    while (!diameter_stream.IsEmpty())
      diameter_list.push_back(to_num<T>(diameter_stream.GetElement()));
    Ndiam = int(diameter_list.size());

    /*** Allocations ***/

    // Allocates memory (aerosol species).
    Allocate();

    // Vertical layers heights.
    for (int k=0; k<this->Nz; k++)
      GridZ5D(k) = this->GridZ4D(k);

    /*** Aerosol species data ***/

    // Reads the aerosol half-lives in the species file.
    if (this->option_process["with_radioactive_decay"])
      {
	half_life_time_aer.resize(Ns_aer);
	for (int i = 0; i < Ns_aer; i++)
	  {
	    species_stream.SetSection("[radioactive_decay]");
	    species_stream.PeekValue(species_list_aer[i],
				     half_life_time_aer(i));
	  }
      }

    // Reads the aerosol biological half-lives in the species file.
    if (this->option_process["with_biological_decay"])
      {
	T day_value, night_value;
	biological_half_life_time_aer.resize(Ns_aer);
	for (int i = 0; i < Ns_aer; i++)
	  {
	    species_stream.SetSection("[biological_decay]");
	    species_stream.Find(species_list_aer[i]);
	    species_stream.GetNumber(day_value);
	    species_stream.GetNumber(night_value);
	    if (this->day_night == "day")
	      biological_half_life_time_aer(i) = day_value;
	    else
	      biological_half_life_time_aer(i) = night_value;
	  }
      }

    /*** Puffs initialization for aerosol species ***/

    InitPuffSource_aer(file_puff_aer);
  }


  //! Puff sources initialization.
  /*! It sets all aerosol sources from a text file. Each source is described
    in a dedicated section "[source]" in which one finds the following
    entries:
    <ul>
    <li> Abscissa: abscissa (m),
    <li> Ordinate: ordinate (m),
    <li> Altitude: height (m),
    <li> Species_name: name of aerosol species.
    <li> Release_time: the release time (s),
    <li> Quantity: the released quantity (mass unit),
    </ul>
    \param file_puff_aer file that describes the sources.
  */
  template<class T>
  void GaussianPuff_aer<T>::InitPuffSource_aer(string file_puff_aer)
  {
    // Puffs definitions.
    T time_puff,quantity, abscissa, ordinate, height;
    string line, species_name;

    ConfigStream puff_stream(file_puff_aer);

    while (!puff_stream.IsEmpty())
      {
	line = puff_stream.GetLine();
	if (split(line)[0] == "[aerosol_source]")
	  {
	    puff_stream.PeekValue("Abscissa", abscissa);
	    puff_stream.PeekValue("Ordinate", ordinate);
	    puff_stream.PeekValue("Altitude", height);
	    puff_stream.PeekValue("Release_time", time_puff);
	    puff_stream.PeekValue("Quantity", quantity);
	    puff_stream.PeekValue("Species_name", species_name);
	    vector< Puff<T> > temp;
	    for (int j = 0; j < Ndiam; j++)
	      temp.push_back(Puff<T>(time_puff, quantity, abscissa,
				     ordinate, height,
				     GetSpeciesIndex_aer(species_name)));
	    PuffList_aer.push_back(temp);
	  }
      }
    Npuff_aer = int(PuffList_aer.size());

    // Sets the half-lives of the puff species.
    if (this->option_process["with_radioactive_decay"])
      for (int i = 0; i < Npuff_aer; i++)
	for (int j = 0; j < Ndiam; j++)
	  PuffList_aer[i][j]
	    .SetHalfTime(half_life_time_aer(PuffList_aer[i][j]
					    .GetSpeciesIndex()));

	// Sets the biological half-lives of the puff species.
    if (this->option_process["with_biological_decay"])
      for (int i = 0; i < Npuff_aer; i++)
	for (int j = 0; j < Ndiam; j++)
	  {
	    int s = PuffList_aer[i][j].GetSpeciesIndex();
	    PuffList_aer[i][j]
	      .SetBioHalfTime(biological_half_life_time_aer(s));
	  }
  }


  //! Initializes meteorological conditions.
  /*! It sets the meteorological data from a configuration file.  The
    situation is described in a dedicated section "[situation]" in which one
    finds the following entries:
    <ul>
    <li> Temperature: the temperature of ambient air (Celsius degrees).
    <li> Wind_angle: the angle between positive x-axis and wind,
    counterclockwise (degrees).
    <li> Wind: the wind velocity (m/s).
    <li> Stability: stability class in [A, F].
    </ul>
    \param meteo ConfigStream instance through which all entries (Temperature,
    Wind_angle, ...) may be read to set the meteorological situation.
    \param show_meteo indicates whether the meteorological data is to be
    displayed on screen.
  */
  template<class T>
  void GaussianPuff_aer<T>::InitMeteo(ConfigStream& meteo, bool show_meteo)
  {

    /*** Main meteorological variables and gaseous species data ***/

    GaussianPuff<T>::InitMeteo(meteo, show_meteo);

    /*** Initialization of puffs positions (aerosol species) ***/

    for (int i = 0; i < Npuff_aer; i++)
      for (int j = 0; j < Ndiam; j++)
	PuffList_aer[i][j].InitPuff();
 
    /*** Scavenging coefficients for aerosol species ***/

    if (this->option_process["with_scavenging"])
      {
	scavenging_coefficient_aer.resize(Ns_aer, Ndiam);
	meteo.Find("Scavenging_coefficient_aer");
	vector<string> scav_coefficient = split(meteo.GetLine());
	int Nscav = int(scav_coefficient.size());

	for (int  i = 0; i < Ns_aer; i++)
	  for (int k = 0; k < Ndiam; k++)
	    {
	      T lambda = 0;
	      for (int j = 0; j < Nscav-1; j++)
		if (scav_coefficient[j] == species_list_aer[i]
		    + string("-") + to_str(k))
		  lambda = to_num<T>(scav_coefficient[j+1]);
	      scavenging_coefficient_aer(i,k) = lambda;
	    }

	// Sets the scavenging coefficient for each puff aerosol species.
	for (int i = 0; i < Npuff_aer; i++)
	  for (int j = 0; j < Ndiam; j++)
	    {
	      int s = PuffList_aer[i][j].GetSpeciesIndex();
	      PuffList_aer[i][j]
		.SetScavengingCoefficient(scavenging_coefficient_aer(s,j));
	    }
      }

    /*** Deposition velocities for aerosol species ***/

    if (this->option_process["with_deposition"])
      {
	deposition_velocity_aer.resize(Ns_aer,Ndiam);
	meteo.Find("Deposition_velocity_aer");
	vector<string> dep_velocity = split(meteo.GetLine());
	int Ndep = int(dep_velocity.size());

	for (int  i = 0; i < Ns_aer; i++)
	  for (int k = 0; k < Ndiam; k++)
	    {
	      T vd = 0;
	      for (int j = 0; j < Ndep-1; j++)
		if (dep_velocity[j] == species_list_aer[i]
		    + string("-") + to_str(k))
		  vd = to_num<T>(dep_velocity[j+1]);
	      deposition_velocity_aer(i,k) = vd;
	    }

	// Sets the deposition velocity for each puff aerosol species.
	for (int i = 0; i < Npuff_aer; i++)
	  for (int j = 0; j < Ndiam; j++)
	    {
	      int s = PuffList_aer[i][j].GetSpeciesIndex();
	      PuffList_aer[i][j]
		.SetDepositionVelocity(deposition_velocity_aer(s, j));
	    }
      }
  }


  //! Method called at each time step to initialize the model.
  /*!
    \note Empty method.
  */ template<class T>
  void GaussianPuff_aer<T>::InitStep()
  {
    GaussianPuff<T>::InitStep();
  }


  //! Computes the new position of the puff center after advection.
  template<class T>
  void GaussianPuff_aer<T>::Advection_aer()
  {
    for (int i = 0; i < Npuff_aer; i++)
      for (int j = 0; j < Ndiam; j++)
	if (this->current_time >= PuffList_aer[i][j].GetReleaseTime())
	  PuffList_aer[i][j].Advection(this->wind_, this->cos_angle_,
				       this->sin_angle_, this->Delta_t);
  }


  //!  Computes the horizontal and vertical standard deviations.
  template<class T>
  void GaussianPuff_aer<T>::Diffusion_aer()
  {
    T sigma_x, sigma_y, sigma_z;
    for (int i = 0; i < Npuff_aer; i++)
      for (int j = 0; j < Ndiam; j++)
	if (this->current_time >= PuffList_aer[i][j].GetReleaseTime())
	  {
	    T distance_x = PuffList_aer[i][j].GetDistance();

	    if (this->land_category == "rural")
	      {
		sigma_x = ComputeRuralPlumeHorizontalSigma(distance_x,
							   this->stability_);
		sigma_y = ComputeRuralPlumeHorizontalSigma(distance_x,
							   this->stability_);
		sigma_z = ComputeRuralPlumeVerticalSigma(distance_x,
							 this->stability_);
	      }
	    else
	      {
		sigma_x = ComputeUrbanPlumeHorizontalSigma(distance_x,
							   this->stability_);
		sigma_y = ComputeUrbanPlumeHorizontalSigma(distance_x,
							   this->stability_);
		sigma_z = ComputeUrbanPlumeVerticalSigma(distance_x,
							 this->stability_);
	      }

	    PuffList_aer[i][j].SetSigma_x(sigma_x);
	    PuffList_aer[i][j].SetSigma_y(sigma_y);
	    PuffList_aer[i][j].SetSigma_z(sigma_z);
	  }
  }


  //! Performs one step forward.
  template<class T>
  void GaussianPuff_aer<T>::Forward()
  {
    Advection_aer();
    Diffusion_aer();

    int puff, diam, i, j, k;
    Concentration_aer.SetZero();
    for (puff = 0; puff < Npuff_aer; puff++)
      for (diam = 0; diam < Ndiam; diam++)
	if (this->current_time >= PuffList_aer[puff][diam].GetReleaseTime())
	  for (k = 0; k < this->Nz; k++)
	    for (j = 0; j < this->Ny; j++)
	      for (i = 0; i < this->Nx; i++)
		Concentration_aer(PuffList_aer[puff][diam].GetSpeciesIndex(),
				  diam, k, j, i) +=
		  PuffList_aer[puff][diam]
		  .ComputeConcentration(GridX5D(i), GridY5D(j), GridZ5D(k),
					this->wind_, this->cos_angle_,
					this->sin_angle_, this->stability_,
					this->inversion_height_,
					this->land_category,
					this->option_process
					["with_radioactive_decay"],
					this->option_process
					["with_biological_decay"],
					this->option_process
					["with_scavenging"],
					this->option_process
					["with_deposition"],
					this->deposition_model,
					this->Nchamberlain_tot);

    GaussianPuff<T>::Forward();
  }


  //! Returns the index of a given aerosol species.
  /*! If the species is not found, an exception is thrown.
    \param species the aerosol species name.
    \return The index (in the list of aerosol species)
    of the species named \a species.
  */
  template<class T>
  int GaussianPuff_aer<T>::GetSpeciesIndex_aer(string species) const
  {
    int index = 0;
    int length = species_list_aer.size();
    while (index < length && species_list_aer[index] != species)
      index++;
    if (index == length)
      throw string("Species \"") + species + "\" unknown.";
    return index;
  }


  //! Returns the concentrations Data for aerosols.
  /*!
    \return The concentrations Data for aerosols.
  */
  template<class T>
  Data<T, 5>& GaussianPuff_aer<T>::GetConcentration_aer()
  {
    return Concentration_aer;
  }


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_GAUSSIANPUFF_AER_CXX
#endif
