// Copyright (C) 2006 CEREA
//     Authors: Hadjira Foudhil, Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the modeling system Polyphemus.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODELS_PLUMESOURCE_CXX


#include "PlumeSource.hxx"


namespace Polyphemus
{


  //! Main constructor.
  /*!
    \param rate source emission rate.
    \param velocity efflux speed of gases.
    \param temperature temperature of emissions.
    \param diameter source diameter.
    \param x abscissa.
    \param y ordinate.
    \param z height.
    \param species_index index associated to the species.
   */
  template<class T>
  PlumeSource<T>::PlumeSource(T rate, T velocity, T temperature, T diameter,
			      T x, T y, T z, int species_index):
    rate_(rate), velocity_(velocity), temperature_(temperature),
    diameter_(diameter), x_(x), y_(y), z_(z), species_index_(species_index)
  {
  }


  //! Returns the source rate.
  /*!
    \return The source rate.
   */
  template<class T>
  inline T PlumeSource<T>::GetRate() const
  {
    return rate_;
  }


  //! Returns the efflux speed.
  /*!
    \return The efflux speed.
   */
  template<class T>
  inline T PlumeSource<T>::GetVelocity() const
  {
    return velocity_;
  }


  //! Returns the temperature of emissions.
  /*!
    \return The temperature of emissions.
   */
  template<class T>
  inline T PlumeSource<T>::GetTemperature() const
  {
    return temperature_;
  }


  //! Returns the source diameter.
  /*!
    \return The source diameter.
   */
  template<class T>
  inline T PlumeSource<T>::GetDiameter() const
  {
    return diameter_;
  }


  //! Returns the source position along x.
  /*!
    \return The source abscissa.
   */
  template<class T>
  inline T PlumeSource<T>::GetX() const
  {
    return x_;
  }


  //! Returns the source position along y.
  /*!
    \return The source ordinate.
   */
  template<class T>
  inline T PlumeSource<T>::GetY() const
  {
    return y_;
  }

 
  //! Returns the source height.
  /*!
    \return The source height.
   */
  template<class T>
  inline T PlumeSource<T>::GetZ() const
  {
    return z_;
  }


  //! Returns the species index.
  /*!
    \return The species index.
   */
  template<class T>
  inline int PlumeSource<T>::GetSpeciesIndex() const
  {
    return species_index_;
  }


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_PLUMESOURCE_CXX
#endif
