// Copyright (C) 2006 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODELS_POLAIR3DAEROSOL_HXX


#include <vector>
#include <utility>
#include "AtmoData.hxx"
#include "Polair3DChemistry.cxx"


namespace Polyphemus
{

  
  using namespace std;
  using namespace AtmoData;

  
  //////////////////////
  // FORTRAN FUNCTION //
  //////////////////////

#if defined(__GNUG__) && __GNUG__ < 4 && !defined (__INTEL_COMPILER)
#define _compute_dry_deposition compute_dry_deposition__
#define _compute_scavenging_coefficient_aer \
compute_scavenging_coefficient_aer__
#else
#define _compute_dry_deposition compute_dry_deposition_
#define _compute_scavenging_coefficient_aer \
compute_scavenging_coefficient_aer_
#endif

  extern "C"
  {
    void _compute_dry_deposition(double*, int*, int*, double*, double*,
				 double*, double*, double*, double*, double*,
				 double*, double*, double*, double*, double*,
				 double*, double*, double*, double*, double*);
    void _compute_scavenging_coefficient_aer(int*, double*, double*, double*,
					     double*, double*, double*,
					     double*, double*);
  }


  /////////////////////
  // POLAIR3DAEROSOL //
  /////////////////////
  

  /*! \brief This class is a solver for an advection-diffusion-reaction
    equation with aerosols.*/
  template<class T, class ClassAdvection,
	   class ClassDiffusion, class ClassChemistry>
  class Polair3DAerosol:
    public Polair3DChemistry<T, ClassAdvection, ClassDiffusion,
			     ClassChemistry>
  {

  public:

    /*** Type declarations ***/

    typedef typename map<string, InputFiles<T> >::iterator
    input_files_iterator;

  public:
    
    /*** Configuration ***/

    //! List of aerosol species (with their bins) with initial conditions.
    vector<pair<string, vector<int> > > species_list_ic_aer;
    //! List of aerosol species (with their bins) with boundary conditions.
    vector<pair<string, vector<int> > > species_list_bc_aer;
    //! List of aerosol bins with deposition velocities.
    vector<int> bin_list_dep_aer;
    //! List of aerosol bins with scavenging.
    vector<int> bin_list_scav_aer;
    //! List of aerosol species (with their bins) with surface emissions.
    vector<pair<string, vector<int> > > species_list_surf_emis_aer;
    //! List of aerosol species (with their bins) with volume emissions.
    vector<pair<string, vector<int> > > species_list_vol_emis_aer;

    /*** Domain ***/

    //! 5D grid along z.
    RegularGrid<T> GridZ5D;
    //! 5D grid along y.
    RegularGrid<T> GridY5D;
    //! 5D grid along x.
    RegularGrid<T> GridX5D;

    //! Coordinates of boundary conditions along x.
    RegularGrid<T> GridX5D_interf_bc;
    //! Coordinates of boundary conditions along y.
    RegularGrid<T> GridY5D_interf_bc;

    /*** Species ***/

    //! List of aerosol bins.
    vector<string> bin_list;

    //! Bins bounds (in m).
    Array<T, 1> BinBound_aer;

    //! 4D grid for aerosol bins.
    RegularGrid<T> GridB4D_aer;
    //! 5D grid for aerosol bins.
    RegularGrid<T> GridB5D_aer;

    //! 4D grid for aerosol species.
    RegularGrid<T> GridS4D_aer;
    //! 5D grid for aerosol species.
    RegularGrid<T> GridS5D_aer;

    //! Fixed density (kg/m^3).
    T fixed_density_aer;
    //! Density per bin (kg/m^3).
    Array<T, 1> Density_aer;

    //! Number of aerosol species with initial conditions.
    int Ns_ic_aer;

    //! Dissolution heat (kcal / mol at 298K).
    map<string, T> dissolution_heat;

    /*** Land data ***/

    // Land data are used to compute deposition velocities. They depend on the
    // land use category and sometimes on the season. It is assumed that there
    // are the four seasons winter, spring, summer, fall (in this order), plus
    // a fifth season for snow.

    //! Number of land categories.
    int Nland;
    //! Amount (in [0, 1]) of each land category.
    Data<T, 3> LUC;
    //! Roughness height for all land categories and seasons (m).
    Array<T, 2> LandRoughnessHeight;
    /*! Characteristic radius of small receptors (deposition) for all land
      categories and seasons (m). */
    Array<T, 2> SmallRadius;
    /*! Characteristic radius of large receptors (deposition) for all land
      categories and seasons (m). */
    Array<T, 2> LargeRadius;
    //! Coefficient from Zhang deposition model.
    Array<T, 1> ZhangAlpha;
    //! Coefficient from Zhang deposition model.
    Array<T, 1> ZhangGamma;

    /*** Meteorological fields ***/

    //! Temperature at 2 m at current date (K).
    Data<T, 2> SurfaceTemperature_i;
    //! Temperature at 2 m at next date (K).
    Data<T, 2> SurfaceTemperature_f;
    //! Buffer for temperature at 2 m.
    Data<T, 2> FileSurfaceTemperature_i;
    //! Buffer for temperature at 2 m.
    Data<T, 2> FileSurfaceTemperature_f;
    
    //! Surface pressure at current date (Pa).
    Data<T, 2> SurfacePressure_i;
    //! Surface pressure at next date (Pa).
    Data<T, 2> SurfacePressure_f;
    //! Surface pressure buffer.
    Data<T, 2> FileSurfacePressure_i;
    //! Surface pressure buffer.
    Data<T, 2> FileSurfacePressure_f;
    
    //! First level wind module at current date (m / s).
    Data<T, 2> FirstLevelWindModule_i;
    //! First level wind module at next date (m / s).
    Data<T, 2> FirstLevelWindModule_f;
    
    //! Liquid water content (kg / kg).
    Data<T, 3> LiquidWaterContent_i;
    //! Liquid water content buffer.
    Data<T, 3> FileLiquidWaterContent_i;
    //! Liquid water content buffer.
    Data<T, 3> FileLiquidWaterContent_f;
    
    //! Relative humidity at current date.
    Data<T, 3> RelativeHumidity_i;
    //! Relative humidity at next date.
    Data<T, 3> RelativeHumidity_f;
    
    //! Snow height at current date (m).
    Data<T, 2> SnowHeight_i;
    //! Snow height at next date (m).
    Data<T, 2> SnowHeight_f;
    //! First level wind module buffer.
    Data<T, 2> FileSnowHeight_i;
    //! First level wind module buffer.
    Data<T, 2> FileSnowHeight_f;
    
    /*** Boundary conditions ***/

    //! Number of aerosol species with boundary conditions.
    int Ns_bc_aer;
    //! Grid for aerosol species with boundary conditions.
    RegularGrid<T> GridS_bc_aer;
    //! Number of aerosol bins with boundary conditions.
    int Nbin_bc_aer;
    //! Grid for aerosol bins with boundary conditions.
    RegularGrid<T> GridB_bc_aer;

    //! Boundary conditions along z at current date for aerosols.
    Data<T, 4> BoundaryCondition_z_aer_i;
    //! Boundary conditions buffer.
    Data<T, 4> FileBoundaryCondition_z_aer_i;
    //! Boundary conditions buffer.
    Data<T, 4> FileBoundaryCondition_z_aer_f;

    //! Boundary conditions along y at current date for aerosols.
    Data<T, 5> BoundaryCondition_y_aer_i;
    //! Boundary conditions buffer.
    Data<T, 5> FileBoundaryCondition_y_aer_i;
    //! Boundary conditions buffer.
    Data<T, 5> FileBoundaryCondition_y_aer_f;

    //! Boundary conditions along x at current date for aerosols.
    Data<T, 5> BoundaryCondition_x_aer_i;
    //! Boundary conditions buffer.
    Data<T, 5> FileBoundaryCondition_x_aer_i;
    //! Boundary conditions buffer.
    Data<T, 5> FileBoundaryCondition_x_aer_f;

    /*** Loss terms ***/

    //! Number of aerosol bins with deposition velocities.
    int Nbin_dep_aer;
    //! Grid for aerosol bins with deposition velocities.
    RegularGrid<T> GridB_dep_aer;
    //! Deposition velocities at current date for aerosols.
    Data<T, 3> DepositionVelocity_aer_i;
    //! Deposition velocities at next date for aerosols.
    Data<T, 3> DepositionVelocity_aer_f;
    //! Deposition velocities buffer.
    Data<T, 3> FileDepositionVelocity_aer_i;
    //! Deposition velocities buffer.
    Data<T, 3> FileDepositionVelocity_aer_f;

    //! Number of aerosol bins with scavenging.
    int Nbin_scav_aer;
    //! Grid for aerosol bins with scavenging.
    RegularGrid<T> GridB_scav_aer;
    //! Scavenging coefficients at current date for aerosols.
    Data<T, 4> ScavengingCoefficient_aer_i;
    //! Scavenging coefficients at next date for aerosols.
    Data<T, 4> ScavengingCoefficient_aer_f;

    /*** Source terms ***/

    //! List of point emissions.
    vector<map<string, string> > point_emission_list_aer;

    //! Number of aerosol species with surface emissions.
    int Ns_surf_emis_aer;
    //! Grid for aerosol species with surface emissions.
    RegularGrid<T> GridS_surf_emis_aer;
    //! Number of aerosol bins with surface emissions.
    int Nbin_surf_emis_aer;
    //! Grid for aerosol bins with surface emissions.
    RegularGrid<T> GridB_surf_emis_aer;
    //! Surface emissions at current date for aerosols.
    Data<T, 4> SurfaceEmission_aer_i;
    //! Surface emissions at next date for aerosols.
    Data<T, 4> SurfaceEmission_aer_f;
    //! Surface emissions buffer.
    Data<T, 4> FileSurfaceEmission_aer_i;
    //! Surface emissions buffer.
    Data<T, 4> FileSurfaceEmission_aer_f;

    //! Number of aerosol species with volume emissions.
    int Ns_vol_emis_aer;
    //! Grid for aerosol species with volume emissions.
    RegularGrid<T> GridS_vol_emis_aer;
    //! Number of aerosol bins with volume emissions.
    int Nbin_vol_emis_aer;
    //! Grid for aerosol bins with volume emissions.
    RegularGrid<T> GridB_vol_emis_aer;
    //! Number of aerosol species with volume emissions.
    int Nz_vol_emis_aer;
    //! Grid for altitudes of volume emissions for aerosols.
    RegularGrid<T> GridZ_vol_emis_aer;
    //! Volume emissions at current date for aerosols.
    Data<T, 5> VolumeEmission_aer_i;
    //! Volume emissions at next date for aerosols.
    Data<T, 5> VolumeEmission_aer_f;
    //! Volume emissions buffer.
    Data<T, 5> FileVolumeEmission_aer_i;
    //! Volume emissions buffer.
    Data<T, 5> FileVolumeEmission_aer_f;

    /*** Aerosol physical properties ***/

    //! Wet aerosol diameter (m).
    Data<T, 4> WetDiameter_aer;

    //! Droplet pH.
    Data<T, 3> pH;

    /*** Sources (source splitting) ***/

    //! Sources for aerosols at current date (for source splitting).
    Data<T, 5> Source_aer_i;
    //! Sources for aerosols at next date (for source splitting).
    Data<T, 5> Source_aer_f;
    
  public:
    
    /*** Constructor and destructor ***/
    
    Polair3DAerosol(string config_file);
    virtual ~Polair3DAerosol();
    
    /*** Configuration ***/

    //! Threshold on liquid water content for clouds (g / m^3).
    T lwc_cloud_threshold;

    virtual void ReadConfiguration();
    virtual void CheckConfiguration();

    bool HasInitialCondition_aer(int s, int b) const;
    bool HasInitialCondition_aer(string name, int b) const;
    
    bool HasBoundaryCondition_aer(int s, int b) const;
    bool HasBoundaryCondition_aer(string name, int b) const;
    vector<int> BoundaryConditionIndex_aer(int s, int b) const;
    
    bool HasDepositionVelocity_aer(int b) const;
    int DepositionVelocityIndex_aer(int b) const;
    
    bool HasScavenging_aer(int s, int b) const;
    bool HasScavenging_aer(string name, int b) const;
    
    bool HasSurfaceEmission_aer(int s, int b) const;
    bool HasSurfaceEmission_aer(string name, int b) const;
    vector<int> SurfaceEmissionIndex_aer(int s, int b) const;
    
    bool HasVolumeEmission_aer(int s, int b) const;
    bool HasVolumeEmission_aer(string name, int b) const;
    int VolumeEmissionIndex_aer(int s) const;
    string VolumeEmissionName_aer(int s) const;
    int VolumeEmissionGlobalIndex_aer(int s) const;

    /*** Initializations ***/

    virtual void Allocate();
    virtual void Init();
    virtual void InitStep();
    void InitWetDiameter_aer(Data<T, 3>& RelativeHumidity_,
			     Data<T, 3>& Temperature_,
			     Data<T, 4>& WetDiameter_aer_);
    void InitDepositionVelocity(Data<T, 3>& Temperature_,
				Data<T, 3>& Pressure_,
				Data<T, 2>& SurfaceTemperature_,
				Data<T, 2>& SurfacePressure_,
				Data<T, 2>& FirstLevelWindModule_,
				Data<T, 3>& RelativeHumidity_,
				Data<T, 2>& SnowHeight_,
				Data<T, 3>& DepositionVelocity_aer_);
    void InitScavengingCoefficient(Data<T, 3>& Temperature_,
				   Data<T, 3>& Pressure_,
				   Data<T, 3>& LiquidWaterContent_,
				   Data<T, 3>& pH_,
				   Data<T, 2>& CloudHeight_,
				   Data<T, 2>& Rain_,
				   Data<T, 4>&
				   ScavengingCoefficient_);
    void InitScavengingCoefficient_aer(Data<T, 3>& Temperature_,
				       Data<T, 3>& Pressure_,
				       Data<T, 4>& WetDiameter_aer_,
				       Data<T, 2>& CloudHeight_,
				       Data<T, 2>& Rain_,
				       Data<T, 4>&
				       ScavengingCoefficient_aer_);

    /*** Integration ***/

    virtual void Advection();
    virtual void Diffusion();
    void PointEmission_aer();
    virtual void Chemistry();

    virtual void Forward();

    /*** Access methods ***/

    int GetNs_source_aer();
    int GetNz_source_aer();
    int GetNbinMax_source_aer();
    int SourceGlobalBinIndex_aer(int s, int b);
    bool HasSource_aer(int s, int b);
    int SourceGlobalIndex_aer(int s);

    Data<T, 5>& GetSource_aer_i();
    Data<T, 5>& GetSource_aer_f();

  protected:

    virtual void InitAllData();

  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_POLAIR3DAEROSOL_HXX
#endif
