// Copyright (C) 2005 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODELS_POLAIR3DTRANSPORT_CXX


#include "Polair3DTransport.hxx"


namespace Polyphemus
{


  ////////////////////////////////
  // CONSTRUCTOR AND DESTRUCTOR //
  ////////////////////////////////


  //! Main constructor.
  /*!
    \param config_file configuration file.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::Polair3DTransport(string config_file): BaseModel<T>(config_file)
  {

    /*** Managed data ***/

    this->option_manage["initial_condition"] = true;
    this->option_manage["temperature"] = true;
    this->option_manage["pressure"] = true;
    this->option_manage["horizontal_wind"] = true;
    this->option_manage["vertical_wind"] = true;
    this->option_manage["rain"] = true;
    this->option_manage["cloud_height"] = true;
    this->option_manage["horizontal_diffusion"] = true;
    this->option_manage["vertical_diffusion"] = true;
    this->option_manage["boundary_condition"] = true;
    this->option_manage["deposition_velocity"] = true;
    this->option_manage["surface_emission"] = true;
    this->option_manage["volume_emission"] = true;
    this->option_manage["scavenging_coefficient"] = true;

    /*** Pointers to 2D data ***/

    this->D2_map["Rain"] = &Rain_i;
    this->D2_map["Rain_i"] = &Rain_i;

    this->D2_map["CloudHeight"] = &CloudHeight_f;
    this->D2_map["CloudHeight_i"] = &CloudHeight_i;
    this->D2_map["CloudHeight_f"] = &CloudHeight_f;

    /*** Pointers to 3D data ***/

    this->D3_map["Temperature"] = &Temperature_f;
    this->D3_map["Temperature_i"] = &Temperature_i;
    this->D3_map["Temperature_f"] = &Temperature_f;
    
    this->D3_map["Pressure"] = &Pressure_f;
    this->D3_map["Pressure_i"] = &Pressure_i;
    this->D3_map["Pressure_f"] = &Pressure_f;
    
    this->D3_map["AirDensity"] = &AirDensity_f;
    this->D3_map["AirDensity_i"] = &AirDensity_i;
    this->D3_map["AirDensity_f"] = &AirDensity_f;
    
    this->D3_map["VerticalWind"] = &VerticalWind_i;
    this->D3_map["VerticalWind_i"] = &VerticalWind_i;

    this->D3_map["MeridionalWind"] = &MeridionalWind_i;
    this->D3_map["MeridionalWind_i"] = &MeridionalWind_i;
    
    this->D3_map["ZonalWind"] = &ZonalWind_i;
    this->D3_map["ZonalWind_i"] = &ZonalWind_i;
    
    this->D3_map["VerticalDiffusionCoefficient"]
      = &VerticalDiffusionCoefficient_f;
    this->D3_map["VerticalDiffusionCoefficient_i"]
      = &VerticalDiffusionCoefficient_i;
    this->D3_map["VerticalDiffusionCoefficient_f"]
      = &VerticalDiffusionCoefficient_f;

    this->D3_map["MeridionalDiffusionCoefficient"]
      = &MeridionalDiffusionCoefficient_i;
    this->D3_map["MeridionalDiffusionCoefficient_i"]
      = &MeridionalDiffusionCoefficient_i;

    this->D3_map["ZonalDiffusionCoefficient"]
      = &ZonalDiffusionCoefficient_i;
    this->D3_map["ZonalDiffusionCoefficient_i"]
      = &ZonalDiffusionCoefficient_i;

    this->D3_map["BoundaryCondition_z"] = &BoundaryCondition_z_i;
    this->D3_map["BoundaryCondition_z_i"] = &BoundaryCondition_z_i;

    this->D3_map["DepositionVelocity"] = &DepositionVelocity_f;
    this->D3_map["DepositionVelocity_i"] = &DepositionVelocity_i;
    this->D3_map["DepositionVelocity_f"] = &DepositionVelocity_f;

    this->D3_map["SurfaceEmission"] = &SurfaceEmission_f;
    this->D3_map["SurfaceEmission_i"] = &SurfaceEmission_i;
    this->D3_map["SurfaceEmission_f"] = &SurfaceEmission_f;

    /*** Pointers to 4D data ***/

    this->D4_map["BoundaryCondition_y"] = &BoundaryCondition_y_i;
    this->D4_map["BoundaryCondition_y_i"] = &BoundaryCondition_y_i;

    this->D4_map["BoundaryCondition_x"] = &BoundaryCondition_x_i;
    this->D4_map["BoundaryCondition_x_i"] = &BoundaryCondition_x_i;

    this->D4_map["VolumeEmission"] = &VolumeEmission_f;
    this->D4_map["VolumeEmission_i"] = &VolumeEmission_i;
    this->D4_map["VolumeEmission_f"] = &VolumeEmission_f;

    this->D4_map["ScavengingCoefficient"] = &ScavengingCoefficient_f;
    this->D4_map["ScavengingCoefficient_i"] = &ScavengingCoefficient_i;
    this->D4_map["ScavengingCoefficient_f"] = &ScavengingCoefficient_f;

    /*** Fields species lists ***/

    this->field_species["InitialCondition"] = &species_list_ic;
    this->field_species["BoundaryCondition_z"] = &species_list_bc;
    this->field_species["BoundaryCondition_y"] = &species_list_bc;
    this->field_species["BoundaryCondition_x"] = &species_list_bc;
    this->field_species["DepositionVelocity"] = &species_list_dep;
    this->field_species["ScavengingCoefficient"] = &species_list_scav;
    this->field_species["SurfaceEmission"] = &species_list_surf_emis;
    this->field_species["VolumeEmission"] = &species_list_vol_emis;
  }


  //! Destructor.
  template<class T, class ClassAdvection, class ClassDiffusion>
  Polair3DTransport<T, ClassAdvection, ClassDiffusion>::~Polair3DTransport()
  {
  }


  ///////////////////
  // CONFIGURATION //
  ///////////////////


  //! Reads the configuration.
  /*! It reads the description of the domain, the simulation starting-date,
    species lists, options (especially which processes are included) and the
    paths to data input-files.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::ReadConfiguration()
  {

    BaseModel<T>::ReadConfiguration();

    /*** Options ***/

    this->config.SetSection("[domain]");
    this->config.PeekValue("Cartesian", option_cartesian);

    this->config.SetSection("[options]");
    this->config.PeekValue("With_advection",
			   this->option_process["with_advection"]);
    this->config.PeekValue("With_diffusion",
			   this->option_process["with_diffusion"]);
    this->config.PeekValue("With_air_density",
			   this->option_process["with_air_density"]);
    this->config.PeekValue("With_initial_condition",
			   this->option_process["with_initial_condition"]);
    this->config.PeekValue("With_boundary_condition",
			   this->option_process["with_boundary_condition"]);
    this->config.PeekValue("With_deposition",
			   this->option_process["with_deposition"]);
    this->config.PeekValue("With_point_emission",
			   this->option_process["with_point_emission"]);
    this->config.PeekValue("With_surface_emission",
			   this->option_process["with_surface_emission"]);
    this->config.PeekValue("With_volume_emission",
			   this->option_process["with_volume_emission"]);
    this->config.PeekValue("Scavenging_model", this->scavenging_model);
    this->scavenging_model = lower_case(this->scavenging_model);

    /*** Disables management of useless fields ***/

    // Pressure and temperature are only needed for air density
    // or microphysical model of scavenging.
    this->option_manage["temperature"] = this->option_manage["temperature"]
      && (this->option_process["with_air_density"]
          || this->scavenging_model == "microphysical");
    this->option_manage["pressure"] = this->option_manage["temperature"]
      && (this->option_process["with_air_density"]
          || this->scavenging_model == "microphysical");

    // Winds are only needed for advection.
    this->option_manage["horizontal_wind"]
      = this->option_manage["horizontal_wind"]
      && this->option_process["with_advection"];
    this->option_manage["vertical_wind"]
      = this->option_manage["vertical_wind"]
      && this->option_manage["horizontal_wind"];

    // Diffusion coefficients are only needed for diffusion.
    this->option_manage["horizontal_diffusion"]
      = this->option_manage["horizontal_diffusion"]
      && this->option_process["with_diffusion"];
    this->option_manage["vertical_diffusion"]
      = this->option_manage["vertical_diffusion"]
      && this->option_process["with_diffusion"];

    // Rain and cloud height are only needed for scavenging.
    this->option_manage["rain"] = this->option_manage["rain"]
      && this->scavenging_model != "none";
    this->option_manage["cloud_height"] = this->option_manage["cloud_height"]
      && this->scavenging_model != "none";

    /*** Input files ***/

    // The configuration-file path is the field "Data_description" in the main
    // configuration file.
    this->config.SetSection("[data]");
    string data_description_file = this->config.PeekValue("Data_description");
    // Opens the configuration file for input data.
    ConfigStream data_description_stream(data_description_file);

    // Meteorological files.
    this->input_files["meteo"].Read(data_description_file, "meteo");

    // Initial conditions files.
    if (this->option_process["with_initial_condition"])
      this->input_files["initial_condition"].ReadFiles(data_description_file,
						       "initial_condition");
    else
      this->input_files["initial_condition"].Empty();
    for (map<string, string>::iterator i
	   = this->input_files["initial_condition"].Begin();
	 i != this->input_files["initial_condition"].End(); i++)
      species_list_ic.push_back(i->first);
    Ns_ic = int(species_list_ic.size());

    // Boundary conditions files.
    if (this->option_process["with_boundary_condition"])
      this->input_files["boundary_condition"].Read(data_description_file,
						   "boundary_condition");
    else
      this->input_files["boundary_condition"].Empty();
    for (map<string, string>::iterator i
	   = this->input_files["boundary_condition"].Begin();
	 i != this->input_files["boundary_condition"].End(); i++)
      species_list_bc.push_back(i->first);
    Ns_bc = int(species_list_bc.size());

    // Deposition velocities files.
    if (this->option_process["with_deposition"])
      this->input_files["deposition_velocity"].Read(data_description_file,
						    "deposition");
    else
      this->input_files["deposition_velocity"].Empty();
    for (map<string, string>::iterator i
	   = this->input_files["deposition_velocity"].Begin();
	 i != this->input_files["deposition_velocity"].End(); i++)
      species_list_dep.push_back(i->first);
    Ns_dep = int(species_list_dep.size());

    // Scavenging.
    if (this->scavenging_model != "none")
      this->input_files["scavenging_coefficient"]
	.ReadFields(data_description_file, "scavenging");
    else
      this->input_files["scavenging_coefficient"].Empty();
    for (map<string, string>::iterator i
	   = this->input_files["scavenging_coefficient"].Begin();
	 i != this->input_files["scavenging_coefficient"].End(); i++)
      species_list_scav.push_back(i->first);
    Ns_scav = int(species_list_scav.size());

    // Point emissions.
    if (this->option_process["with_point_emission"])
      {
	string point_emission_file;
	data_description_stream.SetSection("[point_emission]");
	data_description_stream.PeekValue("file", point_emission_file);
	ConfigStream point_emission_stream(point_emission_file);

	// Parses the file that describes all point emissions.
	string type, str, line;
	T x, y, z;
	int index;
	int Nsource = 0;
	while (!point_emission_stream.IsEmpty())
	  {
	    line = point_emission_stream.GetLine();
	    // A new source should be added.
	    if (split(line)[0] == "[source]")
	      {
		point_emission_list.push_back(map<string, string>());
		// Emitted species.
		point_emission_stream.PeekValue("Species", str);
		point_emission_list[Nsource]["species"] = str;
		// Coordinates.
		point_emission_stream.PeekValue("Abscissa", x);
		point_emission_stream.PeekValue("Ordinate", y);
		point_emission_stream.PeekValue("Altitude", z);
		// Maps its coordinates to model grid indices.
		index = int((x - this->x_min + this->Delta_x / 2.)
			    / this->Delta_x);
		point_emission_list[Nsource]["i"] = to_str(index);
		index = int((y - this->y_min + this->Delta_y / 2.)
			    / this->Delta_y);
		point_emission_list[Nsource]["j"] = to_str(index);
		index = 0;
		while (index < this->Nz && z > this->LayerInterface(index+1))
		  index++;
		point_emission_list[Nsource]["k"] = to_str(index);
		// Source type (instantaneous or continuous).
		point_emission_stream.PeekValue("Type", type);
		point_emission_list[Nsource]["type"] = type;
		if (type == "puff")
		  // Instantaneous emission.
		  {
		    point_emission_stream.PeekValue("Date", str);
		    point_emission_list[Nsource]["date"] = str;
		    // Quantity emitted.
		    point_emission_stream.PeekValue("Quantity", str);
		    point_emission_list[Nsource]["quantity"] = str;
		  }
		else if (type == "continuous")
		  // Continuous emission.
		  {
		    point_emission_stream.PeekValue("Date_beg", str);
		    point_emission_list[Nsource]["date_beg"] = str;
		    point_emission_stream.PeekValue("Date_end", str);
		    point_emission_list[Nsource]["date_end"] = str;
		    // Emission rate.
		    point_emission_stream.PeekValue("Rate", str);
		    point_emission_list[Nsource]["rate"] = str;
		  }
		else
		  throw string("Unknown type of point emission: \"")
		    + type + "\".";
		  Nsource++;
	      }
	  }
      }

    // Surface emission files.
    if (this->option_process["with_surface_emission"])
      this->input_files["surface_emission"].Read(data_description_file,
						 "surface_emission");
    else
      this->input_files["surface_emission"].Empty();
    for (map<string, string>::iterator i
	   = this->input_files["surface_emission"].Begin();
	 i != this->input_files["surface_emission"].End(); i++)
      species_list_surf_emis.push_back(i->first);
    Ns_surf_emis = int(species_list_surf_emis.size());

    // Volume emission files.
    if (this->option_process["with_volume_emission"])
      this->input_files["volume_emission"].Read(data_description_file,
						"volume_emission");
    else
      this->input_files["volume_emission"].Empty();
    for (map<string, string>::iterator i
	   = this->input_files["volume_emission"].Begin();
	 i != this->input_files["volume_emission"].End(); i++)
      species_list_vol_emis.push_back(i->first);
    Ns_vol_emis = int(species_list_vol_emis.size());
    // Additional variable: number of levels.
    if (this->option_process["with_volume_emission"])
      {
	data_description_stream.SetSection("[volume_emission]");
	data_description_stream.PeekValue("Nz", Nz_vol_emis);
      }
    else
      Nz_vol_emis = 0;

    /*** Rest of the configuration ***/

    // Horizontal diffusion coefficient.
    this->config.PeekValue("Horizontal_diffusion", horizontal_diffusion);
    // Isotropy.
    this->config.PeekValue("Isotropic_diffusion", option_isotropic_diffusion);

    /*** Species data ***/

    ConfigStream species_data_stream(this->GetSpeciesFile());
    string species;

    if (this->scavenging_model == "microphysical")
      {
	// Henry constants in mol / L / atm.
	species_data_stream.SetSection("[henry]");
	while (!species_data_stream.IsEmpty())
	  {
	    species = species_data_stream.GetElement();
	    species_data_stream.GetNumber(henry_constant[species]);
	  }

	// Gas phase diffusivity constants in cm^2 / s.
	species_data_stream.SetSection("[diffusivity]");
	while (!species_data_stream.IsEmpty())
	  {
	    species = species_data_stream.GetElement();
	    species_data_stream.GetNumber(gas_phase_diffusivity[species]);
	  }
      }
    else if (this->scavenging_model == "belot")
      {
	// Initializes a and b coefficients.
	species_data_stream.SetSection("[belot]");
	species = species_data_stream.PeekElement();
	if (species == "all")
	  {
	    T a, b;
	    species_data_stream.GetNumber(a);
	    species_data_stream.GetNumber(b);
	    for (int i = 0; i < Ns_scav; i++)
	      {
		belot_constant_a[ScavengingName(i)] = a;
		belot_constant_b[ScavengingName(i)] = b;
	      }
	  }
	else
	  while (!species_data_stream.IsEmpty())
	    {
	      species = species_data_stream.GetElement();
	      species_data_stream.GetNumber(belot_constant_a[species]);
	      species_data_stream.GetNumber(belot_constant_b[species]);
	    }
      }
    else if (this->scavenging_model == "constant")
      {
	species_data_stream.SetSection("[scavenging_coefficient]");
	// Initializes rain threshold.
	species_data_stream.GetValue("Scavenging_rain_threshold",
				     scavenging_rain_threshold);
	// Constant scavenging coefficient in s^{-1}.
	species = species_data_stream.PeekElement();
	if (species == "all")
	  {
	    T scav_constant;
	    species_data_stream.GetNumber(scav_constant);
	    for (int i = 0; i < Ns_scav; i++)
	      scavenging_constant[ScavengingName(i)] = scav_constant;
	  }
	else
	  while (!species_data_stream.IsEmpty())
	    {
	      species = species_data_stream.GetElement();
	      if (!this->IsSpecies(species))
		throw string("ERROR! Species \"") + species
		  + string("\" found in section [scavenging_coefficient]")
		  + " is not a model species.";
	      species_data_stream.GetNumber(scavenging_constant[species]);
	    }
      }
  }

  //! Checks that the configuration is acceptable.
  /*! In case any inconsistency is found, an exception is thrown.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::CheckConfiguration()
  {
    if (this->option_manage["temperature"]
	&& this->input_files["meteo"]("Temperature").empty())
      throw "Temperature is needed but no input data file was provided.";
    if (this->option_manage["pressure"]
	&& this->input_files["meteo"]("Pressure").empty())
      throw "Pressure is needed but no input data file was provided.";

    if (this->option_manage["horizontal_wind"]
	&& this->input_files["meteo"]("ZonalWind").empty())
      throw "Zonal wind is needed but no input data file was provided.";
    if (this->option_manage["horizontal_wind"]
	&& this->input_files["meteo"]("MeridionalWind").empty())
      throw "Meridional wind is needed but no input data file was provided.";

    if (this->option_manage["rain"]
	&& this->input_files["meteo"]("Rain").empty())
      throw "Rain is needed but no input data file was provided.";
    if (this->option_manage["cloud_height"]
	&& this->input_files["meteo"]("CloudHeight").empty())
      throw "Cloud height is needed but no input data file was provided.";

    if (this->option_manage["vertical_diffusion"]
	&& this->input_files["meteo"]("VerticalDiffusion").empty())
      throw string("Vertical diffusion is needed but no input data file")
	+ " was provided.";

    if (!this->option_process["with_diffusion"])
      {
	if (this->option_process["with_deposition"])
	  throw "Deposition velocities cannot be included without diffusion.";
	if (this->option_process["with_surface_emission"])
	  throw "Surface emissions cannot be included without diffusion.";
      }
    
    if (this->scavenging_model == "microphysical")
      {
	// Checks if Henry constant is well initialized for each scavenged
	// species.
	for (int i = 0; i < Ns_scav; i++)
	  if (henry_constant.find(ScavengingName(i)) == henry_constant.end())
	    throw string("ERROR! Henry constant for species \"")
	      + ScavengingName(i)
	      + string("\" not found in section [henry].");
	
	// Checks if gas phase diffusivity constant is well initialized for
	// all scavenged species.
	for (int i = 0; i < Ns_scav; i++)
	  if (gas_phase_diffusivity.find(ScavengingName(i))
	      == gas_phase_diffusivity.end())
	    throw string("ERROR! Gas phase diffusivity constant for")
	      +string(" species \"")+ ScavengingName(i)
	      + string("\" not found in section [diffusivity].");
      }
    else if (this->scavenging_model == "belot")
      {
	// Checks if the coefficients a and b are well initialized for each
	// scavenged species.
	for (int i = 0; i < Ns_scav; i++)
	  if (belot_constant_a.find(ScavengingName(i))
	      == belot_constant_a.end()
	      || belot_constant_b.find(ScavengingName(i))
	      == belot_constant_b.end())
	    throw string("ERROR! Coefficient a or b for species \"")
	      + ScavengingName(i)
	      + string("\" not found in section [belot].");
      }
    else if (this->scavenging_model == "constant")
      {
	// Checks if the scavenging coefficient is well initialized for each
	// scavenged species.
	for (int i = 0; i < Ns_scav; i++)
	  if (scavenging_constant.find(ScavengingName(i))
	      == scavenging_constant.end())
	    throw string("ERROR! Scavenging coefficient for species \"")
	      + ScavengingName(i)
	      + string("\" not found in section [scavenging_coefficient].");
      }
  }


  //! Checks whether a species has initial conditions.
  /*!
    \param s species global index.
    \return True if the species has initial conditions, false otherwise.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  bool Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::HasInitialCondition(int s) const
  {
    return find(species_list_ic.begin(), species_list_ic.end(),
		this->GetSpeciesName(s)) != species_list_ic.end();
  }


  //! Checks whether a species has initial conditions.
  /*!
    \param name species name.
    \return True if the species has initial conditions, false otherwise.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  bool Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::HasInitialCondition(string name) const
  {
    return find(species_list_ic.begin(), species_list_ic.end(), name)
      != species_list_ic.end();
  }


  //! Returns the index in initial conditions of a given species.
  /*!
    \param s species global index.
    \return The species index in initial conditions.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InitialConditionIndex(int s) const
  {
    return find(species_list_ic.begin(), species_list_ic.end(),
		this->GetSpeciesName(s)) - species_list_ic.begin();
  }


  //! Returns the index in initial conditions of a given species.
  /*!
    \param name species name.
    \return The species index in initial conditions.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InitialConditionIndex(string name) const
  {
    return find(species_list_ic.begin(), species_list_ic.end(), name)
      - species_list_ic.begin();
  }


  //! Returns the name of a species with initial conditions.
  /*!
    \param s species index in initial conditions.
    \return The species name.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  string Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InitialConditionName(int s) const
  {
    return species_list_ic.at(s);
  }


  //! Returns the name of a species with initial conditions.
  /*!
    \param s species index in initial conditions.
    \return The species name.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InitialConditionGlobalIndex(int s) const
  {
    return this->GetSpeciesIndex(InitialConditionName(s));
  }


  //! Checks whether a species has boundary conditions.
  /*!
    \param s species global index.
    \return True if the species has boundary conditions, false otherwise.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  bool Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::HasBoundaryCondition(int s) const
  {
    return find(species_list_bc.begin(), species_list_bc.end(),
		this->GetSpeciesName(s)) != species_list_bc.end();
  }


  //! Checks whether a species has boundary conditions.
  /*!
    \param name species name.
    \return True if the species has boundary conditions, false otherwise.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  bool Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::HasBoundaryCondition(string name) const
  {
    return find(species_list_bc.begin(), species_list_bc.end(), name)
      != species_list_bc.end();
  }


  //! Returns the index in boundary conditions of a given species.
  /*!
    \param s species global index.
    \return The species index in boundary conditions.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::BoundaryConditionIndex(int s) const
  {
    return find(species_list_bc.begin(), species_list_bc.end(),
		this->GetSpeciesName(s)) - species_list_bc.begin();
  }


  //! Returns the index in boundary conditions of a given species.
  /*!
    \param name species name.
    \return The species index in boundary conditions.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::BoundaryConditionIndex(string name) const
  {
    return find(species_list_bc.begin(), species_list_bc.end(), name)
      - species_list_bc.begin();
  }


  //! Returns the name of a species with boundary conditions.
  /*!
    \param s species index in boundary conditions.
    \return The species name.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  string Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::BoundaryConditionName(int s) const
  {
    return species_list_bc.at(s);
  }


  //! Returns the name of a species with boundary conditions.
  /*!
    \param s species index in boundary conditions.
    \return The species name.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::BoundaryConditionGlobalIndex(int s) const
  {
    return this->GetSpeciesIndex(BoundaryConditionName(s));
  }


  //! Checks whether a species has deposition velocities.
  /*!
    \param s species global index.
    \return True if the species has deposition velocities, false otherwise.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  bool Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::HasDepositionVelocity(int s) const
  {
    return find(species_list_dep.begin(), species_list_dep.end(),
		this->GetSpeciesName(s)) != species_list_dep.end();
  }


  //! Checks whether a species has deposition velocities.
  /*!
    \param name species name.
    \return True if the species has deposition velocities, false otherwise.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  bool Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::HasDepositionVelocity(string name) const
  {
    return find(species_list_dep.begin(), species_list_dep.end(), name)
      != species_list_dep.end();
  }


  //! Returns the index in deposition velocities of a given species.
  /*!
    \param s species global index.
    \return The species index in deposition velocities.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::DepositionVelocityIndex(int s) const
  {
    return find(species_list_dep.begin(), species_list_dep.end(),
		this->GetSpeciesName(s)) - species_list_dep.begin();
  }


  //! Returns the index in deposition velocities of a given species.
  /*!
    \param name species name.
    \return The species index in deposition velocities.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::DepositionVelocityIndex(string name) const
  {
    return find(species_list_dep.begin(), species_list_dep.end(), name)
      - species_list_dep.begin();
  }


  //! Returns the name of a species with deposition velocities.
  /*!
    \param s species index in deposition velocities.
    \return The species name.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  string Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::DepositionVelocityName(int s) const
  {
    return species_list_dep.at(s);
  }


  //! Returns the name of a species with deposition velocities.
  /*!
    \param s species index in deposition velocities.
    \return The species name.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::DepositionVelocityGlobalIndex(int s) const
  {
    return this->GetSpeciesIndex(DepositionVelocityName(s));
  }


  //! Checks whether a species is scavenged.
  /*!
    \param s species global index.
    \return True if the species is scavenged, false otherwise.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  bool Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::HasScavenging(int s) const
  {
    return find(species_list_scav.begin(), species_list_scav.end(),
		this->GetSpeciesName(s)) != species_list_scav.end();
  }


  //! Checks whether a species is scavenged.
  /*!
    \param name species name.
    \return True if the species is scavenged, false otherwise.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  bool Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::HasScavenging(string name) const
  {
    return find(species_list_scav.begin(), species_list_scav.end(), name)
      != species_list_scav.end();
  }


  //! Returns the index in scavenged species of a given species.
  /*!
    \param s species global index.
    \return The species index in scavenged species.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::ScavengingIndex(int s) const
  {
    return find(species_list_scav.begin(), species_list_scav.end(),
		this->GetSpeciesName(s)) - species_list_scav.begin();
  }


  //! Returns the index in scavenged species of a given species.
  /*!
    \param name species name.
    \return The species index in scavenged species.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::ScavengingIndex(string name) const
  {
    return find(species_list_scav.begin(), species_list_scav.end(), name)
      - species_list_scav.begin();
  }


  //! Returns the name of a species that is scavenged.
  /*!
    \param s species index in scavenged species.
    \return The species name.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  string Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::ScavengingName(int s) const
  {
    return species_list_scav.at(s);
  }


  //! Returns the name of a species that is scavenged.
  /*!
    \param s species index in scavenged species.
    \return The species name.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::ScavengingGlobalIndex(int s) const
  {
    return this->GetSpeciesIndex(ScavengingName(s));
  }


  //! Checks whether a species has surface emissions.
  /*!
    \param s species global index.
    \return True if the species has surface emissions, false otherwise.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  bool Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::HasSurfaceEmission(int s) const
  {
    return find(species_list_surf_emis.begin(), species_list_surf_emis.end(),
		this->GetSpeciesName(s)) != species_list_surf_emis.end();
  }


  //! Checks whether a species has surface emissions.
  /*!
    \param name species name.
    \return True if the species has surface emissions, false otherwise.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  bool Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::HasSurfaceEmission(string name) const
  {
    return find(species_list_surf_emis.begin(), species_list_surf_emis.end(),
		name) != species_list_surf_emis.end();
  }


  //! Returns the index in surface emissions of a given species.
  /*!
    \param s species global index.
    \return The species index in surface emissions.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::SurfaceEmissionIndex(int s) const
  {
    return find(species_list_surf_emis.begin(), species_list_surf_emis.end(),
		this->GetSpeciesName(s)) - species_list_surf_emis.begin();
  }


  //! Returns the index in surface emissions of a given species.
  /*!
    \param name species name.
    \return The species index in surface emissions.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::SurfaceEmissionIndex(string name) const
  {
    return find(species_list_surf_emis.begin(), species_list_surf_emis.end(),
		name) - species_list_surf_emis.begin();
  }


  //! Returns the name of a species with surface emissions.
  /*!
    \param s species index in surface emissions.
    \return The species name.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  string Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::SurfaceEmissionName(int s) const
  {
    return species_list_surf_emis.at(s);
  }


  //! Returns the name of a species with surface emissions.
  /*!
    \param s species index in surface emissions.
    \return The species name.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::SurfaceEmissionGlobalIndex(int s) const
  {
    return this->GetSpeciesIndex(SurfaceEmissionName(s));
  }


  //! Checks whether a species has volume emissions.
  /*!
    \param s species global index.
    \return True if the species has volume emissions, false otherwise.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  bool Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::HasVolumeEmission(int s) const
  {
    return find(species_list_vol_emis.begin(), species_list_vol_emis.end(),
		this->GetSpeciesName(s)) != species_list_vol_emis.end();
  }


  //! Checks whether a species has volume emissions.
  /*!
    \param name species name.
    \return True if the species has volume emissions, false otherwise.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  bool Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::HasVolumeEmission(string name) const
  {
    return find(species_list_vol_emis.begin(), species_list_vol_emis.end(),
		name) != species_list_vol_emis.end();
  }


  //! Returns the index in volume emissions of a given species.
  /*!
    \param s species global index.
    \return The species index in volume emissions.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::VolumeEmissionIndex(int s) const
  {
    return find(species_list_vol_emis.begin(), species_list_vol_emis.end(),
		this->GetSpeciesName(s)) - species_list_vol_emis.begin();
  }


  //! Returns the index in volume emissions of a given species.
  /*!
    \param name species name.
    \return The species index in volume emissions.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::VolumeEmissionIndex(string name) const
  {
    return find(species_list_vol_emis.begin(), species_list_vol_emis.end(),
		name) - species_list_vol_emis.begin();
  }


  //! Returns the name of a species with volume emissions.
  /*!
    \param s species index in volume emissions.
    \return The species name.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  string Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::VolumeEmissionName(int s) const
  {
    return species_list_vol_emis.at(s);
  }


  //! Returns the name of a species with volume emissions.
  /*!
    \param s species index in volume emissions.
    \return The species name.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  int Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::VolumeEmissionGlobalIndex(int s) const
  {
    return this->GetSpeciesIndex(VolumeEmissionName(s));
  }


  /////////////////////
  // INITIALIZATIONS //
  /////////////////////


  //! Allocates memory.
  /*! Allocates grids and fields.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>::Allocate()
  {

    /*** Mesh dimensions ***/

    CellWidth_x.resize(this->Nx);
    CellWidth_y.resize(this->Ny);
    CellWidth_z.resize(this->Nz);

    CellCenterDistance_x.resize(this->Nx - 1);
    CellCenterDistance_y.resize(this->Ny - 1);
    CellCenterDistance_z.resize(this->Nz - 1);
    
    /*** Boundary conditions ***/

    GridS_bc = RegularGrid<T>(Ns_bc);

    GridY4D_interf_bc = RegularGrid<T>(this->y_min - this->Delta_y,
				       T(this->Ny+1) * this->Delta_y, 2);
    GridX4D_interf_bc = RegularGrid<T>(this->x_min - this->Delta_x,
				       T(this->Nx+1) * this->Delta_x, 2);

    // Along Z.
    BoundaryCondition_z_i.Resize(GridS_bc, this->GridY3D, this->GridX3D);
    FileBoundaryCondition_z_i.Resize(GridS_bc, this->GridY3D, this->GridX3D);
    FileBoundaryCondition_z_f.Resize(GridS_bc, this->GridY3D, this->GridX3D);

    // Along Y.
    BoundaryCondition_y_i.Resize(GridS_bc, this->GridZ4D,
				 GridY4D_interf_bc, this->GridX4D);
    FileBoundaryCondition_y_i.Resize(GridS_bc, this->GridZ4D,
				     GridY4D_interf_bc, this->GridX4D);
    FileBoundaryCondition_y_f.Resize(GridS_bc, this->GridZ4D,
				     GridY4D_interf_bc, this->GridX4D);

    // Along X.
    BoundaryCondition_x_i.Resize(GridS_bc, this->GridZ4D,
				 this->GridY4D, GridX4D_interf_bc);
    FileBoundaryCondition_x_i.Resize(GridS_bc, this->GridZ4D,
				     this->GridY4D, GridX4D_interf_bc);
    FileBoundaryCondition_x_f.Resize(GridS_bc, this->GridZ4D,
				     this->GridY4D, GridX4D_interf_bc);

    /*** Temperature ***/

    Temperature_i.Resize(this->GridZ3D, this->GridY3D, this->GridX3D);
    Temperature_f.Resize(this->GridZ3D, this->GridY3D, this->GridX3D);
    FileTemperature_i.Resize(this->GridZ3D, this->GridY3D, this->GridX3D);
    FileTemperature_f.Resize(this->GridZ3D, this->GridY3D, this->GridX3D);

    /*** Pressure ***/

    Pressure_i.Resize(this->GridZ3D, this->GridY3D, this->GridX3D);
    Pressure_f.Resize(this->GridZ3D, this->GridY3D, this->GridX3D);
    FilePressure_i.Resize(this->GridZ3D, this->GridY3D, this->GridX3D);
    FilePressure_f.Resize(this->GridZ3D, this->GridY3D, this->GridX3D);

    /*** Air density ***/

    AirDensity_i.Resize(this->GridZ3D, this->GridY3D, this->GridX3D);
    AirDensity_f.Resize(this->GridZ3D, this->GridY3D, this->GridX3D);

    AirDensity_interf_z_i.Resize(this->GridZ3D_interf, this->GridY3D,
				 this->GridX3D);
    AirDensity_interf_z_f.Resize(this->GridZ3D_interf, this->GridY3D,
				 this->GridX3D);
    AirDensity_interf_y_i.Resize(this->GridZ3D, this->GridY3D_interf,
				 this->GridX3D);
    AirDensity_interf_y_f.Resize(this->GridZ3D, this->GridY3D_interf,
				 this->GridX3D);
    AirDensity_interf_x_i.Resize(this->GridZ3D, this->GridY3D,
				 this->GridX3D_interf);
    AirDensity_interf_x_f.Resize(this->GridZ3D, this->GridY3D,
				 this->GridX3D_interf);

    /*** Rain ***/

    if (this->option_manage["rain"])
      {
	Rain_i.Resize(this->GridY2D, this->GridX2D);
	FileRain_i.Resize(this->GridY2D, this->GridX2D);
	FileRain_f.Resize(this->GridY2D, this->GridX2D);
      }

    /*** Cloud height ***/

    if (this->option_manage["cloud_height"])
      {
	CloudHeight_i.Resize(this->GridY2D, this->GridX2D);
	CloudHeight_f.Resize(this->GridY2D, this->GridX2D);
	FileCloudHeight_i.Resize(this->GridY2D, this->GridX2D);
	FileCloudHeight_f.Resize(this->GridY2D, this->GridX2D);
      }

    /*** Scavenging coefficients ***/

    GridS_scav = RegularGrid<T>(Ns_scav);

    ScavengingCoefficient_i.Resize(GridS_scav, this->GridZ4D,
				   this->GridY4D, this->GridX4D);
    ScavengingCoefficient_f.Resize(GridS_scav, this->GridZ4D,
				   this->GridY4D, this->GridX4D);


    /*** Winds ***/

    ZonalWind_i.Resize(this->GridZ3D, this->GridY3D, this->GridX3D_interf);
    FileZonalWind_i.Resize(this->GridZ3D, this->GridY3D,
			   this->GridX3D_interf);
    FileZonalWind_f.Resize(this->GridZ3D, this->GridY3D,
			   this->GridX3D_interf);

    MeridionalWind_i.Resize(this->GridZ3D, this->GridY3D_interf,
			    this->GridX3D);
    FileMeridionalWind_i.Resize(this->GridZ3D, this->GridY3D_interf,
				this->GridX3D);
    FileMeridionalWind_f.Resize(this->GridZ3D, this->GridY3D_interf,
				this->GridX3D);

    VerticalWind_i.Resize(this->GridZ3D_interf, this->GridY3D, this->GridX3D);

    /*** Diffusion coefficients ***/

    ZonalDiffusionCoefficient_i.Resize(this->GridZ3D, this->GridY3D,
				       this->GridX3D_interf);
    MeridionalDiffusionCoefficient_i.Resize(this->GridZ3D,
					    this->GridY3D_interf,
					    this->GridX3D);

    VerticalDiffusionCoefficient_i.Resize(this->GridZ3D_interf, this->GridY3D,
					  this->GridX3D);
    VerticalDiffusionCoefficient_f.Resize(this->GridZ3D_interf, this->GridY3D,
					  this->GridX3D);
    FileVerticalDiffusionCoefficient_i.Resize(this->GridZ3D_interf,
					      this->GridY3D, this->GridX3D);
    FileVerticalDiffusionCoefficient_f.Resize(this->GridZ3D_interf,
					      this->GridY3D, this->GridX3D);
    
    /*** Deposition velocities ***/

    GridS_dep = RegularGrid<T>(Ns_dep);

    DepositionVelocity_i.Resize(GridS_dep, this->GridY3D, this->GridX3D);
    DepositionVelocity_f.Resize(GridS_dep, this->GridY3D, this->GridX3D);
    FileDepositionVelocity_i.Resize(GridS_dep, this->GridY3D, this->GridX3D);
    FileDepositionVelocity_f.Resize(GridS_dep, this->GridY3D, this->GridX3D);

    /*** Surface emissions ***/

    GridS_surf_emis = RegularGrid<T>(Ns_surf_emis);

    SurfaceEmission_i.Resize(GridS_surf_emis, this->GridY3D, this->GridX3D);
    SurfaceEmission_f.Resize(GridS_surf_emis, this->GridY3D, this->GridX3D);
    FileSurfaceEmission_i.Resize(GridS_surf_emis,
				 this->GridY3D, this->GridX3D);
    FileSurfaceEmission_f.Resize(GridS_surf_emis,
				 this->GridY3D, this->GridX3D);

    /*** Volume emissions ***/

    GridS_vol_emis = RegularGrid<T>(Ns_vol_emis);
    GridZ_vol_emis = RegularGrid<T>(Nz_vol_emis);

    VolumeEmission_i.Resize(GridS_vol_emis, GridZ_vol_emis,
			    this->GridY4D, this->GridX4D);
    VolumeEmission_f.Resize(GridS_vol_emis, GridZ_vol_emis,
			    this->GridY4D, this->GridX4D);
    FileVolumeEmission_i.Resize(GridS_vol_emis, GridZ_vol_emis,
				this->GridY4D, this->GridX4D);
    FileVolumeEmission_f.Resize(GridS_vol_emis, GridZ_vol_emis,
				this->GridY4D, this->GridX4D);
  }


  //! Model initialization.
  /*! It reads the configuration, allocates memory and reads the values of the
    fields at the beginning of the simulation.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>::Init()
  {
    BaseModel<T>::Init();

    Polair3DTransport<T, ClassAdvection, ClassDiffusion>::Allocate();

    /*** Mesh dimensions ***/

    if (option_cartesian)
      {
	CellWidth_x = this->Delta_x;
	CellWidth_y = this->Delta_y;
      }
    else
      ComputeCellWidth(this->Delta_x, this->GridY3D_interf.GetArray(),
		       CellWidth_x, CellWidth_y);
    for (int k = 0; k < this->Nz; k++)
      CellWidth_z(k) = this->GridZ3D_interf(k + 1) - this->GridZ3D_interf(k);

    if (option_cartesian)
      {
	CellCenterDistance_x = this->Delta_x;
	CellCenterDistance_y = this->Delta_y;
      }
    else
      ComputeCellCenterDistance(this->Delta_x,
				this->GridY3D_interf.GetArray(),
				CellCenterDistance_x, CellCenterDistance_y);
    for (int k = 0; k < this->Nz - 1; k++)
      CellCenterDistance_z(k) = this->GridZ3D(k + 1) - this->GridZ3D(k);

    /*** Initial conditions ***/

    if (this->option_manage["initial_condition"])
      {
	this->Concentration.SetZero();
	
	for (int i = 0; i < Ns_ic; i++)
	  {
	    string filename
	      = this->input_files["initial_condition"](species_list_ic[i]);
	    int index = this->GetSpeciesIndex(species_list_ic[i]);
	    Data<T, 3>
	      Concentration_tmp(&(this->Concentration)(index, 0, 0, 0),
				shape(this->Nz, this->Ny, this->Nx));
	    if (is_num(filename))
	      Concentration_tmp.Fill(to_num<T>(filename));
	    else
	      FormatBinary<float>().Read(filename, Concentration_tmp);
	  }
      }

    /*** Input data ***/

    Polair3DTransport<T, ClassAdvection, ClassDiffusion>
      ::SetDate(this->Date_min);

    /*** Numerical schemes ***/

    Advection_.Init(*this);
    Diffusion_.Init(*this);

  }
  
  
  //! Model initialization for each step.
  /*! It reads on file the data that are is needed for the current step.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>::InitStep()
  {

    /*** Temperature ***/

    if (this->option_manage["temperature"])
      this->UpdateData("meteo", "Temperature", FileTemperature_i,
		       FileTemperature_f, Temperature_i, Temperature_f);

    /*** Pressure ***/

    if (this->option_manage["pressure"])
      this->UpdateData("meteo", "Pressure", FilePressure_i,
		       FilePressure_f, Pressure_i, Pressure_f);

    /*** Air density ***/

    if (this->option_process["with_air_density"])
      {
	AirDensity_i.GetArray() = AirDensity_f.GetArray();
	ComputeAirDensity(Temperature_f, Pressure_f, AirDensity_f);
	
	AirDensity_interf_z_i.GetArray() = AirDensity_interf_z_f.GetArray();
	AirDensity_interf_y_i.GetArray() = AirDensity_interf_y_f.GetArray();
	AirDensity_interf_x_i.GetArray() = AirDensity_interf_x_f.GetArray();
	InterpolateInterface_z(CellCenterDistance_z, CellWidth_z,
			       AirDensity_f, AirDensity_interf_z_f);
	InterpolateInterface_y(CellCenterDistance_y, CellWidth_y,
			       AirDensity_f, AirDensity_interf_y_f);
	InterpolateInterface_x(CellCenterDistance_x, CellWidth_x,
			       AirDensity_f, AirDensity_interf_x_f);
      }

    /*** Winds ***/

    if (this->option_manage["horizontal_wind"])
      {
	this->UpdateData("meteo", "ZonalWind", FileZonalWind_i,
			 FileZonalWind_f, ZonalWind_i);
	if (!option_cartesian)
	  TransformZonalWind(ZonalWind_i);
	
	this->UpdateData("meteo", "MeridionalWind", FileMeridionalWind_i,
			 FileMeridionalWind_f, MeridionalWind_i);
	if (!option_cartesian)
	  TransformMeridionalWind(MeridionalWind_i);
      }

    if (this->option_manage["vertical_wind"])
      if (this->option_process["with_air_density"])
	ComputeVerticalWind(CellWidth_x, CellWidth_y, CellWidth_z,
			    AirDensity_interf_x_i, ZonalWind_i,
			    AirDensity_interf_y_i, MeridionalWind_i,
			    AirDensity_interf_z_i, VerticalWind_i);
      else
	ComputeVerticalWind(CellWidth_x, CellWidth_y, CellWidth_z,
			    ZonalWind_i, MeridionalWind_i, VerticalWind_i);

    /*** Rain, cloud height and scavenging coefficients ***/
    
    if (this->option_manage["rain"])
      this->UpdateData("meteo", "Rain", FileRain_i, FileRain_f,
		       Rain_i, false);
    
    if (this->option_manage["cloud_height"])
      this->UpdateData("meteo", "CloudHeight", FileCloudHeight_i,
		       FileCloudHeight_f, CloudHeight_i, CloudHeight_f);
    
    if (this->option_manage["scavenging_coefficient"]
	&& this->scavenging_model != "none")
      {
	ScavengingCoefficient_i.GetArray()
	  = ScavengingCoefficient_f.GetArray();
	if (this->scavenging_model == "microphysical")
	  InitScavengingCoefficient(Temperature_f, Pressure_f,
				    CloudHeight_f, Rain_i,
				    ScavengingCoefficient_f);
	else if (this->scavenging_model == "belot")
	  InitScavengingCoefficient(Rain_i, ScavengingCoefficient_f);
	else if (this->scavenging_model == "constant")
	  InitScavengingCoefficient(ScavengingCoefficient_f);
      }

    /*** Diffusion coefficients ***/
    
    if (this->option_manage["vertical_diffusion"])
      {
	this->UpdateData("meteo", "VerticalDiffusion",
			 FileVerticalDiffusionCoefficient_i,
			 FileVerticalDiffusionCoefficient_f,
			 VerticalDiffusionCoefficient_i,
			 VerticalDiffusionCoefficient_f);

	// Computes rho * Kz.
	if (this->option_process["with_air_density"])
	  VerticalDiffusionCoefficient_f.GetArray() =
	    AirDensity_interf_z_f.GetArray()
	    * VerticalDiffusionCoefficient_f.GetArray();
      }
    
    if (this->option_manage["horizontal_diffusion"]
	&& option_isotropic_diffusion)
      {
	LinearInterpolationRegular(VerticalDiffusionCoefficient_i,
				   ZonalDiffusionCoefficient_i);
	ZonalDiffusionCoefficient_i.ThresholdMin(0.);
	LinearInterpolationRegular(VerticalDiffusionCoefficient_i,
				   MeridionalDiffusionCoefficient_i);
	MeridionalDiffusionCoefficient_i.ThresholdMin(0.);

	if (!option_cartesian)
	  {
	    TransformZonalDiffusion(this->GridY3D_interf.GetArray(),
				    ZonalDiffusionCoefficient_i);
	    TransformMeridionalDiffusion(MeridionalDiffusionCoefficient_i);
	  }
      }

    /*** Boundary conditions ***/

    if (this->option_manage["boundary_condition"])
      for (int i = 0; i < Ns_bc; i++)
	{
	  string filename
	    = this->input_files["boundary_condition"](species_list_bc[i]);
	  Date date = this->input_files["boundary_condition"].GetDateMin();
	  T Delta_t = this->input_files["boundary_condition"].GetDelta_t();
	  this->UpdateData(find_replace(filename, "&c", "z"), date, Delta_t,
			   FileBoundaryCondition_z_i,
			   FileBoundaryCondition_z_f,
			   i, BoundaryCondition_z_i);
	  this->UpdateData(find_replace(filename, "&c", "y"), date, Delta_t,
			   FileBoundaryCondition_y_i,
			   FileBoundaryCondition_y_f,
			   i, BoundaryCondition_y_i);
	  this->UpdateData(find_replace(filename, "&c", "x"), date, Delta_t,
			   FileBoundaryCondition_x_i,
			   FileBoundaryCondition_x_f,
			   i, BoundaryCondition_x_i);
	}

    /*** Deposition velocities ***/

    if (this->option_manage["deposition_velocity"])
      for (int i = 0; i < Ns_dep; i++)
	this->UpdateData("deposition_velocity", species_list_dep[i],
			 FileDepositionVelocity_i, FileDepositionVelocity_f,
			 i, DepositionVelocity_i, DepositionVelocity_f);

    /*** Surface emissions ***/

    if (this->option_manage["surface_emission"])
      for (int i = 0; i < Ns_surf_emis; i++)
	this->UpdateData("surface_emission", species_list_surf_emis[i],
			 FileSurfaceEmission_i, FileSurfaceEmission_f, i,
			 SurfaceEmission_i, SurfaceEmission_f);

    /*** Volume emissions ***/

    if (this->option_manage["volume_emission"])
      for (int i = 0; i < Ns_vol_emis; i++)
	this->UpdateData("volume_emission", species_list_vol_emis[i],
			 FileVolumeEmission_i, FileVolumeEmission_f, i,
			 VolumeEmission_i, VolumeEmission_f);
  }


  //! Initializes scavenging coefficients.
  /*! Assigns a constant value to below-cloud scavenging coefficients of a
    reversibly soluble gas for each species.
    \param ScavengingCoefficient_ (output) the scavenging coefficients
    (s^{-1}).
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InitScavengingCoefficient(Data<T, 4>& ScavengingCoefficient_)
  {
    int Ns_scav = ScavengingCoefficient_.GetLength(0);
    int Nz = ScavengingCoefficient_.GetLength(1);
    int Ny = ScavengingCoefficient_.GetLength(2);
    int Nx = ScavengingCoefficient_.GetLength(3);

    for (int s = 0; s < Ns_scav; s++)
      {
	T scavenging_value = this->scavenging_constant[ScavengingName(s)];
	for (int k = 0; k < Nz; k++)
	  for (int j = 0; j < Ny; j++)
	    for (int i = 0; i < Nx; i++)
	      ScavengingCoefficient_(s, k, j, i) = scavenging_value;
      }
  }

  //! Initializes scavenging coefficients.
  /*! Computes below-cloud scavenging coefficients of a reversibly soluble
    gas.
    \param Rain_ rain intensity (mm/h).
    \param ScavengingCoefficient_ (output) the scavenging coefficients
    (s^{-1}).
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InitScavengingCoefficient(Data<T, 2>& Rain_,
			      Data<T, 4>& ScavengingCoefficient_)
  {
    int Ns_scav = ScavengingCoefficient_.GetLength(0);
    int Nz = ScavengingCoefficient_.GetLength(1);
    int Ny = ScavengingCoefficient_.GetLength(2);
    int Nx = ScavengingCoefficient_.GetLength(3);

    ScavengingCoefficient_.Fill(0.);
    for (int s = 0; s < Ns_scav; s++)
      {
	T cste_a = this->belot_constant_a[ScavengingName(s)];
	T cste_b = this->belot_constant_b[ScavengingName(s)];
	for (int j = 0; j < Ny; j++)
	  for (int i = 0; i < Nx; i++)
	    if (Rain_(j, i) > 0.)
	      {
		T scavenging_value = cste_a * pow(Rain_(j, i), cste_b);
		for (int k = 0; k < Nz; k++)
		  ScavengingCoefficient_(s, k, j, i) = scavenging_value;
	      }
      }
  }


  //! Initializes scavenging coefficients.
  /*! Computes below-cloud scavenging coefficients of a reversibly soluble
    gas.
    \param Temperature_ temperature (K).
    \param Pressure_ pressure (Pa).
    \param CloudHeight_ cloud basis height (m).
    \param Rain_ rain intensity (mm/h).
    \param ScavengingCoefficient_ (output) the scavenging coefficients
    (s^{-1}).
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InitScavengingCoefficient(Data<T, 3>& Temperature_,
			      Data<T, 3>& Pressure_,
			      Data<T, 2>& CloudHeight_,
			      Data<T, 2>& Rain_,
			      Data<T, 4>& ScavengingCoefficient_)
  {
    int Nz = ScavengingCoefficient_.GetLength(1);
    int Ny = ScavengingCoefficient_.GetLength(2);
    int Nx = ScavengingCoefficient_.GetLength(3);

    // Henry constants of scavenged species.
    Array<T, 1> henry_constant_(Ns_scav);
    // Gas phase diffusivities of scavenged species.
    Array<T, 1> gas_phase_diffusivity_(Ns_scav);

    // Extracts species data for scavenged species only.
    for (int s = 0; s < this->Ns_scav; s++)
      {
	henry_constant_(s) = henry_constant[ScavengingName(s)];
	gas_phase_diffusivity_(s) = gas_phase_diffusivity[ScavengingName(s)];
      }
    
    int one = 1;
    _compute_scavenging_coefficient(&Nx, &Ny, &Nz, &Ns_scav,
				    &one, &one, &one,
				    this->GridZ4D.GetArray().data(),
				    gas_phase_diffusivity_.data(),
				    henry_constant_.data(),
				    Temperature_.GetData(),
				    Pressure_.GetData(),
				    Rain_.GetData(),
				    CloudHeight_.GetData(),
				    ScavengingCoefficient_.GetData());
  }
  

  //! Moves the model to a given date.
  /*! This method prepares the model for a time integration at a given
    date. It should be called before InitStep and Forward.
    \param date date.
  */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::SetDate(Date date)
  {
    BaseModel<T>::SetDate(date);
    Polair3DTransport<T, ClassAdvection, ClassDiffusion>::InitAllData();
  }


  ///////////////////////////
  // NUMERICAL INTEGRATION //
  ///////////////////////////


  //! Performs one advection step.
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>::Advection()
  {
    Advection_.Forward(*this);
  }
  
  
  //! Performs one diffusion step.
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>::Diffusion()
  {
    Diffusion_.Forward(*this);
  }


  //! Performs the time integration of point emissions.
  template<class T, class ClassAdvection, class ClassDiffusion>
  void
  Polair3DTransport<T, ClassAdvection, ClassDiffusion>::PointEmission()
  {
    int e, i, j, k, s;
    int Nsource(point_emission_list.size());
    T quantity;
    // Loop over all point sources.
    for (e = 0; e < Nsource; e++)
      {
	i = convert<int>(point_emission_list[e]["i"]);
	j = convert<int>(point_emission_list[e]["j"]);
	k = convert<int>(point_emission_list[e]["k"]);
	s = this->GetSpeciesIndex(point_emission_list[e]["species"]);
	if (point_emission_list[e]["type"] == "puff")
	  {
	    Date date(point_emission_list[e]["date"]);
	    if (this->current_date <= date && date < this->next_date)
	      this->Concentration(s, k, j, i) +=
		convert<T>(point_emission_list[e]["quantity"])
		/ (CellWidth_x(i) * CellWidth_y(j) * CellWidth_z(k));
	  }
	else if (point_emission_list[e]["type"] == "continuous")
	  {
	    Date date_beg(point_emission_list[e]["date_beg"]);
	    Date date_end(point_emission_list[e]["date_end"]);
	    if (this->next_date > date_beg && this->current_date < date_end)
	      {
		quantity = this->Delta_t
		  - max(0., date_beg.GetSecondsFrom(this->current_date))
		  - max(0., this->next_date.GetSecondsFrom(date_end));
		quantity *= convert<T>(point_emission_list[e]["rate"]);
		this->Concentration(s, k, j, i) += quantity
		  / (CellWidth_x(i) * CellWidth_y(j) * CellWidth_z(k));
	      }
	  }
      }
  }


  //! Performs one step forward.
  /*! It performs one advection step, then one diffusion step and finally
    adds volume emissions. These three processes are split (operator
    splitting).
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>::Forward()
  {
    if (this->option_process["with_advection"])
      Advection();
    if (this->option_process["with_diffusion"])
      Diffusion();
    if (this->option_process["with_point_emission"])
      PointEmission();
    if (this->option_process["with_volume_emission"])
      for (int s = 0; s < this->Ns; s++)
	{
	  int k, j, i, emis_s;
	  if (HasVolumeEmission(s))
	    {
	      emis_s = VolumeEmissionIndex(s);
	      for (k = 0; k < Nz_vol_emis; k++)
		for (j = 0; j < this->Ny; j++)
		  for (i = 0; i < this->Nx; i++)
		    this->Concentration(s, k, j, i) +=
		      this->Delta_t * VolumeEmission_i(emis_s, k, j, i);
	    }
	}
    for (int s = 0; s < this->Ns; s++)
      if (HasScavenging(s) && this->scavenging_model != "none")
	{
	  int scav_s = this->ScavengingIndex(s);
	  for (int j = 0; j < this->Ny; j++)
	    for (int i = 0; i < this->Nx; i++)
	      if (this->Rain_i(j, i) > 0.)
		{
		  T CloudHeight_mean = 0.5 *
		    (this->CloudHeight_i(j, i) + this->CloudHeight_f(j, i));
		  for (int k = 0; k < this->Nz; k++)
		    if (CloudHeight_mean < this->GridZ4D(k))
		      this->Concentration(s, k, j, i) *=
			exp(- 0.5 *
			    (this->ScavengingCoefficient_i(scav_s, k, j, i)
			     + this->ScavengingCoefficient_f(scav_s, k, j, i))
			    * this->Delta_t);
		}
	}
    this->AddTime(this->Delta_t);
    this->step++;
  }


  ///////////////////
  // OTHER METHODS //
  ///////////////////


  //! Computes cells widths (in meters).
  /*!
    \param Delta_x_ step along x in degrees.
    \param GridY_interf_ interface coordinates along y in degrees.
    \param CellWidth_x_ (output) cells widths along x in meters.
    \param CellWidth_y_ (output) cells widths along y in meters.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::ComputeCellWidth(T Delta_x_, Array<T, 1>& GridY_interf_,
		     Array<T, 1>& CellWidth_x_, Array<T, 1>& CellWidth_y_)
  {
    const T earth_radius = 6371229.;
    const T pi(3.14159265358979323846264);

    for (int i = 0; i < this->Nx; i++)
      CellWidth_x_(i) = earth_radius * Delta_x_ * pi / 180.;
    
    Array<T, 1> GridY_interf_trans(this->Ny + 1);
    for (int j = 0; j < this->Ny + 1; j++)
      GridY_interf_trans(j)
	= earth_radius * sin(GridY_interf_(j) * pi / 180.);

    for (int j = 0; j < this->Ny; j++)
      CellWidth_y_(j) = GridY_interf_trans(j + 1) - GridY_interf_trans(j);
  }
  
  
  //! Computes the distances between cells centers (in meters).
  /*!
    \param Delta_x_ step along x in degrees.
    \param GridY_interf_ interface coordinates along y in degrees.
    \param CellCenterDistance_x_ (output) distances between cells centers
    along x in meters.
    \param CellCenterDistance_y_ (output) distances between cells centers
    along y in meters.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::ComputeCellCenterDistance(T Delta_x_, Array<T, 1>& GridY_interf_,
			      Array<T, 1>& CellCenterDistance_x_,
			      Array<T, 1>& CellCenterDistance_y_)
  {
    const T earth_radius = 6371229.;
    const T pi(3.14159265358979323846264);

    for (int i = 0; i < this->Nx - 1; i++)
      CellCenterDistance_x_(i) = earth_radius * Delta_x_ * pi / 180.;
    
    Array<T, 1> GridY_interf_trans(this->Ny + 1), GridY_trans(this->Ny);
    for (int j = 0; j < this->Ny + 1; j++)
      GridY_interf_trans(j)
	= earth_radius * sin(GridY_interf_(j) * pi / 180.);
    for (int j = 0; j < this->Ny; j++)
      GridY_trans(j)
	= (GridY_interf_trans(j) + GridY_interf_trans(j + 1)) / 2.;
    
    for (int j = 0; j < this->Ny - 1; j++)
      CellCenterDistance_y_(j) = GridY_trans(j + 1) - GridY_trans(j);
  }
  

  /*! \brief Transforms the zonal wind to ease the numerical integration of
    advection. */
  /*! Formula: ZonalWind = ZonalWind / cos(latitude).
    \param ZonalWind (input/output) zonal wind.
    \note Coordinates associated with ZonalWind must be in degrees.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::TransformZonalWind(Data<T, 3>& ZonalWind)
  {
    int i, j, k;
    
    const T pi(3.14159265358979323846264);
    const T ratio = pi / 180.;
    
    int Nx = ZonalWind.GetLength(2);
    int Ny = ZonalWind.GetLength(1);
    int Nz = ZonalWind.GetLength(0);
    
    for (k=0; k<Nz; k++)
      for (j=0; j<Ny; j++)
	for (i=0; i<Nx; i++)
	  ZonalWind(k, j, i) /= cos(ZonalWind[1].Value(k, j, i)
				    * ratio);
  }
  
  
  /*! \brief Transforms the meridional wind to ease the numerical integration
    of advection. */
  /*! Formula: MeridionalWind = MeridionalWind * cos(latitude).
    \param MeridionalWind (input/output) meridional wind.
    \note Coordinates associated with MeridionalWind must be in degrees.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::TransformMeridionalWind(Data<T, 3>& MeridionalWind)
  {
    
    int i, j, k;
    
    const T pi(3.14159265358979323846264);
    const T ratio = pi / 180.;
    
    int Nx = MeridionalWind.GetLength(2);
    int Ny = MeridionalWind.GetLength(1);
    int Nz = MeridionalWind.GetLength(0);
    
    for (k=0; k<Nz; k++)
      for (j=0; j<Ny; j++)
	for (i=0; i<Nx; i++)
	  MeridionalWind(k, j, i) *= cos(MeridionalWind[1].Value(k, j, i)
					 * ratio);
  }


  /*! \brief Transforms the zonal diffusion coefficients to ease the numerical
    integration of diffusion. */
  /*!
    \param GridY_interf_ interfaces coordinates (in degrees) along y.
    \param ZonalDiffusion_ (input/output) zonal diffusion coefficients.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::TransformZonalDiffusion(Array<T, 1>& GridY_interf_,
			    Data<T, 3>& ZonalDiffusion_)
  {
    int i, j, k;
    
    const T earth_radius = 6371229.;
    const T pi(3.14159265358979323846264);
    
    int Nx = ZonalDiffusion_.GetLength(2) - 1;
    int Ny = ZonalDiffusion_.GetLength(1);
    int Nz = ZonalDiffusion_.GetLength(0);
    
    Array<T, 1> GridY_interf_trans(Ny + 1), GridY_trans(Ny);
    for (int j = 0; j < Ny + 1; j++)
      GridY_interf_trans(j)
	= earth_radius * sin(GridY_interf_(j) * pi / 180.);
    for (int j = 0; j < Ny; j++)
      GridY_trans(j)
	= (GridY_interf_trans(j) + GridY_interf_trans(j + 1)) / 2.;

    Array<T, 1> factor(Ny);
    for (int j = 0; j < Ny; j++)
      factor(j) = 1. - GridY_trans(j) * GridY_trans(j)
	/ (earth_radius * earth_radius);

    for (k = 0; k < Nz; k++)
      for (j = 0; j < Ny; j++)
	for (i = 0; i < Nx + 1; i++)
	  ZonalDiffusion_(k, j, i) /= factor(j);
  }
  
  
  /*! \brief Transforms the meridional diffusion coefficients to ease the
    numerical integration of diffusion. */
  /*!
    \param MeridionalDiffusion_ (input/output) meridional diffusion
    coefficients.
    \note Coordinates associated with MeridionalDiffusion_ must be in degrees.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::TransformMeridionalDiffusion(Data<T, 3>& MeridionalDiffusion_)
  {
    int i, j, k;
    
    const T pi(3.14159265358979323846264);
    const T ratio = pi / 180.;
    
    int Nx = MeridionalDiffusion_.GetLength(2);
    int Ny = MeridionalDiffusion_.GetLength(1) - 1;
    int Nz = MeridionalDiffusion_.GetLength(0);
    
    Array<T, 1> factor(Ny + 1);
    for (int j = 0; j < Ny + 1; j++)
      factor(j) = cos(ratio * MeridionalDiffusion_[1].Value(0, j, 0))
	* cos(ratio * MeridionalDiffusion_[1].Value(0, j, 0));

    for (k = 0; k < Nz; k++)
      for (j = 0; j < Ny + 1; j++)
	for (i = 0; i < Nx; i++)
	  MeridionalDiffusion_(k, j, i) *= factor(j);
  }


  //! Computes the vertical wind.
  /*! Computes the vertical wind so that div(wind) = 0.
    \param CellWidth_x_ cells widths in meters along x.
    \param CellWidth_y_ cells widths in meters along y.
    \param CellWidth_z_ cells widths in meters along z.
    \param ZonalWind_ zonal wind.
    \param MeridionalWind_ meridonal wind.
    \param VerticalWind_ (output) vertical wind.
  */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::ComputeVerticalWind(Array<T, 1>& CellWidth_x_, Array<T, 1>& CellWidth_y_,
			Array<T, 1>& CellWidth_z_,
			Data<T, 3>& ZonalWind_, Data<T, 3>& MeridionalWind_,
			Data<T, 3>& VerticalWind_)
  {
    int k, j, i;
    int Nz = ZonalWind_.GetLength(0);
    int Ny = ZonalWind_.GetLength(1);
    int Nx = MeridionalWind_.GetLength(2);
    
    for (j = 0; j < Ny; j++)
      for (i = 0; i < Nx; i++)
	VerticalWind_(0, j, i) = 0.;
    
    for (k = 1; k < Nz + 1; k++)
      for (j = 0; j < Ny; j++)
	for (i = 0; i < Nx; i++)
	  VerticalWind_(k, j, i) = VerticalWind_(k-1, j, i)
	    - (ZonalWind_(k-1, j, i+1) - ZonalWind_(k-1, j, i))
	    * CellWidth_z_(k-1) / CellWidth_x_(i)
	    - (MeridionalWind_(k-1, j+1, i) - MeridionalWind_(k-1, j, i))
	    * CellWidth_z_(k-1) / CellWidth_y_(j);
  }


  //! Computes the vertical wind.
  /*! Computes the vertical wind so that div(air_density * wind) = 0.
    \param CellWidth_x_ cells widths in meters along x.
    \param CellWidth_y_ cells widths in meters along y.
    \param CellWidth_z_ cells widths in meters along z.
    \param AirDensity_interf_x_ air density at cells interfaces along x.
    \param ZonalWind_ zonal wind.
    \param AirDensity_interf_y_ air density at cells interfaces along y.
    \param MeridionalWind_ meridonal wind.
    \param AirDensity_interf_y_ air density at cells interfaces along z.
    \param VerticalWind_ (output) vertical wind.
  */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::ComputeVerticalWind(Array<T, 1>& CellWidth_x_, Array<T, 1>& CellWidth_y_,
			Array<T, 1>& CellWidth_z_,
			Data<T, 3>& AirDensity_interf_x_,
			Data<T, 3>& ZonalWind_,
			Data<T, 3>& AirDensity_interf_y_,
			Data<T, 3>& MeridionalWind_,
			Data<T, 3>& AirDensity_interf_z_,
			Data<T, 3>& VerticalWind_)
  {
    int k, j, i;
    int Nz = ZonalWind_.GetLength(0);
    int Ny = ZonalWind_.GetLength(1);
    int Nx = MeridionalWind_.GetLength(2);
    
    for (j = 0; j < Ny; j++)
      for (i = 0; i < Nx; i++)
	VerticalWind_(0, j, i) = 0.;
    
    for (k = 1; k < Nz + 1; k++)
      for (j = 0; j < Ny; j++)
	for (i = 0; i < Nx; i++)
	  {
	    VerticalWind_(k, j, i)
	      = VerticalWind_(k-1, j, i) * AirDensity_interf_z_(k-1, j, i)
	      - (ZonalWind_(k-1, j, i+1) * AirDensity_interf_x_(k-1, j, i+1)
		 - ZonalWind_(k-1, j, i) * AirDensity_interf_x_(k-1, j, i))
	      * CellWidth_z_(k-1) / CellWidth_x_(i)
	      - (MeridionalWind_(k-1, j+1, i)
		 * AirDensity_interf_y_(k-1, j+1, i)
		 - MeridionalWind_(k-1, j, i)
		 * AirDensity_interf_y_(k-1, j, i))
	      * CellWidth_z_(k-1) / CellWidth_y_(j);
	    VerticalWind_(k, j, i) /= AirDensity_interf_z_(k, j, i);
	  }
  }


  //! Computes air density on the basis of temperature and pressure.
  /*! Formula: AirDensity = Pressure / (287. * Temperature).
    \param Temperature_ temperature.
    \param Pressure_ pressure.
    \param AirDensity_ (output) air density.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::ComputeAirDensity(Data<T, 3>& Temperature_, Data<T, 3>& Pressure_,
		      Data<T, 3>& AirDensity_)
  {
    const T r = 287.;

    int k, j, i;

    int Nz = AirDensity_.GetLength(0);
    int Ny = AirDensity_.GetLength(1);
    int Nx = AirDensity_.GetLength(2);
    
    for (k = 0; k < Nz; k++)
      for (j = 0; j < Ny; j++)
	for (i = 0; i < Nx; i++)
	  AirDensity_(k, j, i) = Pressure_(k, j, i)
	    / (r * Temperature_(k, j, i));
  }


  //! Interpolates data on cells interfaces along z.
  /*!
    \param Data_ input data.
    \param Data_interf_z_ (output) data interpolated on interfaces along z.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InterpolateInterface_z(Data<T, 3>& Data_, Data<T, 3>& Data_interf_z_)
  {
    int k, k_in, j, i;

    int Nz = Data_.GetLength(0);
    int Ny = Data_.GetLength(1);
    int Nx = Data_.GetLength(2);
    
    T weight_0, weight_1;

    for (k = 0; k < Nz + 1; k++)
      {
	if (k == 0)
	  k_in = 1;
	else if (k == Nz)
	  k_in = Nz - 1;
	else
	  k_in = k;
	
	weight_1 = (Data_interf_z_[0].Value(k, 0, 0)
		    - Data_[0].Value(k_in - 1, 0, 0))
	  / (Data_[0].Value(k_in, 0, 0) - Data_[0].Value(k_in - 1, 0, 0));
	weight_0 = 1. - weight_1;
    
	for (j = 0; j < Ny; j++)
	  for (i = 0; i < Nx; i++)
	    Data_interf_z_(k, j, i) =  weight_0 * Data_(k_in - 1, j, i)
	      + weight_1 * Data_(k_in, j, i);
      }
  }


  //! Interpolates data on cells interfaces along z.
  /*!
    \param CellCenterDistance_z_ distances between cells centers along z.
    \param CellWidth_z_ cells widths along z.
    \param Data_ input data on cell centers.
    \param Data_interf_z_ (output) data interpolated on interfaces along z.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InterpolateInterface_z(Array<T, 1>& CellCenterDistance_z_,
			   Array<T, 1>& CellWidth_z_,
			   Data<T, 3>& Data_, Data<T, 3>& Data_interf_z_)
  {
    int k, j, i;

    int Nz = Data_.GetLength(0);
    int Ny = Data_.GetLength(1);
    int Nx = Data_.GetLength(2);
    
    T weight;

    weight = CellWidth_z_(0) / (2. * CellCenterDistance_z_(0));
    for (j = 0; j < Ny; j++)
      for (i = 0; i < Nx; i++)
	Data_interf_z_(0, j, i) = (1. + weight) * Data_(0, j, i)
	  - weight * Data_(1, j, i);
    
    for (k = 1; k < Nz; k++)
      for (j = 0; j < Ny; j++)
	for (i = 0; i < Nx; i++)
	  Data_interf_z_(k, j, i) =
	    0.5 * (Data_(k - 1, j, i) * CellWidth_z_(k)
		   + Data_(k, j, i) * CellWidth_z_(k - 1))
	    / CellCenterDistance_z_(k - 1);

    weight = CellWidth_z_(Nz - 1) / (2. * CellCenterDistance_z_(Nz - 2));
    for (j = 0; j < Ny; j++)
      for (i = 0; i < Nx; i++)
	Data_interf_z_(Nz, j, i) = (1. + weight) * Data_(Nz - 1, j, i)
	  - weight * Data_(Nz - 2, j, i);
  }


  //! Interpolates data on cells interfaces along y.
  /*!
    \param Data_ input data.
    \param Data_interf_y_ (output) data interpolated on interfaces along y.
  */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InterpolateInterface_y(Data<T, 3>& Data_, Data<T, 3>& Data_interf_y_)
  {
    int k, j, j_in, i;

    int Nz = Data_.GetLength(0);
    int Ny = Data_.GetLength(1);
    int Nx = Data_.GetLength(2);
    
    // Assumption...
    T weight_0, weight_1;

    for (j = 0; j < Ny + 1; j++)
      {
	if (j == 0)
	  j_in = 1;
	else if (j == Ny)
	  j_in = Ny - 1;
	else
	  j_in = j;
	
	weight_1 = (Data_interf_y_[1].Value(0, j, 0)
		    - Data_[1].Value(0, j_in - 1, 0))
	  / (Data_[1].Value(0, j_in, 0) - Data_[1].Value(0, j_in - 1, 0));
	weight_0 = 1. - weight_1;
    
	for (k = 0; k < Nz; k++)
	  for (i = 0; i < Nx; i++)
	    Data_interf_y_(k, j, i) =  weight_0 * Data_(k, j_in - 1, i)
	      + weight_1 * Data_(k, j_in, i);
      }
  }


  //! Interpolates data on cells interfaces along y.
  /*!
    \param CellCenterDistance_y_ distances between cells centers along y.
    \param CellWidth_y_ cells widths along y.
    \param Data_ input data on cell centers.
    \param Data_interf_y_ (output) data interpolated on interfaces along y.
  */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InterpolateInterface_y(Array<T, 1>& CellCenterDistance_y_,
			   Array<T, 1>& CellWidth_y_,
			   Data<T, 3>& Data_, Data<T, 3>& Data_interf_y_)
  {
    int k, j, i;

    int Nz = Data_.GetLength(0);
    int Ny = Data_.GetLength(1);
    int Nx = Data_.GetLength(2);

    T weight;
    
    weight = CellWidth_y_(0) / (2. * CellCenterDistance_y_(0));
    for (k = 0; k < Nz; k++)
      for (i = 0; i < Nx; i++)
	Data_interf_y_(k, 0, i) = (1. + weight) * Data_(k, 0, i)
	  - weight * Data_(k, 1, i);

    for (j = 1; j < Ny; j++)
      for (k = 0; k < Nz; k++)
	for (i = 0; i < Nx; i++)
	  Data_interf_y_(k, j, i) =
	    0.5 * (Data_(k, j - 1, i) * CellWidth_y_(j)
		   + Data_(k, j, i) * CellWidth_y_(j - 1))
	    / CellCenterDistance_y_(j - 1);

    weight = CellWidth_y_(Ny - 1) / (2. * CellCenterDistance_y_(Ny - 2));
    for (k = 0; k < Nz; k++)
      for (i = 0; i < Nx; i++)
	Data_interf_y_(k, Ny, i) = (1. + weight) * Data_(k, Ny - 1, i)
	  - weight * Data_(k, Ny - 2, i);
  }


  //! Interpolates data on cells interfaces along x.
  /*!
    \param Data_ input data.
    \param Data_interf_x_ (output) data interpolated on interfaces along x.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InterpolateInterface_x(Data<T, 3>& Data_, Data<T, 3>& Data_interf_x_)
  {
    int k, j, i, i_in;

    int Nz = Data_.GetLength(0);
    int Ny = Data_.GetLength(1);
    int Nx = Data_.GetLength(2);
    
    // Assumption...
    T weight_0, weight_1;

    for (i = 0; i < Nx + 1; i++)
      {
	if (i == 0)
	  i_in = 1;
	else if (i == Nx)
	  i_in = Nx - 1;
	else
	  i_in = i;
	
	weight_1 = (Data_interf_x_[2].Value(0, 0, i)
		    - Data_[2].Value(0, 0, i_in - 1))
	  / (Data_[2].Value(0, 0, i_in) - Data_[2].Value(0, 0, i_in - 1));
	weight_0 = 1. - weight_1;
    
	for (k = 0; k < Nz; k++)
	  for (j = 0; j < Ny; j++)
	    Data_interf_x_(k, j, i) =  weight_0 * Data_(k, j, i_in - 1)
	      + weight_1 * Data_(k, j, i_in);
      }
  }


  //! Interpolates data on cells interfaces along x.
  /*!
    \param CellCenterDistance_x_ distances between cells centers along x.
    \param CellWidth_x_ cells widths along x.
    \param Data_ input data on cell centers.
    \param Data_interf_x_ (output) data interpolated on interfaces along x.
   */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>
  ::InterpolateInterface_x(Array<T, 1>& CellCenterDistance_x_,
			   Array<T, 1>& CellWidth_x_,
			   Data<T, 3>& Data_, Data<T, 3>& Data_interf_x_)
  {
    int k, j, i;

    int Nz = Data_.GetLength(0);
    int Ny = Data_.GetLength(1);
    int Nx = Data_.GetLength(2);
    
    T weight;

    weight = CellWidth_x_(0) / (2. * CellCenterDistance_x_(0));
    for (k = 0; k < Nz; k++)
      for (j = 0; j < Ny; j++)
	Data_interf_x_(k, j, 0) = (1. + weight) * Data_(k, j, 0)
	  - weight * Data_(k, j, 1);

    for (i = 1; i < Nx; i++)
      for (k = 0; k < Nz; k++)
	for (j = 0; j < Ny; j++)
	  Data_interf_x_(k, j, i) =
	    0.5 * (Data_(k, j, i - 1) * CellWidth_x_(i)
		   + Data_(k, j, i) * CellWidth_x_(i - 1))
	    / CellCenterDistance_x_(i - 1);

    weight = CellWidth_x_(Nx - 1) / (2. * CellCenterDistance_x_(Nx - 2));
    for (k = 0; k < Nz; k++)
      for (j = 0; j < Ny; j++)
	Data_interf_x_(k, j, Nx) = (1. + weight) * Data_(k, j, Nx - 1)
	  - weight * Data_(k, j, Nx - 2);
  }


  ///////////////////////
  // PROTECTED METHODS //
  ///////////////////////


  //! Moves model input-data to the current date.
  /*! This method prepares the model for a time integration from the current
    date. It reads input data to be read before InitStep and Forward.
  */
  template<class T, class ClassAdvection, class ClassDiffusion>
  void Polair3DTransport<T, ClassAdvection, ClassDiffusion>::InitAllData()
  {

    /*** Pressure, temperature and air density ***/

    if (this->option_manage["temperature"])
      this->InitData("meteo", "Temperature", FileTemperature_i,
		     FileTemperature_f, this->current_date, Temperature_f);
    if (this->option_manage["pressure"])
      this->InitData("meteo", "Pressure", FilePressure_i,
		     FilePressure_f, this->current_date, Pressure_f);

    if (this->option_process["with_air_density"])
      {
	ComputeAirDensity(Temperature_f, Pressure_f, AirDensity_f);
	
	InterpolateInterface_z(CellCenterDistance_z, CellWidth_z,
			       AirDensity_f, AirDensity_interf_z_f);
	InterpolateInterface_y(CellCenterDistance_y, CellWidth_y,
			       AirDensity_f, AirDensity_interf_y_f);
	InterpolateInterface_y(CellCenterDistance_y, CellWidth_y,
			       AirDensity_f, AirDensity_interf_y_f);
	InterpolateInterface_x(CellCenterDistance_x, CellWidth_x,
			       AirDensity_f, AirDensity_interf_x_f);
      }
    else
      {
	AirDensity_i.Fill(1.);
	AirDensity_f.Fill(1.);

	AirDensity_interf_z_i.Fill(1.);
	AirDensity_interf_y_i.Fill(1.);
	AirDensity_interf_x_i.Fill(1.);

	AirDensity_interf_z_f.Fill(1.);
	AirDensity_interf_y_f.Fill(1.);
	AirDensity_interf_x_f.Fill(1.);
      }

    /*** Winds ***/

    if (this->option_manage["horizontal_wind"])
      {
	this->InitData("meteo", "ZonalWind", FileZonalWind_i,
		       FileZonalWind_f, this->current_date, ZonalWind_i);
	if (!option_cartesian)
	  TransformZonalWind(ZonalWind_i);

	this->InitData("meteo", "MeridionalWind", FileMeridionalWind_i,
		       FileMeridionalWind_f, this->current_date,
		       MeridionalWind_i);
	if (!option_cartesian)
	  TransformMeridionalWind(MeridionalWind_i);
      }

    /*** Rain, cloud height and scavenging coefficients ***/
    
    if (this->option_manage["rain"])
      this->InitData("meteo", "Rain", FileRain_i, FileRain_f,
		     this->current_date, Rain_i, false);
    
    if (this->option_manage["cloud_height"])
      this->InitData("meteo", "CloudHeight", FileCloudHeight_i,
		     FileCloudHeight_f, this->current_date,
		     CloudHeight_f);
    
    if (this->option_manage["scavenging_coefficient"]
	&& this->scavenging_model != "none")
      {
	this->scavenging_rain_threshold = 0.;
	if (this->scavenging_model == "microphysical")
	  InitScavengingCoefficient(Temperature_f, Pressure_f,
				    CloudHeight_f, Rain_i,
				    ScavengingCoefficient_f);
	else if (this->scavenging_model == "belot")
	  InitScavengingCoefficient(Rain_i, ScavengingCoefficient_f);
	else if (this->scavenging_model == "constant")
	  InitScavengingCoefficient(ScavengingCoefficient_f);
      }

    /*** Diffusion coefficients ***/

    if (this->option_manage["vertical_diffusion"])
      this->InitData("meteo", "VerticalDiffusion",
		     FileVerticalDiffusionCoefficient_i,
		     FileVerticalDiffusionCoefficient_f,
		     this->current_date, VerticalDiffusionCoefficient_f);

    // Computes rho * Kz.
    if (this->option_process["with_air_density"]
	&& this->option_manage["vertical_diffusion"])
      VerticalDiffusionCoefficient_f.GetArray() =
	AirDensity_interf_z_f.GetArray()
	* VerticalDiffusionCoefficient_f.GetArray();

    if (this->option_manage["horizontal_diffusion"])
      if (!option_isotropic_diffusion)
	{
	  ZonalDiffusionCoefficient_i.Fill(horizontal_diffusion);
	  MeridionalDiffusionCoefficient_i.Fill(horizontal_diffusion);
	}
      else
	{
	  LinearInterpolationRegular(VerticalDiffusionCoefficient_f,
				     ZonalDiffusionCoefficient_i);
	  ZonalDiffusionCoefficient_i.ThresholdMin(0.);
	  LinearInterpolationRegular(VerticalDiffusionCoefficient_f,
				     MeridionalDiffusionCoefficient_i);
	  MeridionalDiffusionCoefficient_i.ThresholdMin(0.);
	}
    
    if (this->option_manage["horizontal_diffusion"] && !option_cartesian)
      {
	TransformZonalDiffusion(this->GridY3D_interf.GetArray(),
				ZonalDiffusionCoefficient_i);
	TransformMeridionalDiffusion(MeridionalDiffusionCoefficient_i);
      }

    /*** Boundary conditions ***/

    if (this->option_manage["boundary_condition"])
      for (int i = 0; i < Ns_bc; i++)
	{
	  string filename
	    = this->input_files["boundary_condition"](species_list_bc[i]);
	  Date date = this->input_files["boundary_condition"].GetDateMin();
	  T Delta_t = this->input_files["boundary_condition"].GetDelta_t();
	  
	  this->InitData(find_replace(filename, "&c", "z"), date, Delta_t,
			 FileBoundaryCondition_z_i, FileBoundaryCondition_z_f,
			 this->current_date, i, BoundaryCondition_z_i);
	  this->InitData(find_replace(filename, "&c", "y"), date, Delta_t,
			 FileBoundaryCondition_y_i, FileBoundaryCondition_y_f,
			 this->current_date, i, BoundaryCondition_y_i);
	  this->InitData(find_replace(filename, "&c", "x"), date, Delta_t,
			 FileBoundaryCondition_x_i, FileBoundaryCondition_x_f,
			 this->current_date, i, BoundaryCondition_x_i);
	}

    /*** Deposition velocities ***/

    if (this->option_manage["deposition_velocity"])
      for (int i = 0; i < Ns_dep; i++)
	this->InitData("deposition_velocity", species_list_dep[i],
		       FileDepositionVelocity_i, FileDepositionVelocity_f,
		       this->current_date, i, DepositionVelocity_f);

    /*** Surface emissions ***/

    if (this->option_manage["surface_emission"])
      for (int i = 0; i < Ns_surf_emis; i++)
	this->InitData("surface_emission", species_list_surf_emis[i],
		       FileSurfaceEmission_i, FileSurfaceEmission_f,
		       this->current_date, i, SurfaceEmission_f);
    
    /*** Volume emissions ***/

    if (this->option_manage["volume_emission"])
      for (int i = 0; i < Ns_vol_emis; i++)
	this->InitData("volume_emission", species_list_vol_emis[i],
		       FileVolumeEmission_i, FileVolumeEmission_f,
		       this->current_date, i, VolumeEmission_f);
  }


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_POLAIR3DTRANSPORT_CXX
#endif
