// Copyright (C) 2005 CEREA
//     Authors: Irne Korsakissok, Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODELS_PUFF_CXX


#include "Puff.hxx"


namespace Polyphemus
{


 //! Main constructor.
  /*!
    \param time_puff time at which the puff is released (seconds).
    \param quantity total mass released by the source (mass unit).
    \param source_abscissa abscissa (meters).
    \param source_ordinate ordinate (meters).
    \param source_height height (meters).
    \param species_index index associated to the species.
   */
  template<class T>
  Puff<T>::Puff(T time_puff, T quantity, T abscissa, T ordinate, T height,
		int species_index):
    time_puff_(time_puff), quantity_(quantity), source_abscissa_(abscissa),
    source_ordinate_(ordinate), source_height_(height),
    sigma_x_(0.), sigma_y_(0.), sigma_z_(0.),
    species_index_(species_index)
  {
  }


  //! Initializes the puff.
  /*!
    Sets the puff center coordinates equal to those of the release point.
  */
  template<class T>
  void Puff<T>::InitPuff()
  {
    x_ = source_abscissa_;
    y_ = source_ordinate_;
    z_ = source_height_;
    distance_ = 0.;
  }


  //! Returns the time at which the puff is released.
  /*!
    \return The time at which the puff is released (seconds).
  */
  template<class T>
  T Puff<T>::GetReleaseTime() const
  {
    return time_puff_;
  }


  //! Returns the species index.
  /*!
    \return The species index.
  */
  template<class T>
  int Puff<T>::GetSpeciesIndex() const
  {
    return species_index_;
  }


  //! Returns the puff distance from release point.
  /*!
    \return The puff distance from release point (meters).
   */
  template<class T>
  T Puff<T>::GetDistance() const
  {
    return distance_;
  }


  //! Sets the horizontal standard deviation in the downwind direction.
  /*!  \param sigma_x horizontal standard deviation in the downwind direction
    (meters).
   */
  template<class T>
  void Puff<T>::SetSigma_x(T sigma_x)
  {
    sigma_x_ = sigma_x;
  }


  //! Sets the standard deviation in the crosswind direction.
  /*!  \param sigma_y horizontal standard deviation in the crosswind direction
    (meters).
  */
  template<class T>
  void Puff<T>::SetSigma_y(T sigma_y)
  {
    sigma_y_ = sigma_y;
  }


  //! Sets the vertical standard deviation.
  /*!
    \param sigma_z vertical standard deviation (meters).
  */
  template<class T>
  void Puff<T>::SetSigma_z(T sigma_z)
  {
    sigma_z_ = sigma_z;
  }


  //! Sets the species half-life time.
  /*!
    \param half_time the species half-life time (seconds).
  */
  template<class T>
  void Puff<T>::SetHalfTime(T half_time)
  {
    half_time_ = half_time;
  }


  //! Sets the species biological half-life time.
  /*!
    \param bio_half_time the species biological half-life time (seconds).
  */
  template<class T>
  void Puff<T>::SetBioHalfTime(T bio_half_time)
  {
    bio_half_time_ = bio_half_time;
  }


  //! Sets the species scavenging coefficient.
  /*!
    \param lambda the species scavenging coefficient ( s^(-1) ).
  */
  template<class T>
  void Puff<T>::SetScavengingCoefficient(T lambda)
  {
    scavenging_coefficient_ = lambda;
  }


  //! Sets the species deposition velocity.
  /*!
    \param lambda the species deposition velocity (m/s).
  */
  template<class T>
  void Puff<T>::SetDepositionVelocity(T vd)
  {
    deposition_velocity_ = vd;
  }


  //! Computes the new position of the puff center after advection.
  /*!
    \param u the wind speed (m/s).
    \param cos_angle cosine of wind angle.
    \param sin_angle sine of wind angle.
    \param Delta_t the time step (s).
  */  template<class T>
  void Puff<T>::Advection(T u, T cos_angle, T sin_angle, T Delta_t)
  {
    x_ += u * cos_angle * Delta_t;
    y_ += u * sin_angle * Delta_t;
    distance_ += u * Delta_t;
  }


  //! Computes the species concentration due to the puff at a given point.
  /*!
    \param x abscissa of the point where concentration is computed (m).
    \param y ordinate of the point where concentration is computed (m).
    \param z height of the point where concentration is computed (m).
    \param cos_angle cosine of the wind angle.
    \param sin_angle sine of the wind angle.
    \param stability stability class (integer in [1, 6]).
    \param land_category land category ("rural" or "urban").
    \param option_radioactive_decay indicates if there is radioactive decay.
    \param option_biological_decay indicates if there is biological decay.
    \param option_scavenging indicates if there is wet scavenging.
    \param option_dry_deposition indicates if there is dry deposition.
    \param Nchamberlain_tot number of points to compute the Chamberlain
    integral.
    \return The concentration at (\a x, \a y, \a z).
  */
  template<class T>
  T Puff<T>::ComputeConcentration(T x, T y, T z, T u,
				  T cos_angle, T sin_angle,
				  int stability,
				  T inversion_height, string land_category,
				  bool option_radioactive_decay,
				  bool option_biological_decay,
				  bool option_scavenging,
				  bool option_dry_deposition,
				  string deposition_model,
				  int Nchamberlain_tot) const
  {
    T radioactive_factor = 1.;
    T biological_factor = 1.;
    T scavenging_factor = 1.;
    T chamberlain_factor = 1.;
    T overcamp_factor = 1.;

    const T pi = 3.14159265358979323846264;
    // (2. * pi) ^ (3/2)
    const T pi__1_5 = 15.749609945722419;
    // log(2.)
    const T log2 = 0.69314718055994529;

    // Radioactive decay.
    if (option_radioactive_decay && half_time_ != 0.)
      radioactive_factor = exp(-log2 * distance_ / (u * half_time_));

    // Biological decay.
    if (option_biological_decay && bio_half_time_ != 0.)
      biological_factor = exp(-log2 * distance_ / (u * bio_half_time_));

    // Scavenging.
    if (option_scavenging)
      scavenging_factor = exp(-scavenging_coefficient_ * distance_ / u);

    // Dry deposition.
    if (option_dry_deposition)
      {
	bool rural = land_category == "rural";

	if (deposition_model == "Chamberlain")
	  {
	    T delta_x = distance_ / T(Nchamberlain_tot);
	    T z2 = 0.5 * z_ * z_;
	    T dsigma_z;
	    T x(0.);
	
	    T fdry = 0;
	    for (int i = 1; i < Nchamberlain_tot+1; i++)
	      {
		x += delta_x;
		if (rural)
		  dsigma_z = ComputeRuralPlumeVerticalSigma(x, stability);
		else
		  dsigma_z = ComputeUrbanPlumeVerticalSigma(x, stability);

		fdry += exp(-z2 / (dsigma_z * dsigma_z)) / dsigma_z;
	      }
	    chamberlain_factor = exp(-deposition_velocity_ / u
				     * sqrt(2. / pi) * fdry * delta_x);
	  }
	else
	  {
	    T dsigma_z;
	    if (rural)
	      dsigma_z =
		DifferentiateRuralPlumeVerticalSigma(distance_,
						     stability);
	    else
	      dsigma_z =
		DifferentiateUrbanPlumeVerticalSigma(distance_,
						     stability);
	    overcamp_factor = 1. - 2. * deposition_velocity_
	      / (deposition_velocity_ + u * z_ * dsigma_z / sigma_z_);
	  }
      }

    // Downwind distance from puff center.
    T dx = (x - x_) * cos_angle + (y - y_) * sin_angle;
    // Crosswind distance from puff center.
    T dy = (x_ - x) * sin_angle + (y - y_) * cos_angle;
 
    // Downwind contribution.
    T Fx = exp(-dx * dx  / (2. * sigma_x_ * sigma_x_));

   // Crosswind contribution.
    T Fy = exp(-dy * dy  / (2. * sigma_y_ * sigma_y_));

    // Vertical contribution (real and image sources respectively).
    T dzr = z - z_;
    T dzi = z + z_;
    T sigma_z_2 = sigma_z_ * sigma_z_;
    T Fz = exp(-dzr * dzr / (2. * sigma_z_2))
      + overcamp_factor * exp(-dzi * dzi / (2. * sigma_z_2));

    // Reflexion if inversion (real and image sources respectively).
    T FzI = 0.;

    if (inversion_height > 0.)
      for (int N = 1; N < 6; N++)
	{
	  T dzmi = z - z_ + 2. * N * inversion_height;
	  T dzpi = z + z_ - 2. * N * inversion_height;
	  
	  FzI += exp(-dzmi*dzmi / (2. * sigma_z_2))
	    + overcamp_factor * exp(-dzpi*dzpi / (2. * sigma_z_2));
	}

    return radioactive_factor * biological_factor
      * scavenging_factor * chamberlain_factor * quantity_
      * Fx * Fy * (Fz + FzI)
      / (pi__1_5 * sigma_x_ * sigma_y_ * sigma_z_);
  }


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_PUFF_CXX
#endif
