// Copyright (C) 2005 CEREA
//     Authors: Irne Korsakissok, Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODELS_PUFF_HXX


namespace Polyphemus
{


  //////////////
  // INCLUDES //
  //////////////


#include <list>
  using namespace std;


  //////////////
  // GAUSSIAN //
  //////////////


  //! Class that stores all data about a puff.
  template<class T>
  class Puff
  {
  
  protected:

    //! Time at which the puff is released (seconds).
    T time_puff_;

    //! Total mass released (mass).
    T quantity_;

    // Data about the source that released the puff.
    //! Source abscissa (meters).
    T source_abscissa_;
    //! Source ordinate (meters).
    T source_ordinate_;
    //! Source altitude (meters).
    T source_height_;

    //! Current abscissa (meters).
    T x_;
    //! Current ordinate (meters).
    T y_;
    //! Current altitude (meters).
    T z_;

    //! Downwind distance to the source (meters).
    T distance_;

    // Diffusion coefficients.
    //! Horizontal diffusion coefficient in the downwind direction (meters).
    T sigma_x_;
    //! Horizontal diffusion coefficient in the crosswind direction (meters).
    T sigma_y_;
     //! Vertical diffusion coefficient (meters).
    T sigma_z_;

    //! Species index.
    int species_index_;

    //! Species half-life (seconds).
    T half_time_;

    //! Species biological half-life (seconds).
    T bio_half_time_;

    //! Species scavenging coefficient (seconds^(-1)).
    T scavenging_coefficient_;

    //! Species deposition velocity (m/s).
    T deposition_velocity_;

  public:

    /*** Constructor ***/

    Puff(T time_puff, T quantity, T abscissa, T ordinate, T height,
	 int species_index_);

    /*** Methods ***/

    void InitPuff();

    T GetReleaseTime() const;
    int GetSpeciesIndex() const;
    T GetDistance() const;

    void SetSigma_x(T sigma_x);
    void SetSigma_y(T sigma_y);
    void SetSigma_z(T sigma_z);
    void SetHalfTime(T half_time);
    void SetBioHalfTime(T bio_half_time);
    void SetScavengingCoefficient(T lambda);
    void SetDepositionVelocity(T vd);

    void Advection(T u, T cos_angle, T sin_angle, T Delta_t);
    T ComputeConcentration(T x, T y, T z, T u, T cos_angle, T sin_angle,
			   int stability,
			   T inversion_height, string land_category,
			   bool option_radioactive_decay,
			   bool option_biological_decay,
			   bool option_scavenging,
			   bool option_dry_deposition,
			   string deposition_model, int N) const;
  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_PUFF_HXX
#endif
