C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE compute_scavenging_coefficient(nx, ny, nz, nscav, nxh,
     $     nyh, nzh, level, gas_phase_diffusivity, henry_constant,
     $     temperature, pressure, rain, cloud_height, lambda_scavenging)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     Compute scaveging coefficients for gaseous species.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     TEMPERATURE    : temperature. ([K])
C     PRESSURE   : pressure. ([Pa])
C     RAIN    : rain intensity. ([mm/h])
C     CLOUD_HEIGHT  : height of cloud basis. ([m])
C
C     -- INPUT/OUTPUT VARIABLES
C     
C     -- OUTPUT VARIABLES
C     
C     LAMBDA_SCAVENGING: scavenging coefficient. ([1./s]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C     Parameterization used in this subroutine is presented and
C     justified in " Numerical and theoretical investigation of a
C     simplified model for the parameterization of below-cloud
C     scavenging by falling raindrops " (B. Sportisse and L. Du Bois,
C     Atmospheric Environment, Vol. 36, pp 5719-5727, 2002 )
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Yelva Roustan, CEREA, February 2003.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      integer nx,ny,nz,nscav,nxh,nyh,nzh
      integer i,j,k,ih,jh,kh,jsca

      double precision gas_phase_diffusivity(nscav) !([cm^2/s])
      double precision henry_constant(nxh,nyh,nzh,nscav) !([mol/L/atm])
      double precision temperature(nx,ny,nz)
      double precision pressure(nx,ny,nz)
      double precision rain(nx,ny)
      double precision cloud_height(nx,ny)
      double precision level(nz)

      double precision lambda_scavenging(nx,ny,nz,nscav)

      double precision DLeta,DLnuair,DLrho
      double precision DLurain,DLddrop,DLrey
      
      double precision DLSh,DLKtrans,Pr

      integer henry_3d

C------------------------------------------------------------------------
C     0. Setup 

      if (nxh.eq.nx.and.nyh.eq.ny.and.nzh.eq.nz) then
         henry_3d = 1
      else if (nxh.eq.1.and.nyh.eq.1.and.nzh.eq.1) then
         henry_3d = 0
      else
         write(6, *) "Error in compute_scavenging_coefficient: "
     $        // "incompatible dimensions of array for Henry constants."
         stop
      endif

      Pr=8.20d-2                !Perfect gaz constant ([atm.L/mol/K]).

      do jsca=1,nscav
         do k=1,nz
            do j=1,ny
               do i=1,nx
                  lambda_scavenging(i,j,k,jsca)=0.D0
               enddo
            enddo
         enddo
      enddo

      do j=1,ny
         do i=1,nx
            IF (rain(i,j).GT.0.D0) THEN
               do k=1,nz
                  IF (cloud_height(i,j).GT.level(k)) THEN

C------------------------------------------------------------------------
C     1. Computation of kinematic viscosity through Sutherland's law.

                     DLeta = 1.83d-5 * ( 416.16d0 / 
     $                    ( temperature(i,j,k) + 120.d0) ) *
     $                    ( temperature(i,j,k) / 296.16d0 )**1.5d0

C     Mair/R USI = 28.97d-3/8.314 .

                     DLrho = pressure(i,j,k)*3.48d-3/temperature(i,j,k)

                     DLnuair = DLeta / DLrho

C------------------------------------------------------------------------
C     2. Computation of rain drop diameter, velocity and Reynolds number.

c     DLddrop in cm
                     DLddrop = 0.1d0 * 0.976d0 * rain(i,j)**0.21d0
c     DLurain in m/s
                     DLurain = 9.58d0 * (1.d0 - DEXP(
     &                    (- ( DLddrop / 0.171d0 )**1.147d0 ) ) )

                     DLrey = 1.d-2*DLurain*DLddrop/DLnuair

C------------------------------------------------------------------------
C     3. Computation of mass transfer coefficient.

                     do jsca=1,nscav

C     Sherwood number.
                        DLSh = 2.d0 + 0.6d0 * DLrey**0.5d0
     $                       * ( 1.d4 * DLnuair  
     $                       / gas_phase_diffusivity(jsca))**(1.d0/3.d0)

C     Transfert coefficient ([cm/s]).
                        DLKtrans = ( gas_phase_diffusivity(jsca)
     $                       / DLddrop ) * DLSh

C------------------------------------------------------------------------
C     4. Computation of wet scavenging coefficient for gas.

C     1.67d-6 = 6 * 1.d-6 / 3.6d0.
                        ih = (i-1) * henry_3d + 1
                        jh = (j-1) * henry_3d + 1
                        kh = (k-1) * henry_3d + 1

                        lambda_scavenging(i,j,k,jsca) = 
     $                       1.67d-6 * rain(i,j) / 
     $                       ( DLurain * DLddrop / DLKtrans )
     $                       * dexp( -(cloud_height(i,j)-level(k)) /
     $                       ( DLddrop / 6.d0 / DLKtrans * DLurain
     $                       * henry_constant(ih,jh,kh,jsca) * Pr
     $                       * temperature(i,j,k)) )

                     enddo
                  endif
               enddo
            endif
         enddo
      enddo

      end
