// Copyright (C) 2005 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHMUS_FILE_MODULES_CHEMISTRY_CHEMISTRYRACM_SIREAM_CXX


#include "ChemistryRACM_SIREAM.hxx"


namespace Polyphemus
{


  //! Default constructor.
  template<class T>
  ChemistryRACM_SIREAM<T>::ChemistryRACM_SIREAM():
    Ns(82), Nr(241), Nr_photolysis(23), Ncycle(1), molecular_weight(Ns),
    photolysis_reaction_index(Nr_photolysis), Ns_inorganic_aer(5),
    Ns_inert_aer(2), Ns_organic_aer(9), Ncycle_aer(1),
    Ns_aer(Ns_inert_aer + Ns_inorganic_aer + Ns_organic_aer + 1),
    Noptions_aer(16)
  {
  }


  //! Initialization of the scheme.
  /*! In case the model is incompatible with the chemical mechanism, an
    exception is thrown.
    \param Model model with the following interface:
    <ul>
    <li> GetNs()
    <li> GetSpeciesList()
    <li> GetSpeciesFile()
    <li> GetNr_photolysis()
    <li> GetPhotolysisReactionList()
    <li> GetNs_source()
    <li> GetNz_source()
    <li> SourceGlobalIndex(int)
    </ul>
   */
  template<class T>
  template<class ClassModel>
  void ChemistryRACM_SIREAM<T>::Init(ClassModel& Model)
  {
    if (Model.GetNs() != Ns)
      throw string("Incompatibility: model manages ") + to_str(Model.GetNs())
	+ string(" species while chemistry has ") + to_str(Ns) + " species.";
    if (Model.GetNr_photolysis() != Nr_photolysis)
      throw string("Incompatibility: model manages ")
	+ to_str(Model.GetNr_photolysis())
	+ string(" photolysis reactions while chemistry has ")
	+ to_str(Nr_photolysis) + " photolysis reactions.";
    species_list = Model.GetSpeciesList();

    Nbin_aer = Model.GetNbin_aer();

    // Photolysis reactions.
    ConfigStream config_species(Model.GetSpeciesFile());
    config_species.SetSection("[molecular_weight]");
    for (int i = 0; i < Ns; i++)
      config_species.PeekValue(species_list[i], molecular_weight(i));
    
    config_species.SetSection("[photolysis_reaction_index]");
    string species;
    for (int i = 0; i < Nr_photolysis; i++)
      {
	species = config_species.GetElement();
	photolysis_reaction_name[species] =
	  convert<int>(config_species.GetElement());
      }

    for (int i = 0; i < Nr_photolysis; i++)
      photolysis_reaction_index(i) =
	photolysis_reaction_name[Model.GetPhotolysisReactionList()[i]];
    
    // Volumic sources.
    Ns_source = Model.GetNs_source();
    Nz_source = Model.GetNz_source();
    source_index.resize(Ns_source);
    for (int i = 0; i < Ns_source; i++)
      source_index(i) = Model.SourceGlobalIndex(i);

    Ns_source_aer = Model.GetNs_source_aer();
    NbinMax_source_aer = Model.GetNbinMax_source_aer();
    Nz_source_aer = Model.GetNz_source_aer();

    source_index_species_aer.resize(Ns_source_aer);
    source_index_bin_aer.resize(Ns_source_aer, NbinMax_source_aer);
    source_index_bin_aer = -1;
    for (int i = 0; i < Ns_source_aer; i++)
      {
	source_index_species_aer(i) = Model.SourceGlobalIndex_aer(i);
	for (int j = 0; j < NbinMax_source_aer; j++)
	  source_index_bin_aer(i, j) = Model.SourceGlobalBinIndex_aer(i, j);
      }
    
    ConfigStream config(Model.GetConfigurationFile());
    config.SetSection("[options]");
    
    config.PeekValue("With_coagulation",
		     option_process_aer["with_coagulation"]);
    config.PeekValue("With_condensation",
		     option_process_aer["with_condensation"]);
    config.PeekValue("With_nucleation",
		     option_process_aer["with_nucleation"]);     
    config.PeekValue("With_organics",
		     option_process_aer["with_organics"]);
    config.PeekValue("With_in_cloud_scavenging",
		     option_process_aer["with_in_cloud_scavenging"]);
    config.PeekValue("With_cloud_chemistry",
		     option_process_aer["with_cloud_chemistry"]);
    config.PeekValue("With_heterogeneous_reactions",
		     option_process_aer["with_heterogeneous_reactions"]);
    config.PeekValue("With_kelvin_effect",
		     option_process_aer["with_kelvin_effect"]);
    config.PeekValue("With_volume_emission_aerosol",
		     option_process_aer["with_volume_emission_aer"]);
    config.PeekValue("Dynamic_condensation_solver",
		     dynamic_condensation_solver);
    dynamic_condensation_solver = lower_case(dynamic_condensation_solver);
    config.PeekValue("Fixed_cutting_diameter", fixed_cutting_diameter);
    fixed_cutting_diameter = 1.e-6 * fixed_cutting_diameter;
    config.PeekValue("Sulfate_computation", sulfate_computation);
    sulfate_computation = lower_case(sulfate_computation);      
    config.PeekValue("Redistribution_method", redistribution_method);
    redistribution_method = lower_case(redistribution_method);
    config.PeekValue("Nucleation_model", nucleation_model);
    nucleation_model = lower_case(nucleation_model);
    config.PeekValue("With_fixed_density",
		     option_process_aer["with_fixed_density"]);
    config.PeekValue("Wet_diameter_estimation", wet_diameter_estimation);
    wet_diameter_estimation = lower_case(wet_diameter_estimation);
    
    // Density in kg / m^3.
    config.PeekValue("Fixed_aerosol_density", FixedDensity_aer);
    Density_aer.resize(Nbin_aer);
    Density_aer = FixedDensity_aer;
    cout << "Module: Fixed_aerosol_density = " << FixedDensity_aer << endl;

    // Lwc cloud threshold.
    config.PeekValue("Lwc_cloud_threshold", lwc_cloud_threshold);
    cout << "Module: Lwc_cloud_threshold = " << lwc_cloud_threshold << endl;

    // Reads bin bounds.
    vector<string> bin_list;
    config.SetSection("[domain]");
    config.Find("Bin_bounds");
    bin_list = split(config.GetLine());
    
    // Reads bin bounds in micrometers and converts it to meters.
    BinBound_aer.resize(Nbin_aer + 1);
    for (int i = 0; i < Nbin_aer + 1; i++)
      BinBound_aer(i) = 1.e-6 * convert<T>(bin_list[i]);

    // Compute coagulation partition coefficient, if coagulation active.
    CoagulationCouple.resize(Nbin_aer);
    CoagulationFirstIndex.resize(4 * Nbin_aer, Nbin_aer);
    CoagulationSecondIndex.resize(4 * Nbin_aer, Nbin_aer);
    CoagulationPartitionCoefficient.resize(Nbin_aer, Nbin_aer,
					   Nbin_aer);
    
    if (option_process_aer["with_coagulation"])
      {
	cout << "Module: computing coagulation coefficients ... ";

	_compute_coagulation_coefficient(&Nbin_aer,
					 BinBound_aer.data(),
					 CoagulationCouple.data(),
					 CoagulationFirstIndex.data(),
					 CoagulationSecondIndex.data(),
					 CoagulationPartitionCoefficient.data());
	cout << "done" << endl;
      }

    // Bin corresponding to cutting diameter (for condensation).
    cutting_bin = 0;
    while (BinBound_aer(cutting_bin) < fixed_cutting_diameter &&
	   cutting_bin < Nbin_aer)
	cutting_bin++;

    // List of aerosol options to be passed to fortran routine.
    options_aer.resize(Noptions_aer);

    options_aer(0) = option_process_aer["with_coagulation"] ? 1 : 0;
    if (option_process_aer["with_coagulation"])
      cout << "Module: with coagulation" << endl;
    
    options_aer(1) = option_process_aer["with_condensation"] ? 1 : 0;
    if (option_process_aer["with_condensation"])
      cout << "Module: with condensation" << endl;

    options_aer(2) = option_process_aer["with_nucleation"] ? 1 : 0;
    if (option_process_aer["with_nucleation"])
      cout << "Module: with nucleation" << endl;
 
    options_aer(3) = option_process_aer["with_organics"] ? 1 : 0;
    if (option_process_aer["with_organics"])
      cout << "Module: with organics" << endl;
 
    options_aer(4) = option_process_aer["with_cloud_chemistry"] ? 1 : 0;
    if (option_process_aer["with_cloud_chemistry"])
      cout << "Module: with cloud chemistry" << endl;

     options_aer(5) = option_process_aer["with_in_cloud_scavenging"] ? 1 : 0; 
    if (option_process_aer["with_in_cloud_scavenging"])
      cout << "Module: with in-cloud scavenging" << endl;
    
    options_aer(6) = option_process_aer["with_heterogeneous_reactions"] ? 1 : 0;
    if (option_process_aer["with_heterogeneous_reactions"])
      cout << "Module: with heterogeneous reactions" << endl;
    
    options_aer(7) = option_process_aer["with_kelvin_effect"] ? 1 : 0;
    if (option_process_aer["with_kelvin_effect"])
      cout << "Module: with kelvin effect" << endl;
    
    options_aer(8) = option_process_aer["with_fixed_density"] ? 0 : 1;
    if (option_process_aer["with_fixed_density"])
      cout << "Module: with fixed density" << endl;

    options_aer(9) = cutting_bin;
    if (cutting_bin == 0)
      cout << "Module: all bins at dynamic." << endl;
    else if (cutting_bin == Nbin_aer)
      cout << "Module: all bins at equilibrium." << endl;
    else
      cout << "Module: bins from 1 to " << cutting_bin <<
	"(included) at equilibrium." << endl;

    if (sulfate_computation == "dynamic")
      options_aer(10) = 1;
    else if (sulfate_computation == "equilibrium")
      options_aer(10) = 0;
    else
      throw string("bad string \"") + sulfate_computation +
	string("\" for sulfate computation option,\n") +
	string("possibilities are dynamic or equilibrium.");

    if (dynamic_condensation_solver == "etr")
      options_aer(11) = 1;
    else if (dynamic_condensation_solver == "ros2")
      options_aer(11) = 2;
    else if (dynamic_condensation_solver == "ebi")
      options_aer(11) = 3;
    else
      throw string("bad string \"") + dynamic_condensation_solver +
	string("\" for dynamic c/e solver option,\n") +
	string("possibilities are etr, ros2 or ebi.");
    
    cout << "Module: dynamic condensation solver: " <<
      dynamic_condensation_solver << endl;
    
    if (redistribution_method == "number-conserving")
      options_aer(12) = 1;
    else if (redistribution_method == "interpolation")
      options_aer(12) = 2;
    else
      throw string("bad string \"") + redistribution_method +
	string("\" for redistribution method option,\n") +
	string("possibilities are number-conserving or interpolation.");
    
    cout << "Module: redistribution method: " <<
      redistribution_method << endl;
    
    if (nucleation_model == "binary")
      options_aer(13) = 0;
    else if (nucleation_model == "ternary") 
      options_aer(13) = 1;
    else
      throw string("bad string \"") + nucleation_model +
	string("\" for nucleation model option,\n") +
	string("possibilities are binary or ternary.");
    
    if (option_process_aer["with_nucleation"])
      cout << "Module: nucleation model: " <<
	nucleation_model << endl;
    
    if (wet_diameter_estimation == "isoropia")
      options_aer(14) = 0;
    else if (wet_diameter_estimation == "gerber")
      options_aer(14) = 1;
    else
      throw string("bad string \"") + wet_diameter_estimation +
	string("\" for wet diameter estimation option,\n") +
	string("possibilities are gerber or isoropia.");
    
    cout << "Module: wet diameter estimation: " <<
      wet_diameter_estimation << endl;

    options_aer(15) = option_process_aer["with_volume_emission_aer"] ? 1 : 0;
    if (option_process_aer["with_volume_emission_aer"])
      cout << "Module: with aerosol volume emissions" << endl;

    // Index of species subject to heterogeneous reactions.
    heterogeneous_reaction_index.resize(4);
    heterogeneous_reaction_index(0) = Model.GetSpeciesIndex("HO2");
    heterogeneous_reaction_index(1) = Model.GetSpeciesIndex("NO2");
    heterogeneous_reaction_index(2) = Model.GetSpeciesIndex("NO3");
    heterogeneous_reaction_index(3) = Model.GetSpeciesIndex("N2O5");
  }
  

  //! Performs an integration over one time step.
  /*!
    \param Model model with the following interface:
    <ul>
    <li> GetCurrentDate()
    <li> GetNextDate()
    <li> D3("Attenuation_i")
    <li> D3("SpecificHumidity_i")
    <li> D3("Temperature_i")
    <li> D3("Pressure_i")
    <li> GetSource_i()
    <li> D4("PhotolysisRate_i")
    <li> D3("Attenuation_f")
    <li> D3("SpecificHumidity_f")
    <li> D3("Temperature_f")
    <li> D3("Pressure_f")
    <li> GetSource_f()
    <li> D4("PhotolysisRate_f")
    <li> GetGridXArray1D()
    <li> GetGridYArray1D()
    <li> GetConcentration()
    <li> D3("LiquidWaterContent_i")
    <li> D4("WetDiameter_aer")
    <li> GetConcentration_aer()
    </ul>
   */
  template<class T>
  template<class ClassModel>
  void ChemistryRACM_SIREAM<T>::Forward(ClassModel& Model)
  {
    Date date_i = Model.GetCurrentDate();
    Date date_f = Model.GetNextDate();

    Forward(T(date_i.GetNumberOfSeconds()), Model.D3("Attenuation_i"),
	    Model.D3("SpecificHumidity_i"), Model.D3("Temperature_i"),
	    Model.D3("Pressure_i"), Model.GetSource_i(),
	    Model.D4("PhotolysisRate_i"), T(date_f.GetNumberOfSeconds()),
	    Model.D3("Attenuation_f"), Model.D3("SpecificHumidity_f"),
	    Model.D3("Temperature_f"), Model.D3("Pressure_f"),
	    Model.GetSource_f(), Model.D4("PhotolysisRate_f"),
	    Model.GetGridXArray1D(), Model.GetGridYArray1D(),
	    Model.GetConcentration(), Model.D3("LiquidWaterContent_i"),
	    Model.D4("WetDiameter_aer"), Model.GetConcentration_aer());
  }


  //! Performs an integration over one time step.
  /*!
    \param Model model with the following interface:
    <ul>
    <li> GetCurrentDate()
    <li> GetNextDate()
    <li> D3("SpecificHumidity_i")
    <li> D3("Temperature_i")
    <li> D3("Pressure_i")
    <li> GetConcentration()
    <li> D3("LiquidWaterContent_i")
    <li> D2("Rain_i")
    <li> GetLayerInterface()
    <li> GetConcentration_aer()
    <li> GetSource_aer_i()
    <li> D3("pH")
    </ul>
   */
  template<class T>
  template<class ClassModel>
  void ChemistryRACM_SIREAM<T>::Forward_aer(ClassModel& Model)
  {
    Date date_i = Model.GetCurrentDate();
    Date date_f = Model.GetNextDate();

    Forward_aer(T(date_i.GetNumberOfSeconds()), Model.D3("SpecificHumidity_i"),
		Model.D3("Temperature_i"), Model.D3("Pressure_i"),
		T(date_f.GetNumberOfSeconds()), Model.GetConcentration(),
		Model.D3("LiquidWaterContent_i"), Model.D2("Rain_i"),
		Model.GetLayerInterface(), Model.GetConcentration_aer(),
		Model.GetSource_aer_i(), Model.D3("pH"));
  }


  //! Performs an integration over one time step.
  /*!
    \param current_time starting time in seconds.
    \param Attenuation_i cloud attenuation coefficients at the beginning of
    the time step.
    \param SpecificHumidity_i specific humidity at the beginning of the time
    step.
    \param Temperature_i temperature at the beginning of the time step.
    \param Pressure_i pressure at the beginning of the time step.
    \param Source_i volume sources at the beginning of the time step.
    \param PhotolysisRate_i photolysis rates at the beginning of the time
    step.
    \param next_time time at the end of the time step.
    \param Attenuation_f cloud attenuation coefficients at the end of the
    time step.
    \param SpecificHumidity_f specific humidity at the end of the time step.
    \param Temperature_f temperature at the end of the time step.
    \param Pressure_f pressure at the end of the time step.
    \param Source_f volume sources at the end of the time step.
    \param PhotolysisRate_f photolysis rates at the end of the time step.
    \param Longitude longitudes.
    \param Latitude latitudes.
    \param Concentration gas concentrations.
    \param LiquidWaterContent_i air liquid water content at the beginning of the time step.
    \param WetDiameter_aer Aerosol wet diameters at the beginning of the time step.
    \param Concentration_aer aerosol concentrations.
  */
  template<class T>
  void ChemistryRACM_SIREAM<T>::Forward(T current_time,
					Data<T, 3>& Attenuation_i,
					Data<T, 3>& SpecificHumidity_i,
					Data<T, 3>& Temperature_i,
					Data<T, 3>& Pressure_i,
					Data<T, 4>& Source_i,
					Data<T, 4>& PhotolysisRate_i,
					T next_time,
					Data<T, 3>& Attenuation_f,
					Data<T, 3>& SpecificHumidity_f,
					Data<T, 3>& Temperature_f,
					Data<T, 3>& Pressure_f,
					Data<T, 4>& Source_f,
					Data<T, 4>& PhotolysisRate_f,
					Array<T, 1>& Longitude,
					Array<T, 1>& Latitude,
					Data<T, 4>& Concentration,
					Data<T, 3>& LiquidWaterContent_i,
					Data<T, 4>& WetDiameter_aer,
					Data<T, 5>& Concentration_aer)
    
  {
    int Nz = Concentration.GetLength(1);
    int Ny = Concentration.GetLength(2);
    int Nx = Concentration.GetLength(3);

    int icld = option_process_aer["with_in_cloud_scavenging"] ? 1 : 0;
    int iheter = option_process_aer["with_heterogeneous_reactions"] ? 1 : 0;

    _chem(&Nx, &Ny, &Nz, &Ns, &Nr,
	  &Nr_photolysis, photolysis_reaction_index.data(),
	  &Ns_source, source_index.data(), &Nz_source,
	  &lwc_cloud_threshold, molecular_weight.data(),
	  &current_time, Attenuation_i.GetData(),
	  SpecificHumidity_i.GetData(), Temperature_i.GetData(),
	  Pressure_i.GetData(), Source_i.GetData(),
	  PhotolysisRate_i.GetData(), &next_time, Attenuation_f.GetData(),
	  SpecificHumidity_f.GetData(), Temperature_f.GetData(),
	  Pressure_f.GetData(), Source_f.GetData(),
	  PhotolysisRate_f.GetData(), &Ncycle, Longitude.data(),
	  Latitude.data(), Concentration.GetData(),
	  &icld, &iheter, &Ns_aer, &Nbin_aer,
	  LiquidWaterContent_i.GetData(),
	  BinBound_aer.data(), &FixedDensity_aer,
	  WetDiameter_aer.GetData(),
	  heterogeneous_reaction_index.data(),
	  Concentration_aer.GetData());

  }


    //! Performs an integration over one time step.
  /*!
    \param current_time starting time in seconds.
    \param SpecificHumidity_i specific humidity at the beginning of the time
    step.
    \param Temperature_i temperature at the beginning of the time step.
    \param Pressure_i pressure at the beginning of the time step.
    \param next_time time at the end of the time step.
    \param Concentration gas concentrations.
    \param LiquidWaterContent_i air liquid water content at the beginning of the time step.
    \param Rain_i rain rate at the beginning of the time step.
    \param VerticalInterface height of vertical interfaces in meter.
    \param Concentration_aer aerosol concentrations.
    \param Source_aer_i aerosol emissions.
    \param aerosol pH.
  */
  template<class T>
  void ChemistryRACM_SIREAM<T>::Forward_aer(T current_time,
					Data<T, 3>& SpecificHumidity_i,
					Data<T, 3>& Temperature_i,
					Data<T, 3>& Pressure_i,
					T next_time,
					Data<T, 4>& Concentration,
					Data<T, 3>& LiquidWaterContent_i,
					Data<T, 2>& Rain_i,
					Array<T, 1>& VerticalInterface,
					Data<T, 5>& Concentration_aer,
					Data<T, 5>& Source_aer_i,
					Data<T, 3>& pH)
  {
    int Nz = Concentration.GetLength(1);
    int Ny = Concentration.GetLength(2);
    int Nx = Concentration.GetLength(3);

    _aerosol(&Nx, &Ny, &Nz, &Ns,
	     &lwc_cloud_threshold, &current_time, SpecificHumidity_i.GetData(),
	     Temperature_i.GetData(), Pressure_i.GetData(), &next_time,
	     Concentration.GetData(), &Noptions_aer, options_aer.data(),
	     &Ns_aer, &Nbin_aer, &Ncycle_aer, LiquidWaterContent_i.GetData(),
	     Rain_i.GetData(), BinBound_aer.data(), &FixedDensity_aer,
	     Density_aer.data(), CoagulationCouple.data(),
	     CoagulationFirstIndex.data(), CoagulationSecondIndex.data(),
	     CoagulationPartitionCoefficient.data(), VerticalInterface.data(),
	     Concentration_aer.GetData(), &Ns_source_aer, &NbinMax_source_aer,
	     &Nz_source_aer, source_index_species_aer.data(),
	     source_index_bin_aer.data(), Source_aer_i.GetData(), pH.GetData());
  }


  //! Checks whether a given field is required by this module.
  /*! Checks whether a given field must be available in the underlying model
    for this module to perform properly.
    \param field the field name.
    \return True is the field is required, false otherwise.
   */
  template<class T>
  bool ChemistryRACM_SIREAM<T>::IsRequired(string field)
  {
    if (field == "wet_diameter_aer") return true;
    return false;
  }


  //! Checks whether a given field is computed by this module.
  /*! Checks whether a given field is computed by this module and updated in
    the underlying model.
    \param field the field name.
    \return True is the field is computed, false otherwise.
   */
  template<class T>
  bool ChemistryRACM_SIREAM<T>::IsComputed(string field)
  {
    if (field == "pH") return true;
    return false;
  }

} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODULES_CHEMISTRY_CHEMISTRYRACM_SIREAM_CXX
#endif
