C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE COMPUTE_BIDISPERSE_COAGULATION_KERNEL(temp,airfmp,
     &     d1,d2,m1,m2,kercg)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes coagulation kernels for bidispersed 
C     aerosols.
C     
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     TEMP   : temperature [K].
C     AIRFMP : air free mean path ([m]).
C     D1     : diameter of the first coagulating particle ([m]).
C     D2     : diameter of the second coagulating particle ([m]).
C     M1     : mass of the first coagulating particle ([g]).
C     M2     : mass of the second coagulating particle ([g]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     KERCG : coagulation kernel ([m3.s-1]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      DOUBLE PRECISION temp,airfmp
      DOUBLE PRECISION d1,m1,d2,m2,kercg 

      DOUBLE PRECISION PI,FRAC3,Dmin,Mmin,Knmin
      DOUBLE PRECISION Stick,Kb,MUair,Knmax

      PARAMETER (PI   =3.141592653589D0,
     &     FRAC3=0.333333333333D0,
     &     Kb   =1.381D-23,     ! Boltzmann constant J.K-1
     &     MUair=1.725D-05,     ! air dynamic viscosity kg.m-1.s-1
     &     Stick=1.D0,          ! sticking probability 0< <1
     &     Dmin =1.D-03,        ! min aero diam m
     &     Mmin =1.D-17,        ! min aero mass g (*)
     &     Knmin=0.02D0,        ! min threshold of Knudsen number
     &     Knmax=10.D0)

C     (*) Mmin corresponds roughly to an aerosol 
C     of diameter Dmin and a specific mass 
C     equal to 1.D-06 g.m-3.

      DOUBLE PRECISION kn1,kn2,knp
      DOUBLE PRECISION cdif1,cdif2,cdifp
      DOUBLE PRECISION delta1,delta2,deltap
      DOUBLE PRECISION vm1,vm2,vmp
      DOUBLE PRECISION cor1,cor2,l1,l2
      DOUBLE PRECISION dp,lambdap
      DOUBLE PRECISION d1c,d2c,m1c,m2c

C     *****eventually correct inputs
      d1c=DMAX1(d1,Dmin)
      d2c=DMAX1(d2,Dmin)
      m1c=DMAX1(m1,Mmin)
      m2c=DMAX1(m2,Mmin)
      
C     *****knudsen number for particles in air
      kn1=2.D0*airfmp/d1c       ! adim
      kn2=2.D0*airfmp/d2c       ! adim
      
C     *****diffusion coefficient
      cdif1= Kb                 ! J.K-1
     &     *temp                ! K
     &     /3.D0/PI             ! adim
     &     /MUair               ! air viscosity kg.m-1.s-1
     &     /d1c                 ! m
     &     *1.D06               ! convert m -> m
      
      cor1= ( 5.D0+( kn1*( 4.D0
     &     +6.D0*kn1*( 1.D0+3.D0*kn1))) )
     &     /( 5.D0+kn1*((8.D0+PI)*kn1- 1.D0) )

      cdif1=cdif1*cor1          ! m2.s-1

      cdif2= Kb                 ! J.K-1
     &     *temp                ! K
     &     /3.D0/PI             ! adim
     &     /MUair               ! air viscosity kg.m-1.s-1
     &     /d2c                 ! m
     &     *1.D06               ! convert m -> m
      

      cor2= ( 5.D0+( kn2*( 4.D0
     &     +6.D0*kn2*( 1.D0+3.D0*kn2))) )
     &     /( 5.D0+kn2*((8.D0+PI)*kn2- 1.D0) )
      
      cdif2=cdif2*cor2          ! m2.s-1

C     ******mean quadratic aerosol velocity  ! m.s-1
      vm1=DSQRT( 8.D0/PI        ! adim
     &     *Kb                  ! J.K-1
     &     *temp                ! K
     &     /m1c                 ! g
     &     *1.D09 )             ! convert g -> kg
      
      vm2=DSQRT( 8.D0/PI        ! adim
     &     *Kb                  ! J.K-1
     &     *temp                ! K
     &     /m2c                 ! g
     &     *1.D09 )             ! convert g -> kg

C     ******average values
      cdifp=(cdif1+cdif2)*5.D-01 ! m2.s-1
      vmp=DSQRT(vm1*vm1+vm2*vm2) ! m.s-1
      dp=(d1c+d2c)*5.D-01       ! m
      
      lambdap= 8.D0/PI
     &     *cdifp               ! m2.s-1
     &     /vmp                 ! m.s-1
     &     *1.D06               ! convert m to m
      
      knp= 2.D0
     &     *lambdap             ! m
     &     /dp                  ! m

      IF (knp.LE.knmin) THEN
         
         CALL COMPUTE_COAGULATION_CONTINUOUS(dp,cdifp,kercg)
         
      ELSEIF (knp.GE.knmax) THEN
         
         CALL COMPUTE_COAGULATION_FREE_MOLECULAR(dp,vmp,
     &        Stick,kercg)

      ELSE
                                ! free mean path of aerosols 
         l1= 8.D0/PI            ! adim
     &        *cdif1            ! m2.s-1
     &        /vm1              ! m.s-1
     &        *1.D06            ! convert m -> m

         l2= 8.D0/PI            ! adim
     &        *cdif2            ! m2.s-1
     &        /vm2              ! m.s-1
     &        *1.D06            ! convert m -> m

                                ! aerosol knudsen number
         kn1=l1/d1c             ! adim
         kn2=l2/d2c             ! adim

         delta1=d1c             ! m
     &        *( ( (1.D0+kn1)**3.D0
     &        -(1.D0+kn1*kn1)**1.5D0 )
     &        *FRAC3/kn1- 1.D0)
         
         delta2=d2c             ! m
     &        *( ( (1.D0+kn2)**3.D0
     &        -(1.D0+kn2*kn2)**1.5D0 )
     &        *FRAC3/kn2- 1.D0)


         IF ( delta1*delta1 + delta2*delta2.LE.0.D0) THEN
            WRITE(6,*)'Coagulation kernel: sqrt(<0)'
            STOP
         ENDIF

         deltap=DSQRT( delta1*delta1
     &        +delta2*delta2 )
         
         CALL COMPUTE_COAGULATION_FREE_TRANSITION(dp,cdifp,deltap,
     &        vmp,Stick,kercg)
      ENDIF

      END

