C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE COMPUTE_BINARY_NUCLEATION_KERNEL(rh,
     &     temp,natmp,jnucl,
     &     ntot,xstar,dpnucl)


C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes the binary nucleation rate (H2SO4/H20)
C     on the basis of the parameterization of:
C     Veahkamaki et al, JGR Atmosphere, vol 107, D22, p4622, 2002.           
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     RH   : relative humidity 0<RH<1 ([]).
C     TEMP : temperature              ([K]).
C     NA   : gas H2SO4 concentration  ([#molec.cm-3]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     JNUCL  : nucleation rate ([#part.cm-3.s-1]).
C     NTOT   : number of molecules in the critical cluster size ([]).
C     XSTAR  : mol fraction of H2SO4 in the nucleated aerosol ([]).
C     DPNUCL : nucleation diameter  ([nm]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2003: Bruno Sportisse, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE 
      
      DOUBLE PRECISION rh,temp,na,natmp,xstar
      DOUBLE PRECISION jnucl,ntot,dpnucl
      
      DOUBLE PRECISION lnrh,lnrh2,lnrh3,fa(10)
      DOUBLE PRECISION t2,t3,lnna,lnna2,lnna3

      DOUBLE PRECISION TMIN,TMAX,
     &     RHMIN,RHMAX,
     &     NAMIN,NAMAX
      PARAMETER (TMIN=230.15D0,TMAX=300.15D0,
     &     RHMIN=0.01D0,RHMAX=1.0D0,
     &     NAMIN=1.D04,NAMAX=1.D11)

C     test if parameterization is valid
      
      na = natmp
      IF (na.GT.NAMAX) THEN
         na = NAMAX
      ENDIF
      IF (temp.LT.TMIN.OR.temp.GT.TMAX) THEN
         dpnucl = 0.d0
         jnucl = 0.d0
         xstar = 0.d0
         Ntot = 0.d0
      ELSEIF (rh.GT.RHMAX) THEN
         dpnucl = 0.d0
         jnucl = 0.d0     
         xstar = 0.d0
         Ntot = 0.d0
      ELSEIF(na.LT.NAMIN.or.RH.LT.RHMIN) THEN
         dpnucl = 0.d0
         jnucl = 0.d0
         xstar = 0.d0
         Ntot = 0.d0
      ELSE
         lnna=DLOG(na)
         lnna2=lnna*lnna
         lnna3=lnna2*lnna
         lnrh=DLOG(rh)
         lnrh2=lnrh*lnrh
         lnrh3=lnrh2*lnrh
         t2=temp*temp
         t3=t2*temp
         
C     Compute mole fraction of h2so4 
C     in the critical cluster XSTAR.
         
         xstar= 0.740997D0 
     &        -2.66379D-3*temp
     &        -3.49998D-3*lnna 
     &        +5.04022D-5*temp*lnna
     &        +2.01048D-3*lnrh 
     &        -1.83289D-4*temp*lnrh
     &        +1.57407D-3*lnrh2
     &        -1.79059D-5*temp*lnrh2
     &        +1.84403D-4*lnrh3
     &        -1.50345D-6*temp*lnrh3
         
C     Compute nucleation rate.
         
         CALL VEAHKAMAKI_COEFFICIENTS(temp,t2,t3,xstar,fa)
         
         jnucl=DEXP( fa(1)
     &        +fa(2)*lnrh 
     &        +fa(3)*lnrh2
     &        +fa(4)*lnrh3
     &        +fa(5)*lnna
     &        +fa(6)*lnrh*lnna
     &        +fa(7)*lnrh2*lnna
     &        +fa(8)*lnna2
     &        +fa(9)*lnrh*lnna2
     &        +fa(10)*lnna3 )
         
C     Compute total number of molecules
C     in the critical cluster ntot
         
         CALL VEAHKAMAKI_COEFFICIENTS_NUMBER(temp,t2,t3,xstar,fa)
         
         ntot=DEXP( fa(1)
     &        +fa(2)*lnrh
     &        +fa(3)*lnrh2
     &        +fa(4)*lnrh3
     &        +fa(5)*lnna
     &        +fa(6)*lnrh*lnna
     &        +fa(7)*lnrh2*lnna
     &        +fa(8)*lnna2
     &        +fa(9)*lnrh*lnna2
     &        +fa(10)*lnna3 )
         
C     Compute cluster diameter in nm
         
         dpnucl=2.D0*DEXP( -1.6524245D0
     &        +0.42316402D0*xstar
     &        +0.3346648D0*DLOG(ntot) )
      ENDIF
      
      END 
      
      
      
      SUBROUTINE VEAHKAMAKI_COEFFICIENTS(temp,t2,t3,xstar,fa)
      
C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This suboutine computes the coefficients needed for the nucleation 
C     kernels coefficients on the basis of the parametrization given in:
C     Veahkamaki et al, JGR Atmosphere, vol 107, D22, p4622, 2002.           
C     
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     TEMP : temperature ([K]).
C     T2   : square of the temperature.
C     T3   : temperature to power three.
C     XSTAR: mole fraction  ([]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     FA : nucleation kernel coefficients.
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2003: Bruno Sportisse, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE 
      
      DOUBLE PRECISION xstar,temp,t2,t3,fa(10)

      integer jj,nn
      DOUBLE PRECISION wa(50)

      DATA wa/0.14309D0 , 2.21956D0 ,-2.73911D-2,
     &     7.22811D-5, 5.91822D0 , 0.117489D0,
     &     0.462532D0,-1.18059D-2, 4.04196D-5,
     &     1.57963D01,-0.215554D0,-8.10269D-2,
     &     1.43581D-3,-4.7758D-6 ,-2.91297D0,
     &     -3.58856D0 , 4.9508D-2 ,-2.1382D-4,
     &     3.10801D-7,-2.93333D-2, 1.14598D0,
     &     -0.600796D0, 8.64245D-3,-2.28947D-5,
     &     -8.44985D0 , 2.15855D0 , 8.08121D-2,
     &     -4.07382D-4,-4.01957D-7, 0.721326D0,
     &     1.6241D0  ,-1.60106D-2, 3.77124D-5,
     &     3.21794D-8,-1.13255D-2, 9.71682D0,
     &     -0.115048D0, 1.57098D-4, 4.00914D-7,
     &     0.71186D0 ,-1.05611D0 , 9.03378D-3,
     &     -1.98417D-5, 2.46048D-8,-5.79087D-2,
     &     -0.148712D0, 2.83508D-3,-9.24619D-6,
     &     5.00427D-9,-1.27081D-2/

      DO jj=1,10
         nn=(jj-1)*5
         
         fa(jj)= wa(nn+1)
     &        +wa(nn+2)*temp
     &        +wa(nn+3)*t2
     &        +wa(nn+4)*t3
     &        +wa(nn+5)/xstar
      END DO

      END 


      SUBROUTINE VEAHKAMAKI_COEFFICIENTS_NUMBER(temp,t2,t3,xstar,fa)
C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This suboutine computes the coefficients needed for the number of 
C     the nucleated molecules. 
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     TEMP : temperature ([K]).
C     T2   : square of the temperature.
C     T3   : temperature to power three.
C     XSTAR: mole fraction  ([]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     FA : coefficients for the computation of the number of nucleated
C     # molecules (NTOT).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2003: Bruno Sportisse, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      DOUBLE PRECISION xstar,temp,t2,t3,fa(10)

      integer jj,nn
      double precision wa(50)

      DATA wa/-2.95413D-3 ,-9.76834D-2 , 1.02485D-3,
     &     -2.18646D-6 ,-0.101717D0 ,-2.05064D-3,
     &     -7.58504D-3 , 1.92654D-4 ,-6.7043D-7,
     &     -0.255774D0 , 3.22308D-3 , 8.52637D-4,
     &     -1.54757D-5 , 5.66661D-8 , 3.38444D-2,
     &     4.74323D-2 ,-6.25104D-4 , 2.65066D-6,
     &     -3.67471D-9 ,-2.67251D-4 ,-1.25211D-2,
     &     5.80655D-3 ,-1.01674D-4 , 2.88195D-7,
     &     9.42243D-2 ,-3.8546D-2  ,-6.72316D-4,
     &     2.60288D-6 , 1.19416D-8 ,-8.51515D-3,
     &     -1.83749D-2 , 1.722072D-4,-3.71766D-7,
     &     -5.14875D-10, 2.6866D-4  ,-6.19974D-2,
     &     9.06958D-4 ,-9.11728D-7 ,-5.36796D-9,
     &     -7.74234D-3 , 1.21827D-2 ,-1.0665D-4,
     &     2.5346D-7  ,-3.63519D-10, 6.10065D-4,
     &     3.20184D-4 ,-1.74762D-5 , 6.06504D-8,
     &     -1.42177D-11, 1.35751D-4/

      DO jj=1,10
         nn=(jj-1)*5

         fa(jj)= wa(nn+1)
     &        +wa(nn+2)*temp
     &        +wa(nn+3)*t2
     &        +wa(nn+4)*t3
     &        +wa(nn+5)/xstar
      END DO 

      END

      SUBROUTINE NA_THRESHOLD_VEAHKAMAKI(rh,temp,nanucl)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes the critical H2SO4 concentration for
C     for nucleation rate on the basis of:
C     Veahkamaki et al, JGR Atmosphere, vol 107, D22, p4622, 2002.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     RH   : relative humidity 0<RH<1 ([]).
C     TEMP : temperature ([K]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     NANUCL : gas threshold H2SO4 concentration ([#molec.cm-3]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2004 : optimization (Kathleen Fahey, CEREA).     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2003: Bruno Sportisse, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE 
      
      DOUBLE PRECISION rh,temp,nanucl

      DOUBLE PRECISION lnrh, invtemp

      lnrh = DLOG(rh)
      invtemp = 1.d0 / temp

C     ******Compute threshold h2so4 concentration
C     ******that produces the nucleation rate 1.#part.cm-3.
      
      nanucl=DEXP(  -2.79243D02
     &     +1.17300D+01 * rh
     &     + ( 2.27009D+04
     &     -1.08864D+03 * rh   ) * invtemp
     &     + ( 1.14436D+00
     &     -3.02331D-02 * rh
     &     -1.30254D-03 * temp ) * temp
     &     + (-6.38697D+00
     &     +8.54980D+02 * invtemp
     &     +8.79662D-03 * temp ) * lnrh )

      END 

