C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE COMPUTE_COAGULATION_FREE_TRANSITION(dp,
     &     cdifp,deltap,
     &     vmp,stick,kercg)
      
C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes coagulation kernels for monodispersed 
C     aerosols in the free transition regime.
C     
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     DP    : aerosol diameter                 ([m]).
C     VMP   : mean particle velocity           ([m.s-1]).
C     STICK : sticking probability 0<STICK <1  ([])).
C     CDIFP : diffusion coefficient            ([m2.s-1]).
C     DELTAP: particle Knudsen number          ([m]).
C     
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     KERCG : coagulation kernel    ([m3.s-1]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      DOUBLE PRECISION PI
      PARAMETER (PI  =3.141592653589D0)

      DOUBLE PRECISION dp,cdifp,deltap
      DOUBLE PRECISION vmp,stick,kercg

      DOUBLE PRECISION beta,dpp

      dpp=dp*1.D-06             ! convert m to m

      beta= 1.D0/
     &     ( dp/(dp+deltap)     ! adim
     &     +8.D0*stick          ! adim
     &     *cdifp               ! m2.s-1
     &     /vmp                 ! m.s-1
     &     /dpp )               ! m
      
      kercg= 8.D0*PI            ! adim
     &     *cdifp               ! m2.s-1
     &     *dpp                 ! m
     &     *beta                ! adim

      END



      SUBROUTINE COMPUTE_COAGULATION_FREE_MOLECULAR(dp,vmp,
     &     stick,kercg)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes coagulation kernels for monodispersed 
C     aerosols in the free molecular regime.
C     
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     DP    : aerosol diameter       ([m]).
C     VMP   : mean particle velocity ([m.s-1]).
C     STICK : sticking probability 0<STICK <1 ([])).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     KERCG : coagulation kernel    ([m3.s-1]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      DOUBLE PRECISION PI
      PARAMETER (PI  =3.141592653589D0)

      DOUBLE PRECISION dp,vmp,stick,kercg

      DOUBLE PRECISION dpp

      dpp=dp*1.D-06             ! convert m to m
      
      kercg= PI                 ! adim
     &     *dpp*dpp             ! m2
     &     *vmp                 ! m.s-1
     &     *stick               ! adim  

      END


      SUBROUTINE COMPUTE_COAGULATION_CONTINUOUS(dp,cdifp,kercg)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes coagulation kernels for monodispersed 
C     aerosols in the continuous regime.
C     
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     DP    : aerosol diameter      ([m]).
C     CDIFP : diffusion coefficient ([m2.s-1]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     KERCG : coagulation kernel    ([m3.s-1]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      DOUBLE PRECISION PI
      PARAMETER (PI=3.141592653589D0)

      DOUBLE PRECISION dp,cdifp,kercg

      DOUBLE PRECISION dpp

      dpp=dp*1.D-06             ! convert m to m
      
      kercg= 8.D0*PI            ! adim
     &     *cdifp               ! m2.s-1
     &     *dpp                 ! m

      END

