C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE COMPUTE_TERNARY_NUCLEATION(rh,temp,natmp,mrtmp,jnucl,
     &     ntoth2so4,ntotnh3,dpnucl)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes a ternary nucleation rate on the basis of
C     the parameterization proposed in:
C     Napari et al, JGR Atmosphere, vol 107, D19, 4381 2002.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     RH       : relative humidity 0< <1 ([]).
C     TEMP     : temperature ([K]).
C     NATMP    : gas-phase H2SO4 concentration ([#molec.cm-3]).
C     MRTMP    : mixing ratio of NH3 ([ppt]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     JNUCL     : nucleation rate ([#part.cm-3.s-1]).
C     NTOTH2SO4 : number of molecules of H2SO4 in the critical cluster.
C     NTOTNH3   : number of molecules of NH3 in the critical cluster.
C     DPNUCL    : nucleation diameter ([nm]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     2005/06/07: remove bug + put a0, a1, a2, and a3 in DATA
C     2005-11-29: remove bug for maximal threshold for jnucl.
C     (Marilyne Tombette).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Karine Sartelet, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE 

**************************************************
*     *
*     Parameterization of the nucleation rate fit   *
*     in Napari et al. JGR 2002, vol 107, D19, 4381 *
*     *
**************************************************
      DOUBLE PRECISION a0(20), a1(20), a2(20), a3(20)
******
      DATA a0 / -0.355297d0, 3.13735d0, 19.0359d0,
     &     1.07605d0, 6.0916d0, 0.31176d0, 
     &     -0.0200738d0 ,0.165536d0 ,6.52645d0,
     &     3.68024d0,-0.066514d0,0.65874d0,
     &     0.0599321d0,-0.732731d0,0.728429d0 ,  
     &     41.3016d0,-0.160336d0,8.57868d0,  
     &     0.0530167d0, -2.32736d0 /
      DATA a1 / -33.8449d0,-0.772861d0,-0.170957d0,
     &     1.48932d0,-1.25378d0,1.64009d0,     
     &     -0.752115d0,3.26623d0,-0.258002d0,   
     &     -0.204098d0,-7.82382d0, 0.190542d0,     
     &     5.96475d0,-0.0184179d0,3.64736d0,    
     &     -0.35752d0,0.00889881d0,-0.112358d0,
     &     -1.98815d0,0.0234646d0  /
      DATA a2 / 0.34536d0,0.00561204d0,0.000479808d0,
     &     -0.00796052d0,0.00939836d0,-0.00343852d0,
     &     0.00525813d0,-0.0489703d0,0.00143456d0,
     &     0.00106259d0,0.0122938d0,-0.00165718d0,   
     &     -0.0362432d0,0.000147186d0,-0.027422d0,
     &     0.000904383d0,-0.0000539514d0,0.000472626d0,
     &     0.0157827d0,-0.000076519d0 /
      DATA a3 / -0.000824007d0,-9.74576D-06,-4.14699D-07,
     &     7.61229D-06,-0.0000174927d0,-0.0000109753d0,
     &     -8.98038D-06,0.000146967d0,-2.02036D-06,
     &     -1.2656D-06,0.0000618554d0,3.41744D-06,
     &     0.0000493337d0,-2.37711D-07,0.0000493478d0,
     &     -5.73788D-07,8.39522D-08,-6.48365D-07,
     &     -0.0000293564d0,8.0459D-08 /
**************************************************

      DOUBLE PRECISION rh,temp,natmp,na,mrtmp,mr
      DOUBLE PRECISION jnucl,ntoth2so4,ntotnh3,dpnucl

      double precision fa
      EXTERNAL fa
      DOUBLE PRECISION lnrh,lnmr2,lnmr,lnj,lnj2
      DOUBLE PRECISION t2,t3,lnna,lnna2

      DOUBLE PRECISION TMIN,TMAX,
     &     RHMIN,RHMAX,
     &     NAMIN,NAMAX,MRMIN,MRMAX
      PARAMETER (TMIN=240.D0,TMAX=300.D0,
     &     RHMIN=0.05D0,RHMAX=0.95D0,
     &     NAMIN=1.D04,NAMAX=1.D09,
     s     MRMIN=0.1d0,MRMAX=100.d0)

C     Test if parameterization is valid.

      na = natmp
      mr = mrtmp
      IF (temp.LT.TMIN.OR.temp.GT.TMAX
     &     .OR.rh.LT.RHMIN.OR.rh.GT.RHMAX
     &     .OR.na.LT.NAMIN.OR.mr.LT.MRMIN) THEN
         dpnucl = 0.d0
         jnucl = 0.d0
         ntoth2so4 = 0.d0
         ntotnh3 = 0.d0
         
      ELSE
         na=DMIN1(na,NAMAX)
         mr=DMIN1(mr,MRMAX)
         
         lnna = DLOG(na)
         lnna2 = lnna*lnna
         lnrh = DLOG(rh)
         lnmr = DLOG(mr)
         lnmr2 = lnmr*lnmr
         t2 = temp*temp
         t3 = t2*temp
         
         lnj=( -84.7551d0 + fa(1,temp,t2,t3,a0,a1,a2,a3)/lnna + 
     s        fa(2,temp,t2,t3,a0,a1,a2,a3)*lnna + 
     s        fa(3,temp,t2,t3,a0,a1,a2,a3)*lnna2 + 
     s        fa(4,temp,t2,t3,a0,a1,a2,a3)*lnmr + 
     s        fa(5,temp,t2,t3,a0,a1,a2,a3)*lnmr2+
     s        fa(6,temp,t2,t3,a0,a1,a2,a3)*rh + 
     s        fa(7,temp,t2,t3,a0,a1,a2,a3)*lnrh + 
     s        fa(8,temp,t2,t3,a0,a1,a2,a3)*lnmr/lnna+
     s        fa(9,temp,t2,t3,a0,a1,a2,a3)*lnna*lnmr + 
     s        fa(10,temp,t2,t3,a0,a1,a2,a3)*rh*lnna + 
     s        fa(11,temp,t2,t3,a0,a1,a2,a3)*rh/lnna + 
     s        fa(12,temp,t2,t3,a0,a1,a2,a3)*rh*lnmr + 
     s        fa(13,temp,t2,t3,a0,a1,a2,a3)*lnrh/lnna + 
     s        fa(14,temp,t2,t3,a0,a1,a2,a3)*lnrh*lnmr+
     s        fa(15,temp,t2,t3,a0,a1,a2,a3)*lnmr2/lnna + 
     s        fa(16,temp,t2,t3,a0,a1,a2,a3)*lnna*lnmr2 +
     s        fa(17,temp,t2,t3,a0,a1,a2,a3)*lnna2*lnmr +
     s        fa(18,temp,t2,t3,a0,a1,a2,a3)*rh*lnmr2 + 
     s        fa(19,temp,t2,t3,a0,a1,a2,a3)*rh*lnmr/lnna + 
     s        fa(20,temp,t2,t3,a0,a1,a2,a3)*lnna2*lnmr2)

         jnucl = DEXP(lnj)
         
         IF(jnucl.lt.1.d-5) jnucl = 0.d0

         jnucl=DMIN1(jnucl,1.d6)

C     Compute total number of molecules in the critical cluster:
C     This is NTOT.
         
         lnj2 = lnj*lnj
         ntoth2so4 = 38.1645d0 + 0.774106d0*lnj + 0.00298879d0*lnj2  
     s        -0.357605d0*temp - 0.00366358d0*temp*lnj 
     s        +0.0008553d0*temp**2
         
         ntotnh3 = 26.8982d0 + 0.682905d0*lnj + 0.00357521d0*lnj2
     s        -0.265748d0*temp-0.00341895d0*temp*lnj
     s        +0.000673454d0*temp**2
         
C     Compute cluster diameter in nm.
         
         dpnucl = 2.D0* (0.141027d0 - 0.00122625d0*lnj 
     s        -7.82211d-06*lnj2 - 0.00156727d0*temp
     s        -0.00003076d0*temp*lnj 
     s        +0.0000108375d0*temp**2)

      ENDIF
      
      END 
      
C-------------------------------------------------
      
      FUNCTION fa(i,temp,t2,t3,a0,a1,a2,a3)
      
      DOUBLE PRECISION fa,temp,t2,t3
      DOUBLE PRECISION a0(20),a1(20),a2(20),a3(20)
      
      fa = a0(i) + a1(i)*temp + a2(i)*t2 + a3(i)*t3

      END

