C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE HETRXN(Nesp,NS,temp,press,ICLD,lwctmp,
     &     WetDiam,granulo,rkin1,rkin2,rkin3,rkin4,
     &     dsf_aero,ispeclost,Wmol,LWCmin)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine calculates the kinetic constants
C     of the heterogeneous reactions that happen on the surface of particles, as
C     described in Jacob (2000).  It is considered an irreversible
C     process with a first-order reaction rate.
C     
C     HO2  -->  0.5 H2O2
C     NO2  -->  0.5 HONO + 0.5 HNO3
C     NO3  -->  HNO3
C     N2O5 -->  2 HNO3
C     
C     REFERENCES: 
C     Jacob, D.J. (2000) Heterogeneous chemistry and tropospheric
C     ozone.  Atmospheric Environment.  34, 2131-2159.
C     
C     Jacob, D.J. (2004) Private communication to B. Sportisse.
C     
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     temp: Temerature ([K])
C     press: Pressure ([Pa])
C     LCL: logical for aqueous chemistry (from user choices).
C     lwctmp: Liquid Water content ([]g.m-3)
C     WetDiam: Aerosol size bins wet diameters ([\mu m]) 
C     granulo: Particles number in each bin.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     rkin1, rkin2, rkin3 and rkin4: kinetic constants of the 4 reactions
C     (in that order)
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
c     
C     2005-05-23: kinetic constants in order to insert heterogeneous
C     reaction in the gaseous sheme (instead of in the aerosol module)
C     2005-11-28: add reactions for HO2 and NO3 on cloud droplet surfaces.
C     (Marilyne Tombette, CEREA)
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Kathleen Fahey, CEREA 
C     
C------------------------------------------------------------------------



      IMPLICIT NONE

      INCLUDE 'CONST.INC'
      INCLUDE 'CONST_A.INC'
      INCLUDE 'hetrxn.inc'
      INCLUDE 'aerpar.inc'
      INCLUDE 'droppar.inc'

      INTEGER Nesp,NS


      INTEGER i,js
      INTEGER ispeclost(4)
      DOUBLE PRECISION Ndroplet

      DOUBLE PRECISION temp,press
      DOUBLE PRECISION rkin1,rkin2,rkin3,rkin4      
      DOUBLE PRECISION vi
      DOUBLE PRECISION ktot(4),A
      DOUBLE PRECISION DIFF

      DOUBLE PRECISION WetDiam(NS)
      DOUBLE PRECISION granulo(NS)
      DOUBLE PRECISION dsf_aero(NS)

      DOUBLE PRECISION lwctmp

      INTEGER ICLOUD
      INTEGER ICLD
      
      DOUBLE PRECISION sigm_tmp,parm_tmp,wmol_tmp
      DOUBLE PRECISION Wmol(Nesp),LWCmin

C     Constants.

      sigm_tmp = SIGM_NO2
      parm_tmp = PARM_NO2
      ICLOUD=0
      IF (ICLD.EQ.1.AND.(lwctmp.GE.LWCmin)) THEN
         ICLOUD=1
      ENDIF

C     REACTION PROBABILITIES

      DO i=1,4
         ktot(i)=0.D0
         wmol_tmp = Wmol(ispeclost(i)+1)
         call COMPUTE_GAS_DIFFUSIVITY(TEMP,PRESS,
     &        sigm_tmp, wmol_tmp, parm_tmp, DIFF)
         
         call COMPUTE_QUADRATIC_MEAN_VELOCITY(TEMP,
     &        wmol_tmp,vi)
         
         do js=1,NS   
            IF ((ICLOUD.NE.1).OR.(dsf_aero(js).lt.dactiv*1.d6)) then
               
C     CALCULATE k FOR EACH SECTION

               A = PI*(WetDiam(js)**2.d0)*granulo(js) !surface * nb_aero
               
C     CALCULATE k FOR AEROSOL DISTRIBUTION
               
               ktot(i) = ktot(i)+ (((WetDiam(js)/(2.d0*DIFF))+4.d0/
     &              (vi*Gamma(i)))**(-1.d0))*A ! rate constant for each rxn

            ENDIF
         enddo

C     If we are in a cloud, then reactions on droplet surface for NO3 and HO2.
         IF ((ICLOUD.EQ.1).and.(i.eq.4)) THEN
            Ndroplet = lwctmp
     &           / (RHOwater * 1.d3 !kg.m-3 -> g.m-3
     &           * (1.d0 / 6.d0) * pi * avdiam ** 3.d0) 
            A = PI*(avdiam**2.d0)*Ndroplet !surface * nb_droplet
            ktot(i) = ktot(i)+ (((avdiam/(2.d0*DIFF))+4.d0/
     &           (vi*Gamma(i)))**(-1.d0))*A ! rate constant for each rxn
            
         ENDIF
      enddo

      rkin1=ktot(1)
      rkin2=ktot(2)
      rkin3=ktot(3)
      rkin4=ktot(4)

      END
