C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE FCOAG(neq,nbin_aer,q,iq,dqdt,couples_coag,
     s      first_index_coag,second_index_coag,coefficient_coag,QT,
     s      XSF,MSF,DSF,XSD,MSD,DSD)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes the source terms for the system of
C     Ordinary Differential Equations defined by the coagulation.    
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     NEQ : number of equations.
C     Q   : aerosol concentration ([g.m-3]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     DQDT : time derivative ([g.m-3.s-1]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     2005/10: Add ITHRM hook to compute diameter (Edouard Debry, CEREA).
C     2005/10: Put KERCOAG routine (Edouard Debry, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      INCLUDE 'param.inc'
      INCLUDE 'dynaero.inc'
      INCLUDE 'pointer.inc'
      INCLUDE 'varp.inc'
      INCLUDE 'meteo.inc'
      INCLUDE 'varg.inc'

      INTEGER neq,nbin_aer
      DOUBLE PRECISION q(neq),dqdt(neq)
      INTEGER iq(NEXT,nbin_aer)

      INTEGER js,jj,jesp,j1,j2
      DOUBLE PRECISION gain(neq),loss(nbin_aer)
      DOUBLE PRECISION qnum(nbin_aer),qesp(nbin_aer)
      DOUBLE PRECISION gain2(nbin_aer)

      DOUBLE PRECISION kercg(nbin_aer,nbin_aer)
      DOUBLE PRECISION fcg(nbin_aer,nbin_aer,nbin_aer)

      INTEGER couples_coag(nbin_aer)
      INTEGER first_index_coag(nbin_aer, 4 * nbin_aer)
      INTEGER second_index_coag(nbin_aer, 4 * nbin_aer)
      DOUBLE PRECISION coefficient_coag(nbin_aer, nbin_aer, nbin_aer)

      DOUBLE PRECISION XSF(nbin_aer),MSF(nbin_aer),DSF(nbin_aer)
      DOUBLE PRECISION XSD(nbin_aer),MSD(nbin_aer),DSD(nbin_aer)

      DOUBLE PRECISION QT(nbin_aer)
      DOUBLE PRECISION VSW(nbin_aer),MSW(nbin_aer)
      DOUBLE PRECISION QINT(NINTIS,nbin_aer),DSW(nbin_aer)
      DOUBLE PRECISION QGEQ(NEXT,nbin_aer),RHOW(nbin_aer)

C     Initialization to zero.

      DO jj=1,neq
         gain(jj)=0.D0
      END DO

C     Local gas/aerosol equilibrium needed for computing the
C     aerosol diameter.

      IF (ITHRM.EQ.0) THEN
C     always recompute diam with isorropia
         INORG2=INORG
         IORG2=IORG
         CALL EQPART(nbin_aer,1,nbin_aer,neq,q,iq,QT,
     s      XSF,MSF,DSF,XSD,MSD,DSD,
     s      QINT,QGEQ,VSW,MSW,DSW,RHOW)
      ELSEIF (ITHRM.EQ.1) THEN
C     always recompute diam with fastdiam (Gerber)
         CALL FASTDIAM(nbin_aer,1,nbin_aer,neq,q,iq,QT,
     s      XSF,MSF,DSF,XSD,MSD,DSD,VSW,MSW,DSW,RHOW)
      ENDIF

C     Coagulation kernels
      DO j2=1,nbin_aer
         DO j1=1,j2
            CALL COMPUTE_BIDISPERSE_COAGULATION_KERNEL(TEMP,AIRFMP,
     &           DSW(j1),DSW(j2),
     &           MSW(j1),MSW(j2),
     &           kercg(j1,j2))
            
                                ! symmetric kernels
            kercg(j2,j1)=kercg(j1,j2)
         END DO
      END DO
      
C     Compute coagulation coefficients
      DO js=1,nbin_aer
         DO jj=1,couples_coag(js)
            j1=first_index_coag(js,jj)
            j2=second_index_coag(js,jj)
            
            fcg(j1,j2,js)= coefficient_coag(j1,j2,js)
     &           *kercg(j1,j2)

                                ! symmetric coef
            fcg(j2,j1,js)=fcg(j1,j2,js)
         END DO
      END DO

C     Compute coagulation gain

                                ! for number distribution
      DO js=1,nbin_aer
         qnum(js)=q(js)
      END DO

      CALL CALCGAIN(nbin_aer,qnum,qnum,fcg,gain2,couples_coag,
     s      first_index_coag,second_index_coag)

      DO js=1,nbin_aer
         gain(js)=gain2(js)
      END DO

                                ! for each species
      DO jesp=E1,E2
         DO js=1,nbin_aer
            jj=IQ(jesp,js)
            qesp(js)=q(jj)
         END DO

         CALL CALCGAIN(nbin_aer,qnum,qesp,fcg,gain2,couples_coag,
     s      first_index_coag,second_index_coag)

         DO js=1,nbin_aer
            jj=IQ(jesp,js)
            gain(jj)=gain2(js)
         END DO
      END DO
      
C     Compute coagulation loss

      CALL CALCLOSS(nbin_aer,qnum,kercg,loss)

C     Update time derivatives      

                                ! for number distribution
      DO js=1,nbin_aer
         dqdt(js)= dqdt(js)
     &        +5.D-01*gain(js)
     &        -q(js)*loss(js)
      END DO
      
                                ! for each species
      DO jesp=E1,E2
         DO js=1,nbin_aer
            jj=IQ(jesp,js)
            dqdt(jj)= dqdt(jj)
     &           +gain(jj)
     &           -q(jj)*loss(js)
         END DO
      END DO

      END

