C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE chem (Nx,Ny,Nz,nesp,nr,nrphot,nreactphot,nemis,
     $     nemisspecies,nzemis,LWCmin,Wmol,ts,DLattenuation,DLhumid,
     $     DLtemp,DLpress,DLCsourc,DLCphotolysis_rates,tf,
     $     DLattenuationf,DLhumidf,DLtempf, DLpressf,DLCsourcf,
     $     DLCphotolysis_ratesf,ncycle,dlon,dlat,DLconc,icld,
     $     iheter,nesp_aer,nbin_aer,DLLWC,bin_bound_aer,
     $     fixed_density_aer,wet_diameter_aer,hetero_species_index,
     $     DLconc_aer)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes one timestep for gas-phase chemistry RADM.
C     Chemical kinetics is solved in each grid cell.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     Nx: number of cells in X direction (longitude).
C     Ny: number of cells in Y direction (latitude).
C     Nz: number of vertical levels.
C     NESP: number of gas species.
C     NR: number of gas reactions.
C     NRPHOT: number of photolytic reactions.
C     NREACTPHOT: index of photolysis reactions.
C     NEMIS: number of gas with emissions.
C     NEMISSPECIES: index of gas species with emissions.
C     NZEMIS: highest vertical level for gas volumic emissions.
C     LWCmin: air liquid water content threshold.
C     Wmol: molar mass of gas species ([g/mol]).
C     TS: initial time (GMT, computed from January 1st, [s]).
C     DLATTENUATION: 3D cloud attenuation field at initial time.
C     DLHUMID: 3D specific humidity field at initial time ([%]).
C     DLTEMP: 3D temperature field at initial time ([K]).
C     DLPRESS: 3D pressure field at initial time ([Pa]).
C     DLCSOURC: array of chemical volumic emissions at initial time
C     # ([\mu.g/m^3/s]).
C     DLCPHOTOLYSIS_RATES: photochemical kinetic rates
C     # at initial time ([s^{-1}]).
C     TF: final time (GMT, computed from January 1st, [s]).
C     DLATTENUATIONf: 3D cloud attenuation field at final time.
C     DLHUMIDf: 3D specific humidity field at final time ([%]).
C     DLTEMPf: 3D temperature field at final time ([K]).
C     DLPRESSf: 3D pressure field at final time ([Pa]).
C     DLCSOURCf: array of chemical volumic emissions at final time
C     # ([\mu.g/m^3/s]).
C     DLCPHOTOLYSIS_RATESf: photochemical kinetic rates
C     # at final time ([s^{-1}]).
C     NCYCLE: number of cycle in chemistry computation.
C     DLON: longitude discretization.
C     DLAT: latitude discretization.
C     ICLD: take cloud into account (1).
C     IHETER: take heterogeneous reactions into account (1).
C     NESP_AER: number of aerosol species.
C     NBIN_AER: number of aerosol bins.
C     DLLWC: 3D air liquid water content ([fraction]).
C     BIN_BOUND_AER: aerosol diameters at bin bounds.
C     FIXED_DENSITY_AER: fixed aerosol density ([g/m^3]).
C     WET_DIAMETER_AER: 4D aerosol wet diameters ([\mu.m]).
C     HETERO_SPECIES_INDEX: index of gas species with heterogeneous reactions.
C
C     -- INPUT/OUTPUT VARIABLES
C     
C     DLCONC: array of 3D gas concentrations ([\mu.g/m^3]). 
C     DLCONC_AER: array of 3D aerosol concentrations ([\mu.g/m^3]). 
C     # Before entry, it is given at initial time of the timestep.
C     # On exit, it is computed at final time of the timestep.
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2002/02/26: new treatment of sources (Jaouad Boutahar, CEREA).
C     2006/09/29: updated header (Edouard Debry).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Denis Qulo, CEREA, June 2001.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      INCLUDE 'CONST.INC'
      INCLUDE 'CONST_A.INC'

      DOUBLE PRECISION ts,tf,Dt
      DOUBLE PRECISION tschem,tfchem

      integer nx,ny,nz,nesp,nr,nrphot,nemis,nzemis

      DOUBLE PRECISION DLconc(NX,NY,NZ,nesp),ZC(NESP)
      DOUBLE PRECISION DLtemp(NX,NY,NZ),DLtempf(NX,NY,NZ)
      DOUBLE PRECISION DLattenuation(NX,NY,NZ)
      DOUBLE PRECISION DLattenuationf(NX,NY,NZ)
      DOUBLE PRECISION DLhumid(NX,NY,NZ),DLhumidf(NX,NY,NZ)
      DOUBLE PRECISION DLCsourc(Nx,Ny,Nzemis,Nemis)
      DOUBLE PRECISION DLCsourcf(Nx,Ny,Nzemis,Nemis)
      DOUBLE PRECISION ZCsourc(NESP)
      DOUBLE PRECISION ZCsourcf(NESP)
      DOUBLE PRECISION DLRki(Nr),DLRkf(Nr)
      DOUBLE PRECISION DLpress(NX,NY,NZ),DLpressf(NX,NY,NZ)
      DOUBLE PRECISION DLCphotolysis_rates(Nx,Ny,Nz,NRphot)
      DOUBLE PRECISION DLCphotolysis_ratesf(Nx,Ny,Nz,NRphot)

      double precision dlon(nx),dlat(ny)

      integer ncycle
      double precision Wmol(nesp)

      DOUBLE PRECISION Zangzen,Zangzenf
      DOUBLE PRECISION Zatt,Zattf

      DOUBLE PRECISION muzero,DLmuzero
      EXTERNAL muzero

      integer nreactphot(nrphot)
      integer nemisspecies(nemis)

      INTEGER Jt,Ji,Jj,Jk,Jsp,i,Jb,Jemis

      INTEGER nesp_aer,nbin_aer
      DOUBLE PRECISION DLLWC(NX,NY,NZ)
      DOUBLE PRECISION bin_bound_aer(nbin_aer + 1)
      DOUBLE PRECISION wet_diameter_aer(Nx,Ny,Nz,nbin_aer)
      INTEGER hetero_species_index(4)
      DOUBLE PRECISION DLconc_aer(NX,NY,NZ,nbin_aer,nesp_aer)

      INTEGER ICLD,IHETER

      DOUBLE PRECISION wet_diameter_aer_loc(Nbin_aer)
      DOUBLE PRECISION granulo_aer(Nbin_aer), conc_tot
      DOUBLE PRECISION lwca(NX,NY,NZ)
      DOUBLE PRECISION convers_factor(nesp)
      DOUBLE PRECISION LWCmin

      DOUBLE PRECISION MSF(nbin_aer)
      DOUBLE PRECISION DSF(nbin_aer)
      DOUBLE PRECISION MBF(nbin_aer+1)
      DOUBLE PRECISION DBF(nbin_aer+1)

      DOUBLE PRECISION fixed_density_aer,RHOA

C     Conversion \mu.g/m3 to molecules/cm3.
      DO Jsp = 1, nesp
         convers_factor(Jsp) = Navog * 1.D-12 / Wmol(Jsp)
      ENDDO

C     Aerosol density converted in microg / microm^3.
      RHOA = fixed_density_aer * 1.D-09

C     Aerosol discretization converted in microm.
      DO Jb=1,nbin_aer+1
         DBF(Jb) = bin_bound_aer(Jb) * 1.D06
         MBF(Jb) = RHOA * cst_pi6 * DBF(Jb)**3
      ENDDO
      
      DO Jb=1,nbin_aer
         MSF(Jb) = DSQRT(MBF(Jb) * MBF(Jb+1))
         DSF(Jb) = DSQRT(DBF(Jb) * DBF(Jb+1))
      ENDDO

C     Constants.

      Dt = tf-ts

C     Cloud liquid water content (g/m^3)
      DO Jk=1,NZ
         DO Jj=1,NY
            DO Ji=1,NX     
               lwca(Ji,Jj,Jk)=DLLWC(Ji,Jj,Jk)*1000.d0*
     &              DLpress(Ji,Jj,Jk)/101325.d0*28.97d0/Pr/
     &              DLtemp(Ji,Jj,Jk)
            ENDDO
         ENDDO
      ENDDO

C     Loop on grid cells.
      DO Jk=1,NZ
         DO Jj=1,NY
            DO Ji=1,NX

               DO Jsp=1,Nesp
                  ZCsourc(jsp)=0.D0
                  ZCsourcf(jsp)=0.d0
               ENDDO
C     Spatial extraction for volumic sources.
               if (nemis.gt.0) then
                  if (jk.le.nzemis) then
                     DO Jsp=1,Nemis
                        Jemis = nemisspecies(jsp)+1
                        ZCsourc(jemis) = DLCsourc(Ji,Jj,Jk,Jsp)
     $                       * convers_factor(Jemis)
                        ZCsourcf(jemis)= DLCsourcf(Ji,Jj,Jk,Jsp)
     $                       * convers_factor(Jemis)
                     ENDDO
                  else
                     DO Jsp=1,Nemis
                        ZCsourc(nemisspecies(jsp)+1)=0.D0
                        ZCsourcf(nemisspecies(jsp)+1)=0.D0
                     ENDDO
                  endif
               endif

C     Cloud attenuation.
               
               Zatt = DLattenuation(Ji, Jj, Jk)
               Zattf = DLattenuationf(Ji, Jj, Jk)
               
C     Projection.

               DO Jsp=1,Nesp
                  ZC(Jsp) = DLconc(Ji,Jj,Jk,Jsp) * convers_factor(Jsp)
               ENDDO

C     Initialization of granulo for kinetic cte of heterogeneous rxns
           
               DO Jb=1,Nbin_aer
                  conc_tot=0.0D0
                  DO Jsp=1,Nesp_aer
                     conc_tot = conc_tot + DLconc_aer(Ji,Jj,Jk,Jb,Jsp) 
                  ENDDO
C     compute the particle number (mass/geometric mean diameter)
                  granulo_aer(Jb)=conc_tot / MSF(Jb)
                  wet_diameter_aer_loc(Jb) =
     &                 wet_diameter_aer(Ji,Jj,Jk,jb)
               ENDDO

C     Integration of chemistry (eventually with subcycling).

               DO Jt=1,Ncycle
                  tschem=ts+(Jt-1)*Dt/Ncycle
                  tfchem=tschem+Dt/Ncycle
                  DLmuzero=muzero(tschem,Dlon(ji),Dlat(jj))
                  Zangzen=dabs(DACOS(DLmuzero)*180.D0/PI)
                  DLmuzero=muzero(tfchem,Dlon(ji),Dlat(jj))
                  Zangzenf=dabs(DACOS(DLmuzero)*180.D0/PI)

                  CALL Kinetic(Nesp,Nbin_aer,Nr,
     s                 IHETER,ICLD,DLRKi,DLtemp(Ji,Jj,Jk),
     s                 DLhumid(Ji,Jj,Jk),DLpress(Ji,Jj,Jk),Zangzen,
     s                 Zatt,lwca(Ji,Jj,Jk),granulo_aer,
     s                 wet_diameter_aer_loc,DSF,
     s                 hetero_species_index,Wmol,LWCmin)
                  CALL Kinetic(Nesp,Nbin_aer,Nr,
     s                 IHETER,ICLD,DLRKf,DLtempf(Ji,Jj,Jk),
     s                 DLhumidf(Ji,Jj,Jk),DLpressf(Ji,Jj,Jk), Zangzenf,
     s                 Zattf,lwca(Ji,Jj,Jk),granulo_aer,
     s                 wet_diameter_aer_loc,DSF,
     s                 hetero_species_index,Wmol,LWCmin)

C     Photolytic reactions may be read.
                  IF (Nrphot.GT.0) THEN
                     DO i=1,Nrphot
                        DLRKi(Nreactphot(i)+1) = Zatt *
     $                       DLCphotolysis_rates(Ji, Jj, Jk, i)
                        DLRKf(Nreactphot(i)+1) = Zattf *
     $                       DLCphotolysis_ratesf(Ji, Jj, Jk, i)
                     ENDDO
                  ENDIF

                  CALL roschem (NESP,Nr,nemis,ZC,ZCsourc,
     s                 ZCsourcf,tschem,tfchem,DLRki,DLRkf)
               ENDDO

C     Storage in the 3D array of chemical concentrations.

               DO Jsp=1,NESP
                  DLconc(Ji,Jj,Jk,Jsp) = ZC(Jsp) / convers_factor(Jsp)
               ENDDO

            ENDDO               ! loop x.
         ENDDO                  ! loop y.
      ENDDO                     ! loop z.

      END
