# Copyright (C) 2005 CEREA
#     Author: Vivien Mallet
#
# CEREA (http://www.enpc.fr/cerea/) is a joint laboratory of
# ENPC (http://www.enpc.fr/) and EDF R&D (http://www.edf.fr/).
#
# This file is part of AtmoPy package.
# AtmoPy is a tool for data processing and visualization in atmospheric
# sciences.
#
# AtmoPy is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# AtmoPy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License (file ``license'') for more details.
#
# For more information, please see the AtmoPy home page:
#     http://www.enpc.fr/cerea/atmopy/


import unittest
import datetime
from numarray import *
sys.path.insert(0, \
                os.path.split(os.path.dirname(os.path.abspath(__file__)))[0])
from atmopy import observation
sys.path.pop(0)


class MasksTestCase(unittest.TestCase):
    
    date0 = [datetime.datetime(1900, 1, 2), datetime.datetime(1900, 1, 5), \
             datetime.datetime(1900, 1, 6), datetime.datetime(1900, 1, 7), \
             datetime.datetime(1900, 1, 9)]
    date1 = [datetime.datetime(1900, 1, 1), datetime.datetime(1900, 1, 6), \
             datetime.datetime(1900, 1, 9), datetime.datetime(1900, 1, 10), \
             datetime.datetime(1900, 1, 11)]
    date2 = [datetime.datetime(2000, 1, 1), datetime.datetime(2000, 1, 6)]
    date3 = [datetime.datetime(1900, 1, 5)]
    date4 = [datetime.datetime(1900, 1, 9), datetime.datetime(1901, 8, 19)]

    common01 = (array([False, False, True, False, True]), \
                array([False, True, True, False, False]))
    common03 = (array([False, True, False, False, False]), array([True]))
    common04 = (array([False, False, False, False, True]), \
                array([True, False]))
    
    def SameMasks(self, masks, mask_ref):
        return alltrue(masks[0] == mask_ref[0]) \
               and alltrue(masks[1] == mask_ref[1])

    def testCommonDatesBasic(self):
        common = observation.masks_for_common_dates(self.date0, self.date1)
        self.assert_(self.SameMasks(common, self.common01))
        common = observation.masks_for_common_dates(self.date0, self.date2)
        self.assert_([not alltrue(a) for a in common])
        common = observation.masks_for_common_dates(self.date0, self.date3)
        self.assert_(self.SameMasks(common, self.common03))
        common = observation.masks_for_common_dates(self.date0, self.date4)
        self.assert_(self.SameMasks(common, self.common04))

        common = observation.masks_for_common_dates(self.date0, self.date0)
        self.assert_([alltrue(a) for a in common])
        common = observation.masks_for_common_dates(self.date1, self.date1)
        self.assert_([alltrue(a) for a in common])
        common = observation.masks_for_common_dates(self.date3, self.date3)
        self.assert_([alltrue(a) for a in common])

    def testCommonDatesEmpty(self):
        common = observation.masks_for_common_dates(self.date1, [])
        self.assert_([not alltrue(a) for a in common])
        common = observation.masks_for_common_dates([], self.date0)
        self.assert_([not alltrue(a) for a in common])
        common = observation.masks_for_common_dates([], [])
        self.assert_([not alltrue(a) for a in common])

    def testCommonDatesOrder(self):
        common0 = observation.masks_for_common_dates(self.date0, self.date1)
        common1 = observation.masks_for_common_dates(self.date1, self.date0)
        common1 = list(common1)
        common1.reverse()
        self.assert_(self.SameMasks(common0, common1))
        

class FilterTimesteps(unittest.TestCase):

    date0 = [datetime.datetime(1920, 1, 2) + datetime.timedelta(0, 3600 * x)
             for x in range(23)]
    date1 = [datetime.datetime(1920, 1, 2) + datetime.timedelta(0, 3600 * x)
             for x in range(24)]
    date2 = [datetime.datetime(1920, 1, 2) + datetime.timedelta(0, 3600 * x)
             for x in range(30)]
    date3 = [datetime.datetime(1920, 1, 2) + datetime.timedelta(0, 3600 * x)
             for x in range(300)]
    date4 = [datetime.datetime(1920, 1, 2) + datetime.timedelta(x) for x in
             range(10)]
    date5 = [datetime.datetime(1920, 1, 2) + datetime.timedelta(2 * x) for x
             in range(10)]
    date6 = [datetime.datetime(1920, 1, 2, 5) + datetime.timedelta(0, 3600 *
             x) for x in range(23)]
    date7 = [datetime.datetime(1920, 1, 2, 5) + datetime.timedelta(0, 3600 *
             x) for x in range(50)]
    date7_res = [datetime.datetime(1920, 1, 3) + datetime.timedelta(0, 3600 *
             x) for x in range(24)]

    data0 = arange(23)
    data1 = arange(24)
    data2 = arange(30)
    data3 = arange(300)
    data4 = arange(10)
    data5 = arange(10)
    data6 = arange(23)
    data7 = arange(50)

    def testRemoveIncompleteDays(self):
        date, data \
              = observation.remove_incomplete_days(self.date0, self.data0)
        self.assertEqual(date, [])
        self.assertEqual(len(data), 0)
        date, data \
              = observation.remove_incomplete_days(self.date1, self.data1)
        self.assertEqual(date, self.date1)
        self.assertEqual(abs(data - self.data1).max(), 0)
        date, data \
              = observation.remove_incomplete_days(self.date2, self.data2)
        self.assertEqual(date, self.date1)
        self.assertEqual(abs(data - self.data1).max(), 0)
        date, data \
              = observation.remove_incomplete_days(self.date3, self.data3)
        self.assertEqual(len(date), 288)
        self.assertEqual(abs(data - arange(288)).max(), 0)
        date, data \
              = observation.remove_incomplete_days(self.date4, self.data4)
        self.assertEqual(date, self.date4)
        self.assertEqual(abs(data - self.data4).max(), 0)
        date, data \
              = observation.remove_incomplete_days(self.date5, self.data5)
        self.assertEqual(date, self.date5)
        self.assertEqual(abs(data - self.data5).max(), 0)
        date, data \
              = observation.remove_incomplete_days(self.date6, self.data6)
        self.assertEqual(date, [])
        self.assertEqual(len(data), 0)
        date, data \
              = observation.remove_incomplete_days(self.date7, self.data7)
        self.assertEqual(date, self.date7_res)
        self.assertEqual(abs(data - arange(19, 19 + 24)).max(), 0)


    def testRemoveDays(self):
        date, data \
              = observation.remove_days(self.date0, self.data0, 0)
        self.assertEqual(date, self.date0)
        self.assertEqual(abs(data - self.data0).max(), 0)
        date, data \
              = observation.remove_days(self.date0, self.data0, 1)
        self.assertEqual(date, [])
        self.assertEqual(len(data), 0)
        date, data \
              = observation.remove_days(self.date1, self.data1, 1)
        self.assertEqual(date, [])
        self.assertEqual(len(data), 0)
        date, data \
              = observation.remove_days(self.date2, self.data2, 1)
        self.assertEqual(len(date), 6)
        self.assertEqual(abs(data - arange(24, 30)).max(), 0)
        date, data \
              = observation.remove_days(self.date3, self.data3, 4)
        self.assertEqual(len(date), 204)
        self.assertEqual(abs(data - arange(96, 300)).max(), 0)
        date, data \
              = observation.remove_days(self.date4, self.data4, 2)
        self.assertEqual(len(date), 8)
        self.assertEqual(abs(data - arange(2, 10)).max(), 0)
        date, data \
              = observation.remove_days(self.date5, self.data5, 4)
        self.assertEqual(len(date), 8)
        self.assertEqual(abs(data - arange(2, 10)).max(), 0)


if __name__ == "__main__":
    suite = unittest.makeSuite(MasksTestCase)
    suite.addTest(unittest.makeSuite(FilterTimesteps))
    unittest.TextTestRunner(verbosity=2).run(suite)
