// Copyright (C) 2006 CEREA
//     Authors: Hadjira Foudhil, Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the modeling system Polyphemus.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


//! Computes air density on the basis of temperature and pressure.
/*! Formula: AirDensity = Pressure / (287. * Temperature).
  \param Temperature temperature.
  \param Pressure pressure.
  \param AirDensity (output) air density.
*/
template<class T>
void ComputeAirDensity(const Data<T, 3>& Temperature,
		       const Data<T, 3>& Pressure,
		       Data<T, 3>& AirDensity)
{
  const T r = 287.;

  int k, j, i;

  int Nz = AirDensity.GetLength(0);
  int Ny = AirDensity.GetLength(1);
  int Nx = AirDensity.GetLength(2);
    
  for (k = 0; k < Nz; k++)
    for (j = 0; j < Ny; j++)
      for (i = 0; i < Nx; i++)
	AirDensity(k, j, i) = Pressure(k, j, i)
	  / (r * Temperature(k, j, i));
}


//! Computes the saturation humidity.
/*!
  \param temperature temperature (or virtual temperature) (K).
  \param pressure pressure (Pa).
  \return The saturation humidity (kg/kg).
*/
template<class T>
inline T compute_saturation_humidity(T temperature, T pressure)
{
  T pressure_sat = 610.78 * exp(17.2694 * (temperature - 273.15)
				/ (temperature - 35.86));
  return pressure_sat / (1.608 * pressure - 0.608 * pressure_sat);
}


//! Computes the liquid water potential temperature.
/*!
  \param Temperature temperature (K).
  \param Pressure pressure (Pa).
  \param LiquidWaterContent the liquid water content (kg.kg^{-1}).
  \param PotentialTemperatureLW (output) the liquid water potential
  temperature (K).
  \param P0 (optional) standard pressure. Default: 100000 Pa.
  \param L (optional) latent heat of vaporization for water (J.kg^{-1}).
  Default: 2.501e6 J.kg^{-1}.
  \param cp (optional) specific heat of dry air at constant pressure.
  Default: 1005 J.kg^{-1}.K^{-1}.
  \param r (optional) molar gas constant for air.
  Default: 287.0 J.kg^{-1}.K^{-1}.
*/
template<class T, class TG, int N>
void ComputePotentialTemperatureLW(const Data<T, N, TG>& Temperature,
				   const Data<T, N, TG>& Pressure,
				   const Data<T, N, TG>& LiquidWaterContent,
				   Data<T, N, TG>& PotentialTemperatureLW,
				   T P0 = 100000., T L = 2.501e6,
				   T cp = 1005., T r = 287.)
{
  int k, j, i;
  int Nx = PotentialTemperatureLW.GetLength(2);
  int Ny = PotentialTemperatureLW.GetLength(1);
  int Nz = PotentialTemperatureLW.GetLength(0);
  
  for (k = 0; k < Nz; k++)
    for (j = 0; j < Ny; j++)
      for (i = 0; i < Nx; i++)
	{
	  T theta = Temperature(k, j, i)
	    * pow(P0 / Pressure(k, j, i), r / cp);
	  PotentialTemperatureLW(k, j, i) = theta
	    - L / (cp * Temperature(k, j, i))
	    * theta * LiquidWaterContent(k, j, i);
	}
}


/*! \brief Computes the liquid water content on the basis of ambient air
  characteristics and additional water.
*/
/*! It is assumed that water is added to ambient water. On the basis of the
  new water content and ambient air characteristics, the new liquid water
  content and the new specific humidity are computed.
  \param Pressure pressure (Pa).
  \param AmbientLiquidWaterContent ambient liquid water content (kg.kg^{-1}).
  \param AmbientSpecificHumidity ambient specific humidity (kg.kg^{-1}).
  \param PotentialTemperatureLW liquid water potential temperature computed
  with total water (that is, water in ambient air plus
  \a AdditionalWaterContent) (K).
  \param AdditionalWaterContent total water content (liquid and vapor water)
  added to ambient air (kg.kg^{-1}).
  \param LiquidWaterContent (output) updated liquid water content (with
  additional water content) (kg.kg^{-1}).
  \param P0 (optional) standard pressure. Default: 100000 Pa.
  \param L (optional) latent heat of vaporization for water (J.kg^{-1}).
  Default: 2.501e6 J.kg^{-1}.
  \param cp (optional) specific heat of dry air at constant pressure.
  Default: 1005 J.kg^{-1}.K^{-1}.
  \param r (optional) molar gas constant for air.
  Default: 287.0 J.kg^{-1}.K^{-1}.
 */
template<class T, class TG, int N>
void WaterDiagnosis(const Data<T, N, TG>& Pressure,
		    const Data<T, N, TG>& AmbientLiquidWaterContent,
		    const Data<T, N, TG>& AmbientSpecificHumidity,
		    const Data<T, N, TG>& PotentialTemperatureLW,
		    const Data<T, N, TG>& AdditionalWaterContent,
		    Data<T, N, TG>& LiquidWaterContent,
		    T P0 = 100000., T L = 2.501e6,
		    T cp = 1005., T r = 287.)
{
  int k, j, i;
  int Nx = LiquidWaterContent.GetLength(2);
  int Ny = LiquidWaterContent.GetLength(1);
  int Nz = LiquidWaterContent.GetLength(0);

  T T_liquid, saturation_humidity, delta, alpha, additional_liquid_water;

  for (k = 0; k < Nz; k++)
    for (j = 0; j < Ny; j++)
      for (i = 0; i < Nx; i++)
	{
	  // Liquid temperature (K).
	  T_liquid = PotentialTemperatureLW(k, j, i)
	    * pow(P0 / Pressure(k, j, i), -r / cp);
	  
	  // Saturation humidity computed with the liquid temperature
	  // [Sommeria and Deardorff (1977)].
	  saturation_humidity
	    = compute_saturation_humidity(T_liquid, Pressure(k, j, i));

	  // "Distance" to saturation.
	  delta = AdditionalWaterContent(k, j, i)
	    + AmbientSpecificHumidity(k, j, i) - saturation_humidity;
	  alpha = L * L / (cp * r * 1.608 * T_liquid * T_liquid);

	  // Additional liquid water content.
	  additional_liquid_water
	    = delta / (1. + 1. * alpha * saturation_humidity);
	  if (additional_liquid_water < 0.)
	    additional_liquid_water = 0.;
	  
	  // Total liquid water content.
	  LiquidWaterContent(k, j, i) = additional_liquid_water
	    + AmbientLiquidWaterContent(k, j, i);
	}
}
