// Copyright (C) 2006 CEREA
//     Authors: Hadjira Foudhil, Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the modeling system Polyphemus.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


//////////////
// INCLUDES //

#include <cmath>
#include <iostream>
#include <algorithm>
using namespace std;

#define SELDONDATA_DEBUG_LEVEL_4
#define SELDONDATA_WITH_GRIB

#include "AtmoData.hxx"
using namespace AtmoData;

#include "Diagnosis.cxx"

// INCLUDES //
//////////////


int main(int argc, char** argv)
{

  TRY;
  
  cout << endl;

  string configuration_file("water_plume.cfg"), sec_config_file("");

  if (argc != 1 && argc != 2 && argc != 3)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [secondary configuration file]\n";
      mesg += string("  ") + argv[0] + " [main configuration file]\n";
      mesg += "Arguments:\n";
      mesg += "  [main configuration file] (optional): main configuration file. Default: water_plume.cfg\n";
      mesg += "  [secondary configuration file] (optional): secondary configuration file.\n";
      cout << mesg << endl;
      return 1;
    }

  if (argc == 3)
    sec_config_file = argv[2];
  if (argc == 2 || argc == 3)
    configuration_file = argv[1];

  if (!exists(configuration_file))
    throw string("Unable to find configuration file \"")
      + configuration_file + "\".";


  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////


  typedef float real;

  int k, j, i;


  /////////////////////////
  // CONFIGURATION FILES //
  /////////////////////////


  cout << "Reading configuration files..."; cout.flush();
 
  ConfigStreams configuration(configuration_file);
  if (exists(sec_config_file))
    configuration.AddFile(sec_config_file);

  /*** Input domain ***/

  configuration.SetSection("[domain]");

  int Nz, Ny, Nx;

  configuration.PeekValue("Nz", Nz);
  configuration.PeekValue("Ny", Ny);
  configuration.PeekValue("Nx", Nx);

  /*** Input files ***/

  configuration.SetSection("[input]");

  string temperature_file, pressure_file, specific_humidity_file,
    liquid_water_content_file, plume_water_file;

  // Meteorological files.
  configuration.PeekValue("Temperature", temperature_file);
  configuration.PeekValue("Pressure", pressure_file);
  configuration.PeekValue("SpecificHumidity", specific_humidity_file);
  configuration.PeekValue("LiquidWaterContent", liquid_water_content_file);

  // Plume total water content.
  configuration.PeekValue("PlumeWater", plume_water_file);

  /*** Parameters ***/

  configuration.SetSection("[parameters]");

  // Initial conditions (at source point).
  // Liquid water potential temperature in K at the source.
  real source_temperature;
  // Total water content at the source in kg/kg.
  real source_water_content;

  configuration.PeekValue("source_temperature", source_temperature);
  configuration.PeekValue("source_water_content", source_water_content);

  /*** Output file ***/

  configuration.SetSection("[output]");

  string plume_liquid_file;

  configuration.PeekValue("LiquidWaterContent", plume_liquid_file);

  cout << " done." << endl;


  //////////
  // DATA //
  //////////


  cout << "Memory allocation for data fields..."; cout.flush();

  // Input fields.

  Data<real, 3> Temperature(Nz, Ny, Nx);
  Data<real, 3> Pressure(Nz, Ny, Nx);
  Data<real, 3> SpecificHumidity(Nz, Ny, Nx);
  Data<real, 3> LiquidWaterContent(Nz, Ny, Nx);

  // Total water content in the plume.
  Data<real, 3> PlumeWaterContent(Nz, Ny, Nx);

  // Temporary fields.
  // Liquid water potential temperature in K.
  Data<real, 3> PotentialTemperatureLW(Nz, Ny, Nx);
  Data<real, 3> AirDensity(Nz, Ny, Nx);
  // Liquid water potential temperature in K in the plume.
  Data<real, 3> PlumePotentialTemperatureLW(Nz, Ny, Nx);

  // Output fields.

  // Plume liquid water content (diagnosed)
  // in (kg of liquid water)/(kg of dry air with water vapor).
  Data<real, 3> PlumeLiquidWaterContent(Nz, Ny, Nx);
  // Plume specific humidity (diagnosed)
  // in (kg of water vapor)/(kg of dry air with water vapor).
  Data<real, 3> PlumeSpecificHumidity(Nz, Ny, Nx);
  // Plume nebulosity.
  Data<real, 3> PlumeNebulosity(Nz, Ny, Nx);

  cout << " done." << endl;
  cout << endl;


  /////////////////
  // READS INPUT //
  /////////////////


  cout << "Extracting data..."; cout.flush();

  FormatBinary<float> IO;

  IO.Read(temperature_file, Temperature);
  IO.Read(pressure_file, Pressure);
  IO.Read(specific_humidity_file, SpecificHumidity);
  IO.Read(liquid_water_content_file, LiquidWaterContent);

  // Last record in file that stores the plume total water content.
  int last_record_index = int(file_size(plume_water_file))
    / sizeof(float) / Nz / Ny / Nx - 1;
  // Reads the last step only.
  IO.ReadRecord(plume_water_file, last_record_index,
		PlumeWaterContent);

  cout << " done." << endl;


  ///////////////
  // DIAGNOSES //
  ///////////////


  cout << "Performing diagnoses..."; cout.flush();

  ComputeAirDensity(Temperature, Pressure, AirDensity);

  // From concentrations in g.m^{-3}
  // to (kg of total water)/(kg of dry air with water vapor).
  PlumeWaterContent.GetArray() =
    1.e-3 * PlumeWaterContent.GetArray() / AirDensity.GetArray();

  ComputePotentialTemperatureLW(Temperature, Pressure, LiquidWaterContent,
				PotentialTemperatureLW);

  // Similarity assumption between water content and plume temperature.
  for (k = 0; k < Nz; k++)
    for (j = 0; j < Ny; j++)
      for (i = 0; i < Nx; i++)
	PlumePotentialTemperatureLW(k, j, i)
	  = PlumeWaterContent(k, j, i) / source_water_content *
	  (source_temperature - PotentialTemperatureLW(k, j, i))
	  + PotentialTemperatureLW(k, j, i);
  
  // Computes PlumeSpecificHumidity, PlumeLiquidWaterContent and Nebulosity.
  WaterDiagnosis(Pressure, LiquidWaterContent, SpecificHumidity,
		 PlumePotentialTemperatureLW, PlumeWaterContent,
		 PlumeLiquidWaterContent);

  // From (kg of liquid water)/(kg of dry air with water vapor)
  // to g/(kg of dry air with water vapor).
  PlumeLiquidWaterContent.Mlt(1.e3);

  cout << " done." << endl;


  ////////////////////////
  // WRITES OUTPUT DATA //
  ////////////////////////


  cout << "Writing data..."; cout.flush();

  IO.Write(PlumeLiquidWaterContent, plume_liquid_file);

  cout << " done." << endl;

  cout << endl;

  END;

  return 0;

}
