// Copyright (C) 2004-2005 CEREA
//     Authors: Marilyne Tombette and Denis Qulo
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
// 
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


// Reads concentrations in Polair3D-like format on a coarse mesh (typically
// outputs from a previous simulation) and generates boundary conditions for
// Polair3D on the finer mesh.


//////////////
// INCLUDES //

#include <iostream>

using namespace std;

#define SELDONDATA_DEBUG_LEVEL_4

#include "AtmoData.hxx"
using namespace AtmoData;

// INCLUDES //
//////////////


int main(int argc, char** argv)
{
 
  TRY;
  
  cout << endl;

  string main_config_file("bc.cfg"), sec_config_file("");

  if (argc != 2 && argc != 3 && argc != 4 || !is_num(argv[argc - 1]))
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [secondary config file] [date]\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [date]\n";
      mesg += string("  ") + argv[0] + " [date]\n\n";
      mesg += "Arguments:\n";
      mesg += "  [main configuration file] (optional): main configuration file. Default: Polair3D.cfg\n";
      mesg += "  [secondary configuration file] (optional): secondary configuration file.\n";
      mesg += "  [date]: date in format YYYYMMDD.\n";
      cout << mesg << endl;
      return 1;
    }

  if (argc == 4)
    sec_config_file = argv[2];
  if (argc == 3 || argc == 4)
    main_config_file = argv[1];

  // Configuration files.
  if (!exists(main_config_file))
    throw string("Unable to find configuration file \"")
      + main_config_file + "\".";
  ConfigStreams configuration(main_config_file);
  if (exists(sec_config_file))
    configuration.AddFile(sec_config_file);


  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////

  typedef float real;

  // Input domain: Coarse mesh (mother domain).
  int date_mother_domain;
  int Nt_in, Nz_in, Ny_in, Nx_in;
  real Delta_t_in, Delta_y_in, Delta_x_in;
  real t_min_in, y_min_in, x_min_in;
  
  // Output domain: Fine mesh (child domain).
  int Nt_out, Nz_out, Ny_out, Nx_out;
  real Delta_t_out, Delta_y_out, Delta_x_out;
  real t_min_out, y_min_out, x_min_out;


  /////////////////////////
  // CONFIGURATION FILES //
  /////////////////////////

  cout << "Reading configuration files..."; cout.flush(); 

  configuration.FindFromBeginning("[mother_domain]");

  configuration.PeekValue("Date", date_mother_domain);
  Date date_mother(date_mother_domain);
  Date date(convert<int>(argv[argc - 1]));
  int record = date.GetDaysFrom(date_mother);

  configuration.PeekValue("Nt", Nt_in);
  configuration.PeekValue("Nz", Nz_in);
  configuration.PeekValue("Ny", Ny_in);
  configuration.PeekValue("Nx", Nx_in);

  configuration.PeekValue("Delta_t", Delta_t_in);
  configuration.PeekValue("Delta_y", Delta_y_in);
  configuration.PeekValue("Delta_x", Delta_x_in);

  configuration.PeekValue("t_min", t_min_in);
  configuration.PeekValue("y_min", y_min_in);
  configuration.PeekValue("x_min", x_min_in);

  string vertical_levels_in;
  configuration.PeekValue("Vertical_levels", vertical_levels_in);


  configuration.FindFromBeginning("[domain]");

  configuration.PeekValue("Nz", Nz_out);
  configuration.PeekValue("Ny", Ny_out);
  configuration.PeekValue("Nx", Nx_out);

  configuration.PeekValue("Delta_y", Delta_y_out);
  configuration.PeekValue("Delta_x", Delta_x_out);

  configuration.PeekValue("y_min", y_min_out);
  configuration.PeekValue("x_min", x_min_out);

  t_min_out = t_min_in;
  Nt_out = Nt_in;
  Delta_t_out = Delta_t_in;

  string vertical_levels_out;
  configuration.PeekValue("Vertical_levels", vertical_levels_out);

  // Input/output directories and files.
  string species_file; 
  string Directory_in, Directory_out;

  configuration.FindFromBeginning("[bc_files]");

  configuration.PeekValue("Species", species_file);

  configuration.PeekValue("Database_Polair3D-bc", Directory_in);
  configuration.PeekValue("Directory_Polair3D-bc", Directory_out);
  
  cout << " done." << endl;
  cout << endl;


  ///////////
  // GRIDS //
  ///////////
    
  cout << "Memory allocation for data fields..."; cout.flush();

  // Input grids.
  RegularGrid<real> GridT_in(t_min_in, Delta_t_in , Nt_in);
  
  RegularGrid<real> GridZ_interf_in(Nz_in + 1);
  RegularGrid<real> GridZ_in(Nz_in);
  
  RegularGrid<real> GridY_in(y_min_in, Delta_y_in, Ny_in);
  RegularGrid<real> GridX_in(x_min_in, Delta_x_in, Nx_in);

  // Output grids.
  RegularGrid<real> GridT_out(t_min_out, Delta_t_out, Nt_out);

  RegularGrid<real> GridZ_interf_out(Nz_out + 1);
  RegularGrid<real> GridZ_out(Nz_out);

  RegularGrid<real> GridY_out(y_min_out, Delta_y_out, Ny_out);
  RegularGrid<real> GridX_out(x_min_out, Delta_x_out, Nx_out);


  ///////////////////////////
  // INPUT DATA PROCESSING //
  ///////////////////////////

  // Reads input altitudes. 
  FormatText Heights_in;
  Heights_in.Read(vertical_levels_in, GridZ_interf_in);
  for (int k = 0; k < Nz_in; k++)
    GridZ_in(k) = (GridZ_interf_in(k) + GridZ_interf_in(k+1)) / 2.;
  
  // Reads output altitudes.
  FormatText Heights_out;
  Heights_out.Read(vertical_levels_out, GridZ_interf_out);
  for (int k = 0; k < Nz_out; k++)
    GridZ_out(k) = (GridZ_interf_out(k) + GridZ_interf_out(k+1)) / 2.;
  
  // Grids for boundary conditions.
  RegularGrid<real> GridZ_bcz_out(1);
  RegularGrid<real> GridY_bcy_out(2);
  RegularGrid<real> GridX_bcx_out(2);

  GridY_bcy_out(0) = y_min_out - Delta_y_out;
  GridY_bcy_out(1) = y_min_out + (Ny_out + 1) * Delta_y_out;
  GridX_bcx_out(0) = x_min_out - Delta_x_out;
  GridX_bcx_out(1) = x_min_out + (Nx_out + 1) * Delta_x_out;
  GridZ_bcz_out(0) = GridZ_in(Nz_in-1);

  // Checks that interpolation is feasible.
  if (GridX_bcx_out(0) < GridX_in(0) || GridX_bcx_out(1) > GridX_in(Nx_in-1))
    throw string("Unable to interpolate along x direction: [")
      + to_str(GridX_bcx_out(0)) + string("; ") + to_str(GridX_bcx_out(1)) 
      + string("] (output) is not included in [")
      + to_str(GridX_in(0)) + string("; ") + to_str(GridX_in(Nx_in-1)) 
      + string("] (input).");

  if (GridY_bcy_out(0) < GridY_in(0) || GridY_bcy_out(1) > GridY_in(Ny_in-1))
    throw string("Unable to interpolate along y direction: [")
      + to_str(GridY_bcy_out(0)) + string("; ") + to_str(GridY_bcy_out(1)) 
      + string("] (output) is not included in [")
      + to_str(GridY_in(0)) + string("; ") + to_str(GridY_in(Ny_in-1)) 
      + string("] (input).");

  if (GridZ_out(0) < GridZ_in(0) || GridZ_out(Nz_out-1) > GridZ_in(Nz_in-1))
    throw string("Unable to interpolate along z direction: [")
      + to_str(GridZ_out(0)) + string("; ") + to_str(GridZ_out(Nz_out-1))
      + string("] (output) is not included in [")
      + to_str(GridZ_in(0)) + string("; ") + to_str(GridZ_in(Nz_in-1))
      + string("] (input).");

  cout << " done." << endl;
  cout << endl;


  /////////////////////////
  // BOUNDARY CONDITIONS //
  /////////////////////////

  ExtStream species(species_file);
  string species_names;

  FormatBinary<real> InputPolair;

  if (!species.is_open())
    throw string("Unable to open file \"") + species_file + "\".";

  cout << "Species:" << endl;

  while (species.GetElement(species_names))
    {
      cout << "   " << species_names << endl;
      // Input fields.
      Data<real, 4> Conc_in(GridT_in, GridZ_in, GridY_in, GridX_in);
  
      // Output fields.
      string File_conc = Directory_in + species_names + ".bin";
      InputPolair.ReadRecord(File_conc, record, Conc_in);

      // Boundary Conditions
      Data<real, 4> Conc_bc_out_z(GridT_out, GridZ_bcz_out, GridY_out,
				  GridX_out);
      Data<real, 4> Conc_bc_out_y(GridT_out, GridZ_out, GridY_bcy_out,
				  GridX_out);
      Data<real, 4> Conc_bc_out_x(GridT_out, GridZ_out, GridY_out,
				  GridX_bcx_out);

      // Interpolation
      LinearInterpolationRegular(Conc_in, Conc_bc_out_x);
      LinearInterpolationRegular(Conc_in, Conc_bc_out_y);
      LinearInterpolationRegular(Conc_in, Conc_bc_out_z);

      // Writes output files.
      FormatBinary<float> PolairOut;
      PolairOut.Append(Conc_bc_out_x, Directory_out + species_names 
		       + "_x.bin");
      PolairOut.Append(Conc_bc_out_y, Directory_out + species_names 
		       + "_y.bin");
      PolairOut.Append(Conc_bc_out_z, Directory_out + species_names 
		       + "_z.bin");
    }

  species.close();

  cout << endl;
  
  END;
  
  return 0;

}
