#!/usr/bin/env python
#
# This script processes Gocart raw data files into Polair3D input files.
# Gocart concentrations are expected to be daily averages.

import os
import sys
from optparse import OptionParser
import ConfigParser

prog=os.path.basename(sys.argv[0]) + " : "

#################
# CONFIGURATION #

# Fill in the year.
year = 2001
# List all species. Warning: the order is crucial; in case you need 
# to change it, please contact polyphemus@cerea.enpc.fr.
species_list = ["CC.STD.tv15", "DU.STD.tv20", "SU.STD.tv15", "SS.STD.tv17"]

# Put your file format, where %Y is the year, %M is the month
# and %S is the species name.
file_name_format = "/u/cergrene/Y/fahey/CHIN-BCs/%Y%M.%S.g.day"

# Configuration files.
cfg_general_default = "../general.cfg"

# configuration files corresponding to species 
# in "species_list" (same order !).
cfg_species_list = ["bc-gocart-CC.cfg", "bc-gocart-DU.cfg",
                    "bc-gocart-SU.cfg", "bc-gocart-SS.cfg"]

# configuration file for no3.
cfg_no3 = "bc-no3.cfg"

# configuration file for nh4.
cfg_nh4 = "bc-nh4.cfg"

# CONFIGURATION #
#################


##########
# SCRIPT #
usage = "%prog [options] general_config_file first_month last_month \n\n" \
	+ " Computes aerosol Polair3D boundary conditions" \
	+ " from Gocart data.\n" \
	+ " general_config_file is optional, default is " + cfg_general_default + " .\n" \
	+ " first_month: index of first month to perform.\n" \
	+ " last_month:  index of last  month to perform.\n" \
	+ " 1=january, ..., 12=december."
version= "%prog 1.0"

parser = OptionParser(usage=usage,version=version)

parser.add_option("-n", "--dry-run", action="count", \
		  dest="dry_run", default=0, help="says what it would do," \
		  + " but actually does NOT do it.")
parser.add_option("-c", "--clean", action="count", \
		  dest="clean_all", default=0, help="clean output directory " \
		  + " before processing, default is NO.")
parser.add_option("--no_no3", action="store_true", \
		  dest="no_no3", default=False, help="do not compute nitrate" \
		  + " which is not directly provided by Gocart," \
		  + " default is to compute.")
parser.add_option("--no_nh4", action="store_true", \
		  dest="no_nh4", default=False, help="do not compute ammonia" \
		  + " which is not directly provided by Gocart," \
		  + " default is to compute.")

(options, args) = parser.parse_args()

# Test number of options
if len(args) != 3 and len(args) != 2:
	os.system(sys.argv[0] + " -h")
	sys.exit(2)

# general config file
if len(args) == 3:
      cfg_general = args[0]
elif len(args) == 2:
	cfg_general = cfg_general_default

if os.path.exists(cfg_general):
	print prog, "using general config file " + cfg_general
else:
	print prog, "general config file " + cfg_general + " NOT found !!"
	sys.exit(2)
	
# Month range.
if len(args) == 3:
	month_range = [int(args[1]), int(args[2]) ]
elif len(args) == 2:
	month_range = [int(args[0]), int(args[1]) ]

print prog, "using month range from " \
      + str(month_range[0]) + " to " + str(month_range[1])

month_range = range(month_range[0], month_range[1] + 1)
file_name_format = file_name_format.replace("%Y", str(year))

# Dry run option.
if options.dry_run > 0:
	print prog, "This is a DRY run, nothing actually performed.\n"

# clean option.
if options.clean_all > 0:
	config = ConfigParser.ConfigParser()
	config.readfp(open(cfg_general) )
	directory_Home=config.get("general","Home")
	directory_fields=config.get("general","Directory_computed_fields")

	directory2clean=directory_fields.replace('<Home>',directory_Home)

	print prog, "Cleaning output directory " + directory2clean + ".\n"
	command="rm -f " + directory2clean + "/bc/" \
		+ "P{NA,SO4,NH4,NO3,HCL,ARO1,ARO2,OLE1," \
		+ "ALK1,API1,API2,LIM1,LIM2,BC,MD,POA}_*_?.bin"
	if options.dry_run > 0:
		print prog + command
	else:
		os.system(command)

# Process one given year, that of Gocart files.
str_year = str(year)
print prog, "processing year", str_year


# Process mont per month, because of Gocart files.
for month in month_range:
	if month < 10:
		str_month = "0" + str(month)
	else:
		str_month = str(month)
	
	print prog, "month", month
	sys.stdout.flush()
	
        for species, cfg in zip(species_list, cfg_species_list):
			
		file_name = file_name_format.replace("%M", str_month)
		file_name = file_name.replace("%S", species)

		command = "bc-gocart " + cfg_general + " " \
			  + cfg + " " + file_name + " " \
			  + str_year + str_month

		if options.dry_run > 0:
			print prog + command
		else:
                        print prog, " compute species", species, "..."
		        sys.stdout.flush()
		      	if os.system(command) != 0:
				print " ERROR "
				sys.exit(2)
			else:
				print prog, "... done"

			sys.stdout.flush()


# Perform NO3 for all month range.
if not options.no_no3:
	command = "bc-no3 " + cfg_general \
		  + " " + cfg_no3

	if options.dry_run > 0:
		print prog + command
	else:
		print prog, " compute NO3 ..."
		sys.stdout.flush()
		if os.system(command) != 0:
			print " ERROR "
			sys.exit(2)
		else:
			print prog, "... done"
			
		sys.stdout.flush()


# Perform NH4 for all month range.
if not options.no_nh4:
	command = "bc-nh4 " + cfg_general \
		  + " " + cfg_nh4
	
	if options.dry_run > 0:
		print prog + command
	else:
		print prog, " compute NH4 ..."
		sys.stdout.flush()
		if os.system(command) != 0:
			print " ERROR "
			sys.exit(2)
		else:
			print "done"
			
		sys.stdout.flush()

# SCRIPT #
##########
