// Copyright (C) 2005-2006 CEREA
//     Author: Edouard Debry
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


// Generates boundary conditions for Polair3D based on Gocart concentrations
// for particulate matter.


#include "Talos.hxx"
using namespace Talos;


const int Norder = 9;

// Roots of the Legendre Polynomial of order Norder.
const double root[] = {0.1591988024618696E-01, 0.8198444633668212E-01,
		       0.1933142836497048E+00, 0.3378732882980955E+00,
		       0.5000000000000000E+00, 0.6621267117019045E+00,
		       0.8066857163502952E+00, 0.9180155536633179E+00,
		       0.9840801197538130E+00};

// Weights of the Legendre quadrature formula of order Norder.
const double weight[] = {0.4063719418078723E-01, 0.9032408034742870E-01,
			 0.1303053482014677E+00, 0.1561735385200014E+00,
			 0.1651196775006299E+00, 0.1561735385200014E+00,
			 0.1303053482014677E+00, 0.9032408034742870E-01,
			 0.4063719418078723E-01};


// Class defining one log-normal density.
class LogNormal
{
private:
  float Ntot;
  float Dmean;
  float Sigma;

public:
  LogNormal();
  LogNormal(float, float, float);
  float NumLogNormal(float);
  float VolLogNormal(float);
};


// Constructor.
LogNormal::LogNormal()
{
  Ntot = 0.0;
  Dmean = 10.0;
  Sigma = 10.0;
}


// Constructor.
LogNormal::LogNormal(float Ntot0, float Dmean0, float Sigma0)
{
  if (Ntot0 * Dmean0 * Sigma0 <= 0.0)
    throw string("WARNING: strange parameters in LogNormal\n")
      + string("WARNING: Ntot Dmean Sigma: ") + to_str(Ntot0)
      + string("\t") + to_str(Dmean0) + string("\t") + to_str(Sigma0);

  Ntot = Ntot0;
  Dmean = Dmean0;
  Sigma = Sigma0;
}


// Computes the number concentration at a given diameter (m) for a lognormal
// distribution.
float LogNormal::NumLogNormal(float diameter)
{
  // 1 / sqrt(2 * pi)
  const float sqrt_2pi = 0.3989422804020;

  float tmp1 = log(Sigma);
  float tmp2 = log(diameter / Dmean) / tmp1;
  
  return sqrt_2pi * Ntot / tmp1 * exp(-0.5 * tmp2 * tmp2);
}


// Computes the volume concentration at a given diameter (m) for a lognormal
// distribution.
float LogNormal::VolLogNormal(float diameter)
{
  // pi / 6
  const float pi_6 = 0.523598775598;

  return pi_6 * diameter * diameter * diameter
    * this->NumLogNormal(diameter);
}



// Class for one modal aerosol distribution.
class ModalAerosol
{
private:
  float Dmin, Dmax;
  LogNormal Nuclei;
  LogNormal Accumulation;
  LogNormal Coarse;

public:
  ModalAerosol();
  ModalAerosol(float, float, float*, float*, float*);
  float NumDensity(float);
  float VolDensity(float);
  float NumQuantity(float, float);
  float VolQuantity(float, float);
};


// Default constructor.
ModalAerosol::ModalAerosol():
  Nuclei(), Accumulation(), Coarse()
{
  Dmin = 0.001;
  Dmax = 10.0;
}


// Constructor.
ModalAerosol::ModalAerosol(float Dmin0, float Dmax0, float *Ntot0,
			   float *Dmean0, float *Sigma0):
  Nuclei(Ntot0[0], Dmean0[0], Sigma0[0]),
  Accumulation(Ntot0[1], Dmean0[1], Sigma0[1]),
  Coarse(Ntot0[2], Dmean0[2], Sigma0[2])
{
  if (Dmin0 * Dmax0 <= 0.000001 || Dmax0 <= Dmin0)
    throw string("WARNING: strange parameters in ModalAerosol\n")
      + string("WARNING: Dmin Dmax: ") + to_str(Dmin0)
      + string("\t") + to_str(Dmax0);

  Dmin = Dmin0;
  Dmax = Dmax0;
}


// Computes the aerosol number concentration for a modal aerosol distribution
// at a given diameter (m).
float ModalAerosol::NumDensity(float diameter)
{
  float num = 0.0;
  if (diameter >= Dmin && diameter <= Dmax)
    num = Nuclei.NumLogNormal(diameter) + Accumulation.NumLogNormal(diameter)
      + Coarse.NumLogNormal(diameter);
  return num;
}


// Computes the aerosol volume concentration for a modal aerosol distribution
// at a given diameter (m).
float ModalAerosol::VolDensity(float diameter)
{
  float vol = 0.0;
  if (diameter >= Dmin && diameter <= Dmax)
    vol = Nuclei.VolLogNormal(diameter) + Accumulation.VolLogNormal(diameter)
      + Coarse.VolLogNormal(diameter);
  return vol;
}


// Computes aerosol number quantity of modal
// distribution within an aerosol diameter range.
float ModalAerosol::NumQuantity(float diameter1, float diameter2)
{
  float log_diameter1 = log(diameter1);
  float log_diameter2 = log(diameter2);
  float log_difference = log_diameter2 - log_diameter1;
 
  float num = 0.0;
  if (log_difference > 0.0)
    for (int j = 0; j < Norder - 1; j++)
      {
	float log_diameter = log_diameter1 + root[j] * log_difference;
	float diameter = exp(log_diameter);
	
	num += this->NumDensity(diameter) * weight[j];
      }

  return num * log_difference;
}


// Computes aerosol volume quantity of modal
// distribution within an aerosol diameter range
float ModalAerosol::VolQuantity(float diameter1, float diameter2)
{
  float log_diameter1 = log(diameter1);
  float log_diameter2 = log(diameter2);
  float log_difference = log_diameter2 - log_diameter1;

  float vol = 0.0;
  if (log_difference > 0.0)
    for (int j = 0; j < Norder - 1; j++)
      {
	float log_diameter = log_diameter1 + root[j] * log_difference;
	float diameter = exp(log_diameter);
	
	vol += this->VolDensity(diameter) * weight[j];
      }

  return vol * log_difference;
}



// Class of bin discretization.
class BinDist
{
private:
  int Nbin;
  float Dmin, Dmax;
  float* bound;

public:
  BinDist();
  BinDist(int, float, float);
  ~BinDist();
  float GetLowBound(int);
  float GetUpBound(int);
};


// Default constructor.
BinDist::BinDist()
{
  Nbin = 0;
  Dmin = 0.001;
  Dmax = 10.0;
  bound = NULL;
}


// Constructor.
BinDist::BinDist(int Nbin0, float Dmin0, float Dmax0)
{
  if (Dmin0 * Dmax0 <= 0.000001 || Dmax0 <= Dmin0 || Nbin0 <= 0)
      throw string("WARNING: strange parameters in BinDist\n")
	+ string("WARNING: Dmin Dmax Nbin: ") + to_str(Dmin0) + string("\t")
	+ to_str(Dmax0) + string("\t") + to_str(Nbin0);

  Dmin = Dmin0;
  Dmax = Dmax0;
  Nbin = Nbin0;
  
  bound = new float[Nbin + 1];
  
  float diameter_difference = log(Dmax / Dmin) / float(Nbin) ;
  
  for (int j = 0; j < Nbin + 1; j++)
    bound[j] = Dmin * exp(diameter_difference * float(j));
}


// Gets the lower bound of bin #i.
float BinDist::GetLowBound(int i)
{
  if (i >= 0 && i < Nbin + 1)
    return bound[i];
  else
    return Dmin;
}


// Gets the upper bound of bin #i.
float BinDist::GetUpBound(int i)
{
  if (i >= 0 && i < Nbin + 1)
    return bound[i + 1];
  else
    return Dmax;
}


// Destructor.
BinDist::~BinDist()
{
  if (bound != NULL)
    delete[] bound;
  Nbin = 0;
  Dmin = 0.0;
  Dmax = 0.0;
}
