// Copyright (C) 2006 CEREA
//     Author: Irne Korsakissok
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the modeling system Polyphemus.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


//////////////
// INCLUDES //

#include <vector>
#include <string>
#include <map>
#include <iostream>
#include <fstream>
using namespace std;

#define SELDONDATA_DEBUG_LEVEL_4

#include "SeldonData.hxx"
using namespace SeldonData ;

// INCLUDES //
//////////////


int main(int argc, char **argv)

{
  TRY;
  
  cout << endl;

  if (argc != 2)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + string(" [main configuration file]\n");
      mesg += string("Arguments:\n");
      mesg += string("  [main configuration file]:")
	+string("main configuration file.\n");
      cout << mesg << endl;
      return 1;
    }

  string configuration_file(argv[1]);


  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////


  typedef double real;

  // Meteorological data.
  vector<map<string, real> > meteo_data;
  // Number of meteorological situations.
  int Nmeteo;
  // Stability conditions.
  vector<string> stability;

  // List of species.
  vector<string> species_list;
  // Number of species.
  int Ns;

  // Files.
  string meteo_file, species_file, output_file;

  // Option: are comments to be written?
  bool with_comment;

  // Parameterization types to compute scavenging coefficients and deposition
  // velocities.
  string scavenging_type, velocity_type;

  // Scavenging coefficients (s^(-1)).
  Array<real, 2> scavenging_coefficient;
  // Deposition velocities (m/s).
  Array<real, 2> deposition_velocity;


  /////////////////////////
  // CONFIGURATION FILES //
  /////////////////////////


  cout << "Reading configuration file..."; cout.flush();

  ConfigStream config(configuration_file);
  config.SetSection("[data]");
  config.PeekValue("Species", species_file);
  config.PeekValue("Meteo", meteo_file);

  config.SetSection("[scavenging]");
  config.PeekValue("Type", scavenging_type);
 
  config.SetSection("[deposition]");
  config.PeekValue("Type", velocity_type);

  config.SetSection("[output]");
  config.PeekValue("With_comment", with_comment);
  config.PeekValue("Output_file", output_file);

  cout << " done." << endl;


  ////////////////
  // DATA FILES //
  ////////////////


  cout << "Reading meteorological data..."; cout.flush();

  // Browses all sections "[situation]".
  ConfigStream meteo(meteo_file);
  Nmeteo = 0;
  while (!meteo.IsEmpty())
    if (split(meteo.GetLine())[0] == "[situation]")
      {
	map<string, real> data;
	string stability_;
	
	meteo.PeekValue("Temperature", data["temperature"]);
	meteo.PeekValue("Wind_angle", data["wind_angle"]);
	meteo.PeekValue("Wind", data["wind"]);
	meteo.PeekValue("Inversion_height", data["inversion_height"]);
	meteo_data.push_back(data);
	meteo.PeekValue("Stability", stability_);
	stability.push_back(stability_);
	Nmeteo++;
      }
  meteo.Rewind();
  cout << " done." << endl;

  cout << "Reading species..."; cout.flush();
  ConfigStream species(species_file);

  // Section "[species]" contains all species names.
  species.SetSection("[species]");
  while (!species.IsEmpty())
    species_list.push_back(species.GetElement());
  Ns = int(species_list.size());
  cout << " done." << endl;


  ////////////////
  // SCAVENGING //
  ////////////////


  cout << "Computation of the scavenging coefficients..."; cout.flush();

  /*** Reads species for which scavenging occurs ***/

  // List of the species for which scavenging occurs.
  vector<string> scavenging_list;
  // Number of species for which scavenging occurs.
  int Nscav;

  species.SetSection("[scavenging]");
  while(!species.IsEmpty())
    scavenging_list.push_back(species.GetElement());
  Nscav = int(scavenging_list.size());

  vector<bool> species_scavenging(Ns);
  for (int i = 0; i < Ns; i++)
    {
      bool in_list = false;
      for (int j = 0; j < Nscav; j++)
	if (species_list[i] == scavenging_list[j])
	  in_list = true;
      species_scavenging[i] = in_list;
    }

  /*** Computation of the scavenging coefficients ***/

  scavenging_coefficient.resize(Nmeteo, Ns);

  // Scavenging type: none. All coefficients for gaseous species are set to 0.
  if (scavenging_type == "none")
    scavenging_coefficient = 0.;

  // Scavenging type: constant. All coefficients for gaseous species are
  // read in the section [scavenging_constant] of the species file.
  else  if (scavenging_type == "constant")
    {
      Array<real, 1> scav_coeff(Ns);
      species.SetSection("[scavenging_constant]");
      for (int i = 0; i < Ns; i++)
	if (species_scavenging[i])
	  species.PeekValue(species_list[i], scav_coeff(i));
	else
	  scav_coeff(i) = 0.;

      for (int i = 0; i < Nmeteo; i++)
	for (int j = 0; j < Ns; j++)
	  scavenging_coefficient(i, j) = scav_coeff(j);
    }

  // Scavenging type: belot. All coefficients for gaseous species are computed
  // with Belot parameterization. The Belot coefficients are read in the
  // section [scavenging_belot] of the species_file.
  else if (scavenging_type == "belot")
    {
      Array<real, 2> belot_coefficient(Ns,2);
      for (int i = 0; i < Ns; i++)
	if (species_scavenging[i])
	  {
	    species.SetSection("[scavenging_belot]");
	    species.Find(species_list[i]);
	    species.GetNumber(belot_coefficient(i, 0));
	    species.GetNumber(belot_coefficient(i, 1));
	  }
	else
	  {
	    belot_coefficient(i, 0) = 0.;
	    belot_coefficient(i, 1) = 0.;
	  }

      for (int i = 0; i < Nmeteo; i++)
	{
	  real rainfall_rate;
	  meteo.Find("[situation]");
	  meteo.PeekValue("Rainfall_rate", rainfall_rate);

	  for (int j = 0; j < Ns; j++)
	    scavenging_coefficient(i,j) =  belot_coefficient(j, 0) *
	      pow(rainfall_rate,  belot_coefficient(j, 1));
	}
    }
  else
    {
      cout << "The parameterization \"" << scavenging_type
	   << "\" is unknown." << endl;
      return 1;
    }

  cout << " done." << endl;

 
  ////////////////
  // DEPOSITION //
  ////////////////


  cout << "Computation of the deposition velocities..."; cout.flush();
 
  /*** Reads species for which deposition occurs ***/

  // List of the species for which deposition occurs.
  vector<string> deposition_list;
  // Number of species for which deposition occurs.
  int Ndep;

  species.SetSection("[deposition]");
  while(!species.IsEmpty())
    deposition_list.push_back(species.GetElement());
  Ndep = int(deposition_list.size());

  vector<bool> species_deposition(Ns);
  for (int i = 0; i < Ns; i++)
    {
      bool in_list = false;
      for (int j = 0; j < Ndep; j++)
	if (species_list[i] == deposition_list[j])
	  in_list = true;
      species_deposition[i] = in_list;
    }

  /*** Computation of the deposition velocities ***/

  deposition_velocity.resize(Nmeteo, Ns);

  // Deposition type: none. All velocities for gaseous species are set to 0.
  if (velocity_type == "none")
    deposition_velocity = 0.;

  // Deposition type: constant. All velocities for gaseous species are
  // read in the section [deposition_constant] of the species file.
  else  if (velocity_type == "constant")
    {
      Array<real, 1> dep_velocity(Ns);
      species.SetSection("[deposition_constant]");
      for (int i = 0; i < Ns; i++)
	if (species_deposition[i])
	  species.PeekValue(species_list[i], dep_velocity(i));
	else
	  dep_velocity(i) = 0.;

      for (int i = 0; i < Nmeteo; i++)
	for (int j = 0; j < Ns; j++)
	  deposition_velocity(i,j) = dep_velocity(j);
    }
  else
    {
      cout << "The parameterization \"" << velocity_type
	   << "\" is unknown." << endl;
      return 1;
    }

  cout << " done." << endl;
 

  ////////////
  // OUTPUT //
  ////////////


  cout << "Writing data..."; cout.flush();

  ofstream meteo_stream(output_file.c_str());
  if (!meteo_stream.is_open())
    {
      cout << "unable to open file \"" << output_file << "\"." << endl;
      return 1;
    }

  for (int i = 0; i < Nmeteo; i++)
    {
      meteo_stream << "[situation]" << endl << endl;
 
     if (with_comment)
	meteo_stream << "# Temperature (Celsius degrees)\n";
      meteo_stream    << "Temperature = " << meteo_data[i]["temperature"]
		      << endl;

      if (with_comment)
	meteo_stream << endl <<"# Wind angle (degrees)\n";
      meteo_stream << "Wind_angle = " <<  meteo_data[i]["wind_angle"]
		   << endl;

      if (with_comment)
	meteo_stream << endl << "# Wind speed (m/s)\n";
      meteo_stream << "Wind = " << meteo_data[i]["wind"] << endl;

      if (with_comment)
	meteo_stream << endl << "# Inversion height (m)\n";
      meteo_stream << "Inversion_height = "
		   << meteo_data[i]["inversion_height"] << endl;

      if (with_comment)
	meteo_stream << endl << "# Stability class \n";
      meteo_stream << "Stability = " << stability[i] << endl;

      if (with_comment)
	meteo_stream << endl << "# Scavenging coefficients"
		     << "of the species (s^-1)\n";
      meteo_stream << "Scavenging_coefficient =\n";
      for (int j = 0; j < Ns; j++)
	meteo_stream << species_list[j] << "  "
		     << scavenging_coefficient(i, j) << "  ";
      meteo_stream << endl;

      if (with_comment)
	meteo_stream << endl
		     << "# Deposition velocities of the species (m/s)\n";
      meteo_stream << "Deposition_velocity =\n";
      for (int j = 0; j < Ns; j++)
	meteo_stream << species_list[j] << "  "
		     << deposition_velocity(i, j) << "  ";
      
      meteo_stream << endl << endl << endl;
    }
  
  meteo_stream.close();

  cout << " done." << endl;

  END;

  return 0;
}
