// Copyright (C) 2006 CEREA
//     Author: Vivien Mallet
//
// This file is part of Polyphemus, a modeling system for air quality.
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This program converts Chimere emission files to files in format suitable
// for Castor (and Polair3D). It is part of Polyphemus.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#include <iostream>
#include <vector>
#include <list>

#define SELDONDATA_DEBUG_LEVEL_4
#include "AtmoData.hxx"
using namespace AtmoData;


int main(int argc, char* argv[])
{
  
  TRY;

  cout << endl;

  typedef float real;

  if (argc != 2)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [configuration file]\n";
      cout << mesg << endl;
      return 1;
    }

  string configuration_file(argv[1]);

  
  ///////////////////
  // CONFIGURATION //
  ///////////////////


  int h, i, j, k, s;

  cout << "Reading configuration..."; cout.flush();

  // Configuration file.
  ConfigStreams configuration(configuration_file);

  /*** Description ***/
  
  int Nx, Ny, Nz;
  string month, year;

  configuration.SetSection("[description]");

  configuration.PeekValue("Nx", Nx);
  configuration.PeekValue("Ny", Ny);
  // Number of emission levels.
  configuration.PeekValue("Nz", Nz);

  configuration.PeekValue("Month", month);
  configuration.PeekValue("Year", year);

  // Sets corresponding date and the number of days to be processed.
  Date date_ref(year + fill(month, 2, '0', ostringstream::right));
  Date date_next(year + fill(month, 2, '0', ostringstream::right));
  date_next.AddMonths(1);
  // Number of days in the target month.
  int Nt_out = date_next.GetDaysFrom(date_ref) * 24;

  /*** Path ***/

  string file_in, directory_out;

  configuration.SetSection("[path]");
  
  configuration.PeekValue("File_in", file_in);
  configuration.PeekValue("Directory_out", directory_out);

  /*** Species ***/

  int Ns;
  vector<string> species_name;

  configuration.SetSection("[species]");

  // Warning: species must be in the same order as in Chimere file.
  while (!configuration.IsEmpty())
    species_name.push_back(configuration.GetElement());

  Ns = int(species_name.size());

  cout << " done." << endl;

  cout << "   Input file: " << file_in
       << " (month: " << date_ref.GetDate("%y-%m") << ")" << endl;
  cout << "   Number of species: " << Ns << endl;


  ////////////////
  // READS DATA //
  ////////////////

  
  cout << "Reading input emissions..."; cout.flush();

  // Input emissions. The second dimension (length 3) is for the type of day
  // (weekday, Saturday and Sunday). The last dimension is for hours.
  Data<real, 6> Emission_in(Ns, 3, Ny, Nx, Nz, 24);

  FormatText().Read(file_in, Emission_in);

  cout << " done." << endl;
  

  ////////////////
  // CONVERSION //
  ////////////////


  cout << "Converting to Castor and Polair3D emissions..." << endl;

  // Output emissions for a given species.
  Data<real, 4> Emission_out(Nt_out, Nz, Ny, Nx);

  // 0 for weekdays, 1 for Saturday and 2 for Sunday.
  int day_type;

  Date date;
  // For all species.
  for (s = 0; s < Ns; s++)
    {
      cout << "   + " << species_name[s] << endl;
      date = date_ref;
      // Hour per hour.
      for (h = 0; h < Nt_out; h++)
	{
	  day_type = max(date.GetWeekDay() - 4, 0);
	  for (k = 0; k < Nz; k++)
	    for (j = 0; j < Ny; j++)
	      for (i = 0; i < Nx; i++)
		Emission_out(h, k, j, i) = Emission_in(s, day_type, j, i,
						       k, date.GetHour());
	  date.AddHours(1);
	}
      FormatBinary<float>().Append(Emission_out,
				   directory_out + species_name[s] + ".bin");
    }

  cout << " done." << endl;

  cout << endl;

  END;
  
  return 0;
  
}
