// Copyright (C) 2006 CEREA
// Author: Irne Korsakissok
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the modeling system Polyphemus.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


//////////////
// INCLUDES //

#include <vector>
#include <string>
#include <map>
#include <iostream>
#include <fstream>
using namespace std;

#define SELDONDATA_DEBUG_LEVEL_4

#include "SeldonData.hxx"
using namespace SeldonData ;

// INCLUDES //
//////////////


int main(int argc, char **argv)

{
  TRY;
  
  cout << endl;

  string configuration_file("");

  if (argc != 2)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] \n";
      mesg += "Arguments:\n";
      mesg += string("  [main configuration file] (optional):")
	+string("main configuration file.\n");
      cout << mesg << endl;
      return 1;
    }

  if (argc == 2)
    configuration_file = argv[1];


  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////


  typedef double real;

  // Source characterization.
  real rate, quantity, source_velocity, velocity, temperature, section;
  string species_name, source_type;

  // Files.
  string trajectory_file, output_file;

  // Option: are comments written?
  bool with_comment;

  // Number of points on the trajectory.
  int Np;

  // Time step.
  real delta_t;
  real t_init;

  vector<real>  puff_quantity;


  /////////////////////////
  // CONFIGURATION FILES //
  /////////////////////////


  cout << "Reading configuration file..."; cout.flush();
  ConfigStreams config(configuration_file);

  config.SetSection("[trajectory]");

  // File containing the trajectory data.
  config.PeekValue("Trajectory_file", trajectory_file);

  // Number of points on the trajectory.
  config.PeekValue("Np", Np);

  // Time step  between two points on the trajectory (for mobile source).
  config.PeekValue("Delta_t", delta_t);

  // Source data.
  config.SetSection("[source]");

  config.PeekValue("Source_type", source_type);
  config.PeekValue("Species_name", species_name);

  if (source_type == "puff")
    {
      config.SetSection("[puff-source]");
      config.PeekValue("Quantity", quantity);
      config.PeekValue("Source_velocity", source_velocity);
      config.PeekValue("tinit", t_init);
    }
  else if (source_type == "continuous")
    {
      config.SetSection("[plume-source]");
      config.PeekValue("Rate", rate);
      config.PeekValue("Velocity", velocity);
      config.PeekValue("Temperature", temperature);
      config.PeekValue("Section", section);
    }
  else
    throw string("Source type should be puff or continuous, but ")
      + source_type + " was provided";

  // Output file.
  config.SetSection("[output]");
  config.PeekValue("With_comment", with_comment);
  config.PeekValue("Source_file", output_file);

  cout << " done." << endl;


  ////////////////
  // TRAJECTORY //
  ////////////////


  map<string, real> coord;
  vector< map<string, real> >  node;
 
  int Nnode;

  cout << "Reading trajectory data..."; cout.flush();

  ExtStream trajectory_stream(trajectory_file);
  if (!trajectory_stream.is_open())
    throw string("File \"") + trajectory_file + "\" does not exist.";

  real X, Y, Z;

  while (!trajectory_stream.IsEmpty())
    {
      trajectory_stream.GetNumber(X);
      trajectory_stream.GetNumber(Y);
      trajectory_stream.GetNumber(Z);
    
      coord["X"] = X;
      coord["Y"] = Y;
      coord["Z"] = Z;

      node.push_back(coord);
    }
  Nnode = int(node.size());

  cout << "  done." << endl;


  ////////////////////
  // DISCRETIZATION //
  ////////////////////


  map<string, real> data;
  vector< map<string, real> > source_data;
  vector<string> species;
  int Nsource;

  vector<real> segment_length(Nnode-1);
  real length = 0.;
  real v = 0.;
  int Ntot = 0;
  vector<real> time(Nnode);

  if (source_type == "puff")
    {
      v = source_velocity / 3.6;
      time[0] = t_init;
    }

  // Calculation of each segment length and of the total trajectory length.
  for (int i = 0; i < Nnode-1; i++)
    {
      segment_length[i] = sqrt( pow((node[i]["X"] - node[i+1]["X"]), 2) +
				pow((node[i]["Y"] - node[i+1]["Y"]), 2) +
				pow((node[i]["Z"] - node[i+1]["Z"]), 2) );
      length += segment_length[i];
      if (source_type == "puff" && v != 0.)
	time[i+1] = time[i] + segment_length[i] / v;
      else if (source_type == "puff" && v == 0.)
	time[i+1] = t_init;
    }
  cout << "Length of the trajectory: " << length << endl;

  // Calculation of the points coordinates.
  for (int i = 0; i < Nnode-1; i++)
    {
      real  delta_X, delta_Y, delta_Z;
      real X1, X2, Y1, Y2, Z1, Z2, t1(0.);
      int Npi;
  
      X1 = node[i]["X"];
      Y1 = node[i]["Y"];
      Z1 = node[i]["Z"];
      X2 = node[i+1]["X"];
      Y2 = node[i+1]["Y"];
      Z2 = node[i+1]["Z"];

      if (source_type == "puff")
	t1 = time[i];

      // Number of points to calculate on the current segment.
      if (source_type == "continous" || v == 0.)
	{
	  if (i != Nnode-2)
	    {
	      Npi =  int(segment_length[i] * Np / length);
	      Ntot += Npi;
	    }
	  else
	    Npi = Np - Ntot;

	  delta_X = (X2-X1) / (Npi+1);
	  delta_Y = (Y2-Y1) / (Npi+1);
	  delta_Z = (Z2-Z1) / (Npi+1);
	}
      else
	{
	  Npi = max(int(segment_length[i] / (v * delta_t)), 0);
	  delta_X = (X2-X1) * v * delta_t / segment_length[i];
	  delta_Y = (Y2-Y1) * v * delta_t / segment_length[i];
	  delta_Z = (Z2-Z1) * v * delta_t / segment_length[i];
	}

      // Data of the calculated points on the current segment.
      for (int j = 0; j < Npi+1; j++)
	{
	  data["X"] = X1 + real(j) * delta_X;
	  data["Y"] = Y1 + real(j) * delta_Y;
	  data["Z"] = Z1 + real(j) * delta_Z;
	  if (source_type == "puff" && v != 0.)
	    data["t"] = t1 + real(j) * delta_t;
	  else if (source_type == "puff" && v == 0.)
	    data["t"] = t_init;
	  if (source_type == "continuous")
	    {
	      data["rate"] = rate;
	      data["velocity"] = velocity;
	      data["temperature"] = temperature;
	      data["section"] = section;
	    }
	  source_data.push_back(data);
	  species.push_back(species_name);
   	}
    }

  data["X"] = node[Nnode-1]["X"];
  data["Y"] = node[Nnode-1]["Y"];
  data["Z"] = node[Nnode-1]["Z"];
  if (source_type == "puff")
    data["t"] = time[Nnode-1];
  if (source_type == "continuous")
    {
      data["rate"] = rate;
      data["velocity"] = velocity;
      data["temperature"] = temperature;
      data["section"] = section;
    }

  source_data.push_back(data);
  species.push_back(species_name);

  Nsource = int(source_data.size());
  cout << "Number of points on the trajectory: " << Nsource << endl;

  // Calculation of the mass released for each puff.
  if (source_type == "puff")
    for (int i = 0; i < Nsource; i++)
      puff_quantity.push_back(quantity / Nsource);


  ////////////
  // OUTPUT //
  ////////////


  cout << "Writing source data..."; cout.flush();

  ofstream sources(output_file.c_str());

  if (!sources.is_open())
    throw string("Unable to open file \"") + output_file + "\".";

  for (int i = 0; i < Nsource; i++)
    {
      sources << "[source]" << endl << endl;
      if (with_comment)
	sources << "# Source coordinates (meters)" << endl;
      sources << "Abscissa: " << source_data[i]["X"] << endl;
      sources << "Ordinate: " << source_data[i]["Y"] << endl;
      sources << "Altitude: " << source_data[i]["Z"] << endl;
      if (with_comment)
	sources << "# Species name" << endl;
      sources << "Species_name: " << species[i] << endl;

      if (source_type == "puff")
	{
	  if (with_comment)
	    sources << endl << "# Release time (seconds)" << endl;
	  sources << "Release_time: " << source_data[i]["t"] << endl;
	  if (with_comment)
	    sources << "# Total mass released (mass)" << endl;
	  sources << "Quantity: " << puff_quantity[i] << endl;
	  sources << endl << endl;
	}
      else if (source_type == "continuous")
	{
	  if (with_comment)
	    sources << endl << "# Source rate (mass/s)" << endl;
	  sources << "Rate: " << source_data[i]["rate"] << endl;
	  if (with_comment)
	    sources << "# Source velocity (m/s)" << endl;
	  sources << "Velocity: " << source_data[i]["velocity"] << endl;
	  if (with_comment)
	    sources << "# Source temperature (Celsius degree)" << endl;
	  sources << "Temperature: " << source_data[i]["temperature"] << endl;
	  if (with_comment)
	    sources << "# Source section (m^2)" << endl;
	  sources << "Section: " << source_data[i]["section"] << endl;
	  sources << endl << endl;
	}
    }
  sources.close();

  cout << " done." << endl << endl;

  END;

  return 0;
}
