// Copyright (C) 2003-2005 CEREA
//     Authors: Vivien Mallet and Herv Njomgang
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


// This program generates LUC data for Polair3D based on USGS data.


//////////////
// INCLUDES //

#include <cmath>
#include <iostream>
using namespace std;

#define SELDONDATA_DEBUG_LEVEL_4

#include "AtmoData.hxx"
using namespace AtmoData;

// INCLUDES //
//////////////


int main(int argc, char **argv)
{
  
  TRY;

  cout << endl;

  string configuration_file(""), sec_config_file("");

  if (argc != 2 && argc != 3)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [secondary configuration file]\n";
      mesg += string("  ") + argv[0] + " [main configuration file]\n";
      mesg += "Arguments:\n";
      mesg += "  [main configuration file]: main configuration file.\n";
      mesg += "  [secondary configuration file] (optional): secondary configuration file.\n";
      cout << mesg << endl;
      return 1;
    }

  if (argc == 3)
    sec_config_file = argv[2];
  if (argc == 2 || argc == 3)
    configuration_file = argv[1];

  if (!exists(configuration_file))
    throw string("Unable to find configuration file \"")
      + configuration_file + "\".";


  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////

  typedef float real;

  // Units: SI.
  real lon_origin, lat_origin, lon_origin_af, lat_origin_af;
  real lon_upper_left, lat_upper_left, lon_upper_left_af, lat_upper_left_af;

  // Pixel size (LUC, USGS): 1000 meters.
  real step;

  int i, j, k, l;

  // Input files.
  string dir_in, file_in, file_in_af;

  // Output files.
  string dir_out, file_out;

  // Input dimensions.
  int Nx_in_LUC, Ny_in_LUC, Nx_in_af_LUC, Ny_in_af_LUC;

  // Number of LUC categories.
  int Nc;

  // Output dimensions.
  int Nx_out, Ny_out;
  real Delta_y_out, Delta_x_out, x_min_out, y_min_out;


  /////////////////////////
  // CONFIGURATION FILES //
  /////////////////////////

  cout << "Reading configuration files..."; cout.flush();
 
  ConfigStreams config(configuration_file);
  if (exists(sec_config_file))
    config.AddFile(sec_config_file);

  config.FindFromBeginning("[paths]");

  config.PeekValue("Database_luc-usgs", dir_in);
  config.PeekValue("LUC_in_ea", file_in);
  config.PeekValue("LUC_in_af", file_in_af);
  config.PeekValue("Directory_luc-usgs", dir_out);
  config.PeekValue("LUC_out", file_out);

  config.FindFromBeginning("[USGS]");

  config.PeekValue("lon_origin_ea", lon_origin);
  config.PeekValue("lat_origin_ea", lat_origin);
  config.PeekValue("lon_origin_af", lon_origin_af);
  config.PeekValue("lat_origin_af", lat_origin_af);
  config.PeekValue("lon_upper_left_ea", lon_upper_left);
  config.PeekValue("lat_upper_left_ea", lat_upper_left);
  config.PeekValue("lon_upper_left_af", lon_upper_left_af);
  config.PeekValue("lat_upper_left_af", lat_upper_left_af);
  config.PeekValue("Step", step);
  config.PeekValue("Nx_ea", Nx_in_LUC);
  config.PeekValue("Ny_ea", Ny_in_LUC);
  config.PeekValue("Nx_af", Nx_in_af_LUC);
  config.PeekValue("Ny_af", Ny_in_af_LUC);
  config.PeekValue("Nc", Nc);

  int sea_index;
  config.PeekValue("Sea_index", sea_index);

  config.FindFromBeginning("[domain]");

  config.PeekValue("Nx", Nx_out);
  config.PeekValue("Ny", Ny_out);
  config.PeekValue("Delta_x", Delta_x_out);
  config.PeekValue("Delta_y", Delta_y_out);
  config.PeekValue("x_min", x_min_out);
  config.PeekValue("y_min", y_min_out);

  cout << " done." << endl;


  /////////////////
  // ALLOCATIONS //
  /////////////////

  cout << "Memory allocation for data fields..."; cout.flush();

  // Input settings.

  // Input grids.
  RegularGrid<int> GridX_in_LUC(Nx_in_LUC), GridY_in_LUC(Ny_in_LUC),
    GridX_in_af_LUC(Nx_in_af_LUC), GridY_in_af_LUC(Ny_in_af_LUC);
  // Input LUC data.
  Data<char, 2, int> LUC_in(GridX_in_LUC, GridY_in_LUC),
    LUC_in_af(GridX_in_af_LUC, GridY_in_af_LUC);

  // Output settings.

  // Output grids.
  RegularGrid<real> GridY_out(y_min_out, Delta_y_out, Ny_out);
  RegularGrid<real> GridX_out(x_min_out, Delta_x_out, Nx_out);
  RegularGrid<real> GridC(Nc);
  // Output data.
  Data<float, 3, real> LUC_out(GridC, GridY_out, GridX_out);
  Data<float, 3, real> LUC_out_af(GridC, GridY_out, GridX_out);
  Data<float, 2, real> NbCells(GridY_out, GridX_out);
  Data<float, 2, real> NbCells_af(GridY_out, GridX_out);

  cout << " done." << endl;
  cout << endl;


  /////////
  // LUC //
  /////////

  cout << "Reading LUC data..."; cout.flush();
  FormatBinary<char> FormatIn, FormatIn_af;
  FormatIn.Read(dir_in + "/" + file_in, LUC_in);
  FormatIn_af.Read(dir_in + "/" + file_in_af, LUC_in_af);
  cout << " done." << endl;

  // Convertion from Lambert azimuthal equal area to latitude/longitude.
  LaeaToLonlat<real> Convertion(lon_origin, lat_origin);
  LaeaToLonlat<real> Convertion_af(lon_origin_af, lat_origin_af);

  real x, y, lon, lat;

  cout << "Building LUC data on output grid..."; cout.flush();

  LUC_out.SetZero();
  NbCells.SetZero();

  for (i = 0; i < Nx_in_LUC; i++)
    for (j = 0; j < Ny_in_LUC; j++)
      if (int(LUC_in(i, j)) != 0)
	{
	  x = j * step + lon_upper_left;
	  y = -i * step + lat_upper_left;
	  Convertion(x, y, lon, lat);
	  k = int((lon - x_min_out + Delta_x_out / 2.0) / Delta_x_out);
	  l = int((lat - y_min_out + Delta_y_out / 2.0) / Delta_y_out);
	  if (k >= 0 && k < Nx_out && l >= 0 && l < Ny_out)
	    {
	      LUC_out(int(LUC_in(i, j)) - 1, l, k)++;
	      NbCells(l, k)++;
	    }
	}

  for (i = 0; i < Nx_out; i++)
    for (j = 0; j < Ny_out; j++)
      for (k = 0; k < Nc; k++)
	if (NbCells(j, i) != 0.)
	  LUC_out(k, j, i) /= NbCells(j, i);

  LUC_out_af.SetZero();
  NbCells_af.SetZero();

  for (i = 0; i < Nx_in_af_LUC; i++)
    for (j = 0; j < Ny_in_af_LUC; j++)
      if (int(LUC_in_af(i, j)) != 0 && int(LUC_in_af(i, j)) < 25)
	{
	  x = j * step + lon_upper_left_af;
	  y = -i * step + lat_upper_left_af;
	  Convertion_af(x, y, lon, lat);
	  k = int((lon - x_min_out + Delta_x_out / 2.0) / Delta_x_out);
	  l = int((lat - y_min_out + Delta_y_out / 2.0) / Delta_y_out);
	  if (k >= 0 && k < Nx_out && l >= 0 && l < Ny_out)
	    {
	      LUC_out_af(int(LUC_in_af(i, j)) - 1, l, k)++;
	      NbCells_af(l, k)++;
	    }
	}

  for (i = 0; i < Nx_out; i++)
    for (j = 0; j < Ny_out; j++)
      for (k = 0; k < Nc; k++)
	if (NbCells_af(j, i) != 0.)
	  LUC_out_af(k, j, i) /= NbCells_af(j, i);

  // Merges the two LUC arrays.
  for (i = 0; i < Nx_out; i++)
    for (j = 0; j < Ny_out; j++)
      if ( (NbCells(j, i) == 0. || LUC_out(sea_index, j, i) == 1.)
	   && NbCells_af(j, i) != 0.)
	for (k = 0; k < Nc; k++)
	  LUC_out(k, j, i) = LUC_out_af(k, j, i);

  cout << " done." << endl;
  cout << endl;


  ////////////////////
  // WRITING OUTPUT //
  ////////////////////

  cout << "Writing output data..."; cout.flush();

  FormatBinary<float> PolairFormat;
  PolairFormat.Write(LUC_out, dir_out + "/" + file_out);

  cout << " done." << endl;


  END;

  return 0;

}
